Package: fritools2
Title: Utilities for the Forest Research Institute of the State
        Baden-Wuerttemberg
Version: 4.1.0
Authors@R: 
    person(given = "Andreas Dominik",
           family = "Cullmann",
           role = c("aut", "cre"),
           email = "fvafrcu@mailbox.org")
Description: Miscellaneous utilities, tools and helper
    functions for finding and searching files on disk, searching for and
    removing R objects from the workspace.
    Does not import or depend on any third party package, but on core R 
    only (i.e. it may depend on packages with priority 'base').
License: BSD_2_clause + file LICENSE
URL: https://gitlab.com/fvafrcu/fritools
Depends: R (>= 3.3.0)
Imports: methods, stats, utils
Suggests: callr, checkmate, covr, desc, devtools, digest, dplyr,
        pkgload, reshape, rmarkdown, RUnit, testthat (>= 3.0.0),
        tinytest, whoami
Encoding: UTF-8
Language: en-US
RoxygenNote: 7.2.3
VignetteBuilder: utils
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2023-06-07 12:11:57 UTC; qwer
Author: Andreas Dominik Cullmann [aut, cre]
Maintainer: Andreas Dominik Cullmann <fvafrcu@mailbox.org>
Repository: CRAN
Date/Publication: 2023-06-09 14:00:17 UTC
Built: R 4.6.0; ; 2026-04-26 18:33:44 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(convert_umlauts_to_ascii,character)
S3method(convert_umlauts_to_ascii,data.frame)
S3method(summary,filesearch)
S3method(weighted_variance,data.frame)
S3method(weighted_variance,numeric)
export(bulk_read_csv)
export(bulk_write_csv)
export(call_conditionally)
export(call_safe)
export(check_ascii_file)
export(clipboard_path)
export(column_sums)
export(compare_vectors)
export(convert_umlauts_to_ascii)
export(convert_umlauts_to_tex)
export(count_groups)
export(csv2csv)
export(delete_trailing_blank_lines)
export(delete_trailing_whitespace)
export(develop_test)
export(file_copy)
export(file_modified_last)
export(file_save)
export(find_files)
export(find_missing_family)
export(find_missing_see_also)
export(fromto)
export(get_boolean_envvar)
export(get_lines_between_tags)
export(get_mtime)
export(get_options)
export(get_package_version)
export(get_path)
export(get_r_cmd_batch_script_path)
export(get_rscript_script_path)
export(get_run_r_tests)
export(get_script_name)
export(get_script_path)
export(get_unique_string)
export(golden_ratio)
export(grep_file)
export(index_groups)
export(is_batch)
export(is_cran)
export(is_difftime_less)
export(is_false)
export(is_files_current)
export(is_force)
export(is_installed)
export(is_not_false)
export(is_null_or_true)
export(is_of_length_zero)
export(is_path)
export(is_r_cmd_check)
export(is_r_package_installed)
export(is_running_on_fvafrcu_machines)
export(is_running_on_gitlab_com)
export(is_success)
export(is_valid_primary_key)
export(is_version_sufficient)
export(is_windows)
export(load_internal_functions)
export(memory_hogs)
export(r_cmd_install)
export(read_csv)
export(relative_difference)
export(round_commercially)
export(round_half_away_from_zero)
export(run_r_tests_for_known_hosts)
export(search_files)
export(search_rows)
export(set_hash)
export(set_options)
export(set_path)
export(set_run_r_tests)
export(split_code_file)
export(str2num)
export(strip_off_attributes)
export(subset_sizes)
export(tapply)
export(touch)
export(touch2)
export(un_hash)
export(view)
export(vim)
export(weighted_variance)
export(wipe_clean)
export(wipe_tempdir)
export(with_dir)
export(write_csv)

[![pipeline status](https://gitlab.com/fvafrcu/fritools/badges/master/pipeline.svg)](https://gitlab.com/fvafrcu/fritools/-/commits/master)    
[![coverage report](https://gitlab.com/fvafrcu/fritools/badges/master/coverage.svg)](https://gitlab.com/fvafrcu/fritools/-/commits/master)
<!-- 
    [![Build Status](https://travis-ci.org/fvafrcu/fritools2.svg?branch=master)](https://travis-ci.org/fvafrcu/fritools2)
    [![Coverage Status](https://codecov.io/github/fvafrcu/fritools2/coverage.svg?branch=master)](https://codecov.io/github/fvafrcu/fritools2?branch=master)
-->
[![CRAN_Status_Badge](https://www.r-pkg.org/badges/version/fritools2)](https://cran.r-project.org/package=fritools2)
[![RStudio_downloads_monthly](https://cranlogs.r-pkg.org/badges/fritools2)](https://cran.r-project.org/package=fritools2)
[![RStudio_downloads_total](https://cranlogs.r-pkg.org/badges/grand-total/fritools2)](https://cran.r-project.org/package=fritools2)

<!-- README.md is generated from README.Rmd. Please edit that file -->



# fritools2
## Introduction

After installation, please read the help page:

```r
help("fritools2-package", package = "fritools2")
```


```
#> Utilities for the Forest Research Institute of the State
#> Baden-Wuerttemberg
#> 
#> Description:
#> 
#>      Miscellaneous utilities, tools and helper functions.
#> 
#> Details:
#> 
#>      You will find the details in
#>      'vignette("An_Introduction_to_fritools", package = "fritools2")'.
```

## Installation

You can install fritools2 from gitlab via:


```r
if (! require("remotes")) install.packages("remotes")
remotes::install_gitlab("fvafrcu/fritools2")
```

#' Split a Code File Into Multiple Files
#'
#' I tend to find files with dozens of functions. They don't read well.
#' So I split a code file into multiple files each containing a single function.
#' @param file The code file to be split.
#' @param output_directory Where to create the new files.
#' @param encoding The encoding passed to \code{\link{source}}.
#' @param write_to_disk Set the output_directory to
#' \code{dirname(file)}? Just a shortcut.
#' @export
#' @return \code{\link[base:invisible]{Invisibly}} a vector of paths to the new
#' files.
#' @family file utilities
#' @examples
#' infile <- system.file("files", "test_helpers.R", package = "fritools2")
#' \dontrun{
#'    file.show(infile)
#' }
#' paths <- split_code_file(file = infile)
#' \dontrun{
#'    file.show(paths[2])
#' }
split_code_file <- function(file, output_directory = tempdir(),
                            encoding = getOption("encoding"),
                            write_to_disk = getOption("write_to_disk")) {
    status <- FALSE
    if (isTRUE(write_to_disk)) {
        output_directory <- dirname(file)
    } else {
        dir.create(output_directory, recursive = TRUE, showWarnings = FALSE)
    }
    e <- new.env()
    source(file, local = e, echo = FALSE, keep.source = TRUE,
           encoding = encoding)
    content <- readLines(file, encoding = encoding)
    funs <- NULL
    names <- ls(e, all.names = TRUE)
    for (name in names) {
        if (is.function(e[[name]])) {
            funs <- rbind(funs,
                          position_in_content(content = content, name = name,
                                              envir = e))
        }
    }
    if (! 1 %in% funs[["start_index"]]) {
        msg <- paste0("There is a header at the top of file ", file, "!")
        warning(msg)
    }
    function_files <- NULL
    for (i in seq_len(nrow(funs))) {
        function_file <- file.path(output_directory,
                                   paste0(funs[i, "name"], ".R"))
        writeLines(content[funs[i, "start_index"]:funs[i, "stop_index"]],
                   function_file)
        function_files <- c(function_files, function_file)
    }
    if (isTRUE(write_to_disk)) {
        unlink(file)
    }
    status <- function_files
    return(invisible(status))
}

position_in_content <- function(name, content, envir = environment()) {
    index <- grep(name, content)
    comment_index <- grep(" *#", content)
    if (length(index) > 1) {
        index <- grep(paste0("^", name), content)
        if (length(index) > 1) {
            index <- grep(paste0("^", name, " *<-"), content)
        }
        if (length(index) > 1) {
            stop(paste("Found lines", content[index]))
        }
    }
    if (length(comment_index) > 0) {
        ci <- comment_index - index
        ci <- rev(-ci[ci < 0])
        ref <- seq(along = ci)
        intercept <- sum(ci == ref)
        start_index <- index - intercept
    } else {
        start_index <- index
    }
    body <- utils::capture.output(envir[[name]])
    body <- body[!grepl("^<environment:", body)]
    is_inline_func_def <- grepl("function\\(", content[index])
    stop_index <- index + length(body) - as.numeric(is_inline_func_def)
    result <- data.frame(name, start_index, index, stop_index)
    return(result)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


infile <- tempfile()
file.copy(
  system.file("files", "test_helpers.R", package = "fritools2"),
  infile
)
result <- basename(split_code_file(file = infile, write_to_disk = TRUE))
expectation <- c(
  "get_boolean_envvar.R",
  "get_run_r_tests.R",
  "is_running_on_fvafrcu_machines.R",
  "is_running_on_gitlab_com.R",
  "run_r_tests_for_known_hosts.R",
  "set_run_r_tests.R"
)
expect_identical(result, expectation)

### R code from vignette source '/home/qwer/git/cyclops/fvafrcu/fritools/vignettes/Not_an_Introduction_to_fritools2.Rnw'

###################################################
### code chunk number 1: missings
###################################################
path <- system.file(package = "fritools2")
if (file.exists(file.path(path, "source")))
    path <- file.path(path, "source")
fmsa <- fritools2::find_missing_see_also
f <- capture.output(type = "message",
                    missing <- suppressWarnings(fmsa(path = path,
                                                     list_families = TRUE)))
print(f)
if (length(missing) > 1) {
    print(missing)
    stop("Functions without context.")
} else {
    print("All functions with context.")
}

#' Is an Object of Length Zero?
#'
#' Some expressions evaluate to \code{\link{integer}(0)} or the like.
#' @param x The object.
#' @param class An optional character vector of length 1 giving the class. See
#' \emph{examples}.
#' @template return_boolean
#' @family logical helpers
#' @export
#' @examples
#' x <- ""; length(x); is_of_length_zero(x)
#' x <- grep(" ", "")
#' print(x)
#' is_of_length_zero(x)
#' is_of_length_zero(x, "character")
#' is_of_length_zero(x, "numeric")
#' is_of_length_zero(x, "integer")
is_of_length_zero <- function(x, class = NULL) {
    if (is.null(class))
        result <- identical(length(x), 0L)
    else
        result <- identical(length(x), 0L) && methods::is(x, class)
    return(result)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


x <- grep(" ", "")
expect_true(is_of_length_zero(x))
expect_true(!is_of_length_zero(x, "character"))
expect_true(is_of_length_zero(x, "numeric"))
expect_true(is_of_length_zero(x, "integer"))

#' Extract All Items of a Vector Between Two Patterns
#'
#' This comes in handy to cut lines from a file read by \code{\link{readLines}}.
#' @param x A vector.
#' @param from A pattern, use NA to start with the first item.
#' @param to Another pattern, use NA to stop with the last item.
#' @param from_i If the from pattern matches multiple times, which one is to be
#' used.
#' @param to_i Analogously to to_i.
#' @param shift_from The number of items to shift from the item selected via
#' from and from_i.
#' @param shift_to Analogously to shift_from.
#' @param remove_empty_item Remove empty items?
#' @return The extracted vector.
#' @family searching functions
#' @export
#' @examples
#' foo <- c("First", "f1", "A", "f2", rep("B", 4), "t1", "f3", "C", "t2",
#'          rep("D", 4), "t3", "Last")
#' fromto(foo, "^f", "^t")
#' fromto(foo, NA, "^t")
#' fromto(foo, "^f", NA)
#' fromto(foo, "^f", "^t", from_i = 2)
#' fromto(foo, "^f", "^t", from_i = 2, to_i = 2)
#' fromto(foo, "^f", "^t", from_i = 2, to_i = 2, shift_from = 1, shift_to = -1)
#' fromto(foo, "^f", "^t", from_i = 2, to_i = 2, shift_from = -1, shift_to = 2)
fromto <- function(x, from, to, from_i = 1, to_i = 1,
                   shift_from = 0, shift_to = 0,
                   remove_empty_item = TRUE) {
    if (is.na(from)) {
        i <- 1
    } else {
        i <- grep(from, x)
        if (is_of_length_zero(i)) throw(paste0("Pattern `", from,
                                               "` not found in x."))
        i <- i[from_i]
    }
    if (is.na(to)) {
        j <- length(x)
    } else {
        j <- grep(to, x)
        if (is_of_length_zero(j)) throw(paste0("Pattern `", to,
                                               "` not found in x."))
        j <- j[j > i][to_i]
    }
    result <- x[(i + shift_from):(j + shift_to)]
    if (isTRUE(remove_empty_item)) result <- result[result != ""]
    return(result)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


foo <- c(
  "f1", "A", "f2", rep("B", 4), "t1", "f3", "C", "t2",
  rep("D", 4), "t3"
)
result <- fromto(foo, "^f", "^t")
expectation <- c("f1", "A", "f2", "B", "B", "B", "B", "t1")
expect_identical(result, expectation)

result <- fromto(foo, "^f", "^t", from_i = 2)
expectation <- c("f2", "B", "B", "B", "B", "t1")
expect_identical(result, expectation)

result <- fromto(foo, "^f", "^t", from_i = 2, to_i = 2)
expectation <- c("f2", "B", "B", "B", "B", "t1", "f3", "C", "t2")
expect_identical(result, expectation)

result <- fromto(foo, "^f", "^t",
  from_i = 2, to_i = 2, shift_from = 1,
  shift_to = -1
)
expectation <- c("B", "B", "B", "B", "t1", "f3", "C")
expect_identical(result, expectation)

result <- fromto(foo, "^f", "^t",
  from_i = 2, to_i = 2, shift_from = -1,
  shift_to = 2
)
expectation <- c(
  "A", "f2", "B", "B", "B", "B", "t1", "f3",
  "C", "t2", "D", "D"
)
expect_identical(result, expectation)
# give NA
foo <- c(
  "f1", "A", "f2", rep("B", 4), "t1", "f3", "C", "t2",
  rep("D", 4), "t3"
)
result <- fromto(foo, "^A", "^B", to_i = 3)
expectation <- c("A", "f2", "B", "B", "B")
expect_identical(result, expectation)

result <- fromto(foo, NA, "^B", to_i = 3)
expectation <- c("f1", "A", "f2", "B", "B", "B")
expect_identical(result, expectation)

result <- fromto(foo, NA, NA, to_i = 3)
expectation <- foo
expect_identical(result, expectation)

expect_error(fromto(x = foo, from = "^Not_there", to = "B"))
expect_error(fromto(x = foo, from = NA, to = "^Not_there"))

#' Provide \code{isFALSE} for R < 3.5.0
#'
#' I still use R 3.3.3 for testing, \code{isFALSE()} was introduced in R 3.5.0.
#'
#' @param x The object to be tested.
#' @return \code{\link[base:TRUE]{TRUE}} if the object is set to
#' \code{\link[base:FALSE]{FALSE}},
#' \code{\link[base:FALSE]{FALSE}}
#' otherwise.
#' @export
#' @family logical helpers
#' @examples
#' is_false("not false")
#' is_false(FALSE)
is_false <- function(x) {
    if (exists("isFALSE", where = "package:base", mode = "function")) {
        res <- base::isFALSE(x)
    } else {
        res <- is.logical(x) && length(x) == 1L && !is.na(x) && !x
    }
    return(res)
}


#' Is an Object \code{\link[base:TRUE]{TRUE}} or \code{\link[base:NULL]{NULL}}?
#'
#' Is an object \code{\link[base:TRUE]{TRUE}} or \code{\link[base:NULL]{NULL}}?
#' @param x The object to be tested.
#' @return \code{\link[base:TRUE]{TRUE}} if the object is set to
#' \code{\link[base:TRUE]{TRUE}} or \code{\link[base:NULL]{NULL}},
#' \code{\link[base:FALSE]{FALSE}}
#' otherwise.
#' @export
#' @family logical helpers
#' @examples
#' is_null_or_true("true") # FALSE
#' is_null_or_true(TRUE) # TRUE
#' is_null_or_true(NULL) # TRUE
#' suppressWarnings(rm("not_defined"))
#' try(is_null_or_true(not_defined)) # error
is_null_or_true <- function(x) return(isTRUE(x) || is.null(x))

#' Is an Object Set and not Set to \code{\link[base:FALSE]{FALSE}}?
#'
#' Sometimes you need to know whether or not an object exists and is not set to
#' \code{\link[base:FALSE]{FALSE}} (and possibly not
#' \code{\link[base:NULL]{NULL}}).
#'
#' @param x The object to be tested.
#' @param null_is_false Should \code{\link[base:NULL]{NULL}} be treated as
#' \code{\link[base:FALSE]{FALSE}}?
#' @param ... Parameters passed to \code{\link{exists}}. See Examples.
#' @return \code{\link[base:TRUE]{TRUE}} if the object is set to something
#' different than \code{\link[base:FALSE]{FALSE}},
#' \code{\link[base:FALSE]{FALSE}}
#' otherwise.
#' @export
#' @family logical helpers
#' @examples
#' a  <- 1
#' b  <- FALSE
#' c  <- NULL
#' is_not_false(a)
#' is_not_false(b)
#' is_not_false(c)
#' is_not_false(c, null_is_false = FALSE)
#' is_not_false(not_defined)
#' f <- function() {
#'     print(a)
#'     print(is_not_false(a))
#' }
#' f()
#'
#' f <- function() {
#'     a <- FALSE
#'     print(a)
#'     print(is_not_false(a))
#' }
#' f()
#'
#' f <- function() {
#'     print(a)
#'     print(is_not_false(a, null_is_false = TRUE,
#'                        inherits = FALSE))
#' }
#' f()
#' ### We use this to check whether an option is set to something
#' ### different than FALSE:
#' # Make sure an option is not set:
#' set_options("test" = NULL, package = "fritools2")
#' tmp <- get_options("test")
#' is_not_false(tmp)
#' is_not_false(tmp, null_is_false = FALSE)
#' # Does not work on the option directly as it is not an object defined:
#' options("foo" = NULL)
#' is_not_false(getOption("foo"), null_is_false = FALSE)
is_not_false <- function(x, null_is_false = TRUE, ...) {
    if (requireNamespace("checkmate", quietly = TRUE))
        checkmate::qassert(null_is_false, "B1")
    condition <- exists(deparse(substitute(x)), ...)
    if (isTRUE(null_is_false)) {
        condition <- condition && ! is.null(x) && x != FALSE
    } else {
        condition <- condition && (is.null(x) || x != FALSE)
    }
    if (condition)
        result <- TRUE
    else
        result <- FALSE
    return(result)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


f <- function() {
  a <- FALSE
  return(is_not_false(a))
}
result <- f()
expectation <- FALSE
expect_identical(result, expectation)

a <- NULL
result <- is_not_false(a)
expectation <- FALSE
expect_identical(result, expectation)

if (FALSE) {
    result <- is_not_false(a, null_is_false = FALSE)
    expectation <- TRUE
    # TODO: does not work in batch?!
    if (interactive())
        expect_identical(result, expectation)

    a  <- "not_false"
    # finds a in parent.frame()
    f <- function() {
        return(is_not_false(a))
    }
    result <- f()
    expectation <- TRUE
    # TODO: does not work in batch?!
    if (interactive())
        expect_identical(result, expectation)
}
# suppress search in parent.frame()
f <- function() {
  return(is_not_false(a,
    null_is_false = TRUE,
    inherits = FALSE
  ))
}
result <- f()
expectation <- FALSE
expect_identical(result, expectation)






expectation <- TRUE
result <- is_false(FALSE)
expect_identical(result, expectation)






expectation <- TRUE
result <- is_null_or_true(TRUE)
result <- is_null_or_true(NULL)
expect_identical(result, expectation)

expectation <- FALSE
result <- is_null_or_true(FALSE)
expect_identical(result, expectation)
result <- is_null_or_true("not true")
expect_identical(result, expectation)

#' Is the Current R Process an \command{R CMD check}?
#'
#' Check for system variables to guess whether or not this is an
#' \command{R CMD check}.
#' @export
#' @family logical helpers
#' @family test helpers
#' @template return_boolean
is_r_cmd_check <- function() {
    is_r_cmd_check <- ("CheckExEnv" %in% search()) ||
        any(c("_R_CHECK_TIMINGS_", "_R_CHECK_LICENSE_") %in%
            names(Sys.getenv()))
    return(is_r_cmd_check)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


result <- is_cran()
if (is_running_on_fvafrcu_machines()) {
  if (fritools2::is_r_cmd_check()) {
    expect_true(strip_off_attributes(result))
  } else {
    expect_true(strip_off_attributes(!result))
  }
}

#' Round Half Away From Zero
#'
#' Commercial rounding is done a lot, especially with invoices. There is even
#' standard 1333 by the German Institute for Standardization.
#' \code{\link{round}} rounds half to even, see \code{\link{round}}'s Details
#' section.
#' @param x A number to be rounded.
#' @param digits The number of digits, as in \code{\link{round}}.
#' @return The rounded number.
#' @export
#' @family statistics
#' @examples
#' x <- 22.5
#' round_half_away_from_zero(x)
#' round(x)
#' round_half_away_from_zero(-x)
#' round(-x)
round_half_away_from_zero <- function(x, digits = 0) {
    res <- sign(x) * trunc(abs(x) * 10^digits + 0.5) / 10^digits
    return(res)
}

#' @description
#' \code{round_commercially} is just a link to \code{round_half_away_from_zero}.
#' @export
#' @rdname round_half_away_from_zero
#' @aliases round_commercially
round_commercially <- round_half_away_from_zero

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


x <- 22.5
expect_equal(round_half_away_from_zero(x), 23)
expect_equal(base::round(x), 22)
expect_equal(round_half_away_from_zero(-x), -23)
expect_equal(base::round(-x), -22)

#' Get a Boolean Environment Variable
#'
#' A convenience wrapper to \code{\link{Sys.getenv}}.
#'
#' As \code{\link{Sys.getenv}} seems to always return a character vector, the
#' \code{\link{class}} of the value you set it to does not matter.
#' @param x The name of the Environment Variable.
#' @param stop_on_failure Throw an error instead of returning
#' \code{\link{FALSE}} if the environment variable is not set or cannot be
#' converted to boolean.
#' @family test helpers
#' @family operating system functions
#' @return The value the environment variable is set to, converted to boolean.
#' \code{\link{FALSE}} if the environment variable is not set or cannot be
#' converted to boolean. But see \strong{Arguments}: \emph{stop_on_failure}.
#' @export
#' @examples
#' message("See\n example(\"get_run_r_tests\", package = \"fritools2\")")
get_boolean_envvar <- function(x, stop_on_failure = FALSE) {
    r <- Sys.getenv(x)
    if (identical(r, "")) {
        if (isTRUE(stop_on_failure)) {
            throw(paste("Environment variable", x, "is not set."))
        } else {
            r <- FALSE
        }
    } else {
        # Sys.getenv seems to always return a character vector.
        # So we first try to convert to numeric to take care of "0" and "1".
        # But this will crash for strings like "TRUE" and "FALSE", so we need to
        # try only.
        r <- tryCatch(as.numeric(r), warning = function(w) return(r))
        r <- as.logical(r)
        if (is.na(r)) {
            if (isTRUE(stop_on_failure)) {
                throw(paste("Environment variable", x, "is set to",
                            "a value not interpretable as boolean."))
            } else {
                r <- FALSE
            }
        } else {
            ## do nothing: r is set and not NA.
        }
    }
    return(r)
}

#' Is the Machine Running the Current \R Process Owned by FVAFRCU?
#'
#' Is the machine running the current \R process known to me?
#' @template return_boolean
#' @param type An optional selection.
#' @family test helpers
#' @family logical helpers
#' @export
#' @examples
#' is_running_on_fvafrcu_machines()
is_running_on_fvafrcu_machines <- function(type = c("any", "cu", "fvafr")) {
    sys_info <- Sys.info()
    h <- sys_info[["nodename"]] %in% c("h6", "h7") &&
        .Platform[["OS.type"]] == "unix" &&
        sys_info[["effective_user"]] == "qwer"
    v <- grepl("^fvafr.*CU.*$", sys_info[["nodename"]]) &&
        .Platform[["OS.type"]] == "unix" &&
        sys_info[["effective_user"]] %in%
        c("dominik.cullmann", "dominik", "nik")
    w <- (grepl("^FVAFR-PC.*$", sys_info[["nodename"]])
          && sys_info[["effective_user"]] == "dominik.cullmann" ||
          grepl("^L-FVAFR-NB84223$", sys_info[["nodename"]])
      ) && is_windows()

    r <- switch(match.arg(type),
                "cu" = h,
                "fvafr" = v || w,
                "any" = ,
                h || v || w)
    return(r)
}

#' Is the Current Machine Owned by \url{https://about.gitlab.com}?
#'
#' Check whether the current machine is located on
#' \url{https://about.gitlab.com}.
#' This check is an approximation only.
#' @param verbose Be verbose?
#' @template return_boolean
#' @family logical helpers
#' @family test helpers
#' @export
#' @examples
#' is_running_on_gitlab_com()
is_running_on_gitlab_com <- function(verbose = TRUE) {
    gitlab_pattern <- paste0("^runner-.*-project-.*-",
                             "concurrent-.*$")
    r <-  grepl(gitlab_pattern, Sys.info()[["nodename"]]) &&
        .Platform[["OS.type"]] == "unix"
    if (isTRUE(verbose) && !r) {
        msg <- paste(Sys.info()[["nodename"]], .Platform[["OS.type"]])
        attr(r, "message") <- msg
        message(msg)
    }
    return(r)
}

#' Set the System Variable RUN_R_TESTS
#'
#' A convenience wrapper to  \code{\link{Sys.getenv}} for setting
#' \kbd{RUN_R_TESTS}.
#' @param x A logical, typically some function output.
#' @param force Overwrite the variable if already set?
#' @return The value RUN_R_TESTS is set to, \code{\link{NULL}} if nothing is
#' done.
#' @family test helpers
#' @export
#' @examples
#' set_run_r_tests(is_running_on_fvafrcu_machines())
#' get_run_r_tests()
#' set_run_r_tests(TRUE, force = TRUE)
#' get_run_r_tests()
set_run_r_tests  <- function(x, force = FALSE) {
    r <- NULL
    is_unset <- identical(Sys.getenv("RUN_R_TESTS", unset = ""), "")
    if (is_unset || isTRUE(force)) {
        r <- x
        Sys.setenv("RUN_R_TESTS" = r)
    }
    return(invisible(r))
}

#' Get System Variable RUN_R_TESTS
#'
#' A convenience wrapper to
#' \code{\link{get_boolean_envvar}("RUN_R_TESTS")}.
#'
#' @inheritParams get_boolean_envvar
#' @family test helpers
#' @family operating system functions
#' @family logical helpers
#' @return The value RUN_R_TESTS is set to, converted to boolean.
#' \code{\link{FALSE}} if RUN_R_TESTS is not set or cannot be converted to
#' boolean.
#' @export
#' @examples
#' set_run_r_tests("", force = TRUE) # make sure it is not set.
#' get_run_r_tests()
#' try(get_run_r_tests(stop_on_failure = TRUE))
#' set_run_r_tests("A", force = TRUE) # "A" is not boolean.
#' get_run_r_tests()
#' try(get_run_r_tests(stop_on_failure = TRUE))
#' set_run_r_tests(4213, force = TRUE) # All numbers apart from 0 are TRUE
#' get_run_r_tests()
#' set_run_r_tests("0", force = TRUE) # 0 (and "0") is FALSE
#' get_run_r_tests()
#' set_run_r_tests("FALSE", force = TRUE)
#' get_run_r_tests()
#' set_run_r_tests(TRUE, force = TRUE)
#' get_run_r_tests()
get_run_r_tests <- function(stop_on_failure = FALSE) {
    r <- get_boolean_envvar("RUN_R_TESTS", stop_on_failure = stop_on_failure)
    return(r)
}

#' Force Testing on Known Hosts
#'
#' Enforce the environment variable RUN_R_TESTS to TRUE on known hosts.
#'
#' This should go into \code{\link{.onLoad}} to force tests on known hosts.
#' @template return_invisibly_null
#' @export
#' @family test helpers
#' @inherit set_run_r_tests return
#' @examples
#' get_run_r_tests()
#' if (isFALSE(get_run_r_tests())) {
#'     run_r_tests_for_known_hosts()
#'     get_run_r_tests()
#' }
run_r_tests_for_known_hosts <- function() {
    r <- set_run_r_tests(is_running_on_fvafrcu_machines() ||
                         is_running_on_gitlab_com(verbose = FALSE),
                     force = TRUE)
    return(invisible(r))
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}

if (run_r_tests_for_known_hosts()) {
  expectation <- normalizePath(tempdir(), mustWork = FALSE)
  result <- normalizePath(with_dir(expectation, getwd()))

  expect_identical(expectation, result)
}
if (interactive()) {

}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}

if (get_run_r_tests() && !is_running_on_gitlab_com()) {
  path <- system.file("DESCRIPTION", package = "fritools2")
  deps <- desc::desc_get_deps()
  result <- !any(deps[["type"]] == "Depends" & deps[["package"]] != "R")
  msg <- "fritools2 must only depend on R itself!"
  expect_true(result, info = msg)
  if (is_running_on_fvafrcu_machines()) {
      core <- row.names(installed.packages(priority = "base"))
      dput(core)
  } else {
      core <- c("base", "compiler", "datasets", "graphics", "grDevices", "grid",
                "methods", "parallel", "splines", "stats", "stats4", "tcltk",
                "tools", "utils")
  }
  result <- !any(deps[["type"]] == "Imports" %in% core)
  msg <- paste(
    "fritools2 must not import any package!",
    "You may suggest packages and use dem conditionally as I",
    "have done with checkmate, see fritools2::is_not_false."
  )
  expect_true(result, info = msg)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}

if (get_run_r_tests() && !fritools2::is_windows()) {
  e <- new.env()
  assign("va", rep(mtcars, 1), envir = e)
  assign("vb", rep(mtcars, 1000), envir = e)
  assign("vc", rep(mtcars, 2000), envir = e)
  assign("vd", rep(mtcars, 100), envir = e)
  result <- memory_hogs(envir = e)
  expectation <- structure(c(
    va = 4424, vd = 352824, vb = 3520824,
    vc = 7040824
  ), units = "bytes")
  expect_true(all(expectation %in% result))
  result <- memory_hogs(unit = "Mb", decreasing = TRUE, envir = e)
  expectation <- structure(c(
    vc = 6.7, vb = 3.4, vd = 0.3,
    expectation = 0, result = 0, va = 0
  ),
  units = "Mb"
  )
  expect_true(all(expectation %in% result))
  expect_true(all(c("va", "vb", "vc", "vd") %in% ls(envir = e)))
  # see if we can remove some objects
  rm(
    list = names(tail(memory_hogs(decreasing = FALSE, envir = e),
      n = 2
    )),
    envir = e
  )
  expect_true(all(c("va", "vd") %in% ls(envir = e)))
  expect_true(all(!c("vb", "vc") %in% ls(envir = e)))
  # check for character output
  result <- memory_hogs(envir = e, return_numeric = FALSE)
  expectation <- c(va = "4424 bytes", vd = "352824 bytes")
  expect_true(all(expectation %in% result))
}
if (interactive()) {
  rm(list = ls(pattern = "v[a-d]"))
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


message(
  "These functions can't be tested, as the result differs on",
  "different machines"
)
expectation <- if (result <- is_running_on_fvafrcu_machines()) {
  TRUE
} else {
  FALSE
}
expect_identical(result, expectation)

## this is tested direct via .gitlab-ci/gitlab-com.R and a call to it in
## .gitlab-ci.yml
expectation <- if (result <- is_running_on_gitlab_com()) {
  TRUE
} else {
  FALSE
}
attributes(result) <- NULL

expect_identical(result, expectation)

# Well, that's ...
result <- run_r_tests_for_known_hosts()
expectation <-
  is_running_on_gitlab_com(verbose = FALSE) ||
    is_running_on_fvafrcu_machines()
expect_identical(result, expectation)






set_run_r_tests("", force = TRUE) # make sure it is not set.
# % get unset option
expect_identical(get_run_r_tests(), FALSE)
expect_error(get_run_r_tests(stop_on_failure = TRUE))

# % set option
set_run_r_tests("", force = TRUE) # make sure it is not set.
expect_identical(set_run_r_tests(12), 12)
expect_identical(set_run_r_tests("12", force = TRUE), "12")
expect_identical(set_run_r_tests(TRUE), NULL)


# % get set option
## % passing
set_run_r_tests(4213, force = TRUE) # All numbers apart from 0 are TRUE
expect_true(get_run_r_tests())
set_run_r_tests("4213", force = TRUE) # All numbers apart from 0 are TRUE
expect_true(get_run_r_tests())
set_run_r_tests(TRUE, force = TRUE)
expect_true(get_run_r_tests())
set_run_r_tests("TRUE", force = TRUE)
expect_true(get_run_r_tests())

## % failing
set_run_r_tests("A", force = TRUE) # "A" is not boolean.
expect_identical(get_run_r_tests(), FALSE)
expect_error(get_run_r_tests(stop_on_failure = TRUE))
set_run_r_tests("0", force = TRUE) # 0 (and "0") is FALSE
expect_identical(get_run_r_tests(), FALSE)
set_run_r_tests(0, force = TRUE) # 0 (and "0") is FALSE
expect_identical(get_run_r_tests(), FALSE)

set_run_r_tests("FALSE", force = TRUE)
expect_identical(get_run_r_tests(), FALSE)
set_run_r_tests(FALSE, force = TRUE)
expect_identical(get_run_r_tests(), FALSE)

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


if (is_running_on_fvafrcu_machines() || is_running_on_gitlab_com()) {
  # NOTE: There are CRAN machines where neither "R" nor "R-devel" is in
  # the path, so we skipt this test on unkown machines.
  expect_true(is_installed("R"))
}
expect_true(!is_installed("This_program_is_not_installed"))





expect_true(is_r_package_installed("fritools2", "1.1.0"))
expect_true(!is_r_package_installed("fritools2", "9999"))

#' Search All Rows Across Columns of a Matrix-like Structure
#'
#' I sometimes need to see which rows of a matrix-like structure
#' contain a string matched by a search pattern.
#' This somewhat similar to writing a matrix-like structure to disk and then
#' using \code{\link{search_files}} on it.
#' @param x A \code{\link{matrix}} or \code{\link{data.frame}}.
#' @param pattern A pattern.
#' @param include_row_names Include row names into the search?
#' @return All rows where the pattern was found in at least one column.
#' @export
#' @family searching functions
#' @examples
#' p <- "\\<4.0[[:alpha:]]*\\>"
#' search_rows(x = mtcars, pattern = p)
#' search_rows(x = mtcars, pattern = p, include_row_names = FALSE)
#' try(search_rows(x = mtcars, pattern = "ABC"))
search_rows <- function(x, pattern = ".*",
                           include_row_names = TRUE) {
    as_char <- apply(x, 2, as.character)
    if (isTRUE(include_row_names)) as_char <- cbind(rownames(x), as_char)
    i <- apply(as_char, 1, function(x) any(grepl(pattern, x)))
    if (all(!i)) {
        throw(paste0("Could not find ", pattern, " in ", deparse(substitute(x)),
                     "."))
    } else {
        return(x[i, TRUE])
    }
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}
if (fritools2::is_version_sufficient(
  fritools2::get_package_version("base"),
  "4.0.0"
)) {
  if (interactive()) pkgload::load_all(".")

  data(mtcars)
  p <- "\\<4.0[[:alpha:]]*\\>"
  expectation <- structure(list(
    mpg = c(
      16.4, 17.3, 15.2, 10.4, 14.7,
      19.2
    ),
    cyl = c(8, 8, 8, 8, 8, 8),
    disp = c(
      275.8, 275.8, 275.8, 460, 440,
      400
    ),
    hp = c(180, 180, 180, 215, 230, 175),
    drat = c(3.07, 3.07, 3.07, 3, 3.23, 3.08),
    wt = c(
      4.07, 3.73, 3.78, 5.424, 5.345,
      3.845
    ),
    qsec = c(
      17.4, 17.6, 18, 17.82, 17.42,
      17.05
    ),
    vs = c(0, 0, 0, 0, 0, 0),
    am = c(0, 0, 0, 0, 0, 0),
    gear = c(3, 3, 3, 3, 3, 3),
    carb = c(3, 3, 3, 4, 4, 2)
  ),
  row.names = c(
    "Merc 450SE", "Merc 450SL",
    "Merc 450SLC",
    "Lincoln Continental",
    "Chrysler Imperial",
    "Pontiac Firebird"
  ),
  class = "data.frame"
  )
  result <- search_rows(x = mtcars, pattern = p)
  expect_identical(result, expectation)
  expectation <- structure(list(
    mpg = c(10.4, 14.7, 19.2),
    cyl = c(8, 8, 8),
    disp = c(460, 440, 400),
    hp = c(215, 230, 175),
    drat = c(3, 3.23, 3.08),
    wt = c(5.424, 5.345, 3.845),
    qsec = c(17.82, 17.42, 17.05),
    vs = c(0, 0, 0),
    am = c(0, 0, 0),
    gear = c(3, 3, 3),
    carb = c(4, 4, 2)
  ),
  row.names = c(
    "Lincoln Continental",
    "Chrysler Imperial",
    "Pontiac Firebird"
  ),
  class = "data.frame"
  )
  result <- search_rows(
    x = mtcars, pattern = p,
    include_row_names = FALSE
  )
  expect_identical(result, expectation)
  expect_error(search_rows(x = mtcars, pattern = "ABC"))
}
if (interactive()) {

}

#' Tex Codes for German Umlauts
#'
#' Convert German umlauts in a string to their plain TeX representation.
#' @param x A string.
#' @return A string with the umlauts converted to plain TeX.
#' @family German umlaut converters
#' @export
#' @examples
#' string <- paste("this is \u00e4 string")
#' print(string)
#' print(convert_umlauts_to_tex(string))
convert_umlauts_to_tex <- function(x) {
    s <- iconv(enc2native(x), to = "UTF-8", sub = "unicode")
    s <- gsub("\u00e4", "\\\\\u0022a{}", s)
    s <- gsub("\u00c4", "\\\\\u0022A{}", s)
    s <- gsub("\u00f6", "\\\\\u0022o{}", s)
    s <- gsub("\u00d6", "\\\\\u0022O{}", s)
    s <- gsub("\u00fc", "\\\\\u0022u{}", s)
    s <- gsub("\u00dc", "\\\\\u0022U{}", s)
    s <- gsub("\u00df", "\\\\ss{}", s)
    return(s)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}
if (fritools2::is_version_sufficient(
  fritools2::get_package_version("base"),
  "4.0.0"
)) {
  if (interactive()) pkgload::load_all(".")

  string <- c(
    "\u00e4", "\u00c4", "\u00f6", "\u00d6", "\u00fc", "\u00dc",
    "\u00df"
  )
  expectation <- c(
    "\\\"a{}", "\\\"A{}", "\\\"o{}", "\\\"O{}", "\\\"u{}",
    "\\\"U{}", "\\ss{}"
  )
  result <- convert_umlauts_to_tex(string)
  expect_identical(result, expectation)
}
if (interactive()) {

}

#' Create a Fairly Unique String
#'
#' I sometimes need a fairly unique string, mostly for file names, that should
#' start with the current date.
#' @export
#' @return A fairly unique string.
#' @family file utilities
#' @examples
#' replicate(20, get_unique_string())
get_unique_string <- function() {
    name <- paste(sep = "_", strftime(Sys.time(), format = "%Y_%m_%d_%H_%M_%S"),
                  Sys.getpid(), sub("^file", "", basename(tempfile())))
    return(name)
}

.options <- function(name, value) {
    # programmatically set options
    eval(parse(text = paste("options(", name, "= value)")))
}

#' Set Options For Packages
#'
#' A convenience function for \code{\link{options}}.
#'
#' @param package_name The package's name.
#' @param overwrite [boolean(1)]\cr Overwrite options already set?
#' @param ... See \code{\link{options}}.
#' @template return_invisibly_true
#' @export
#' @family option functions
#' @examples
#' options("cleanr" = NULL)
#' defaults <- list(max_file_width = 80, max_file_length = 300,
#'                  max_lines = 65, max_lines_of_code = 50,
#'                  max_num_arguments = 5, max_nesting_depth = 3,
#'                  max_line_width = 80, check_return = TRUE)
#'
#' set_options(package_name = "cleanr", defaults)
#' getOption("cleanr")
#' set_options(package_name = "cleanr", list(max_line_width = 3,
#'             max_lines = "This is nonsense!"))
#' set_options(package_name = "cleanr", check_return = NULL, max_lines = 4000)
#' get_options(package_name = "cleanr")
set_options <- function(..., package_name = .packages()[1], overwrite = TRUE) {
    if (requireNamespace("checkmate", quietly = TRUE))
        checkmate::qassert(overwrite, "B1")
    option_list <- list(...)
    if (length(option_list) == 1L && is.list(option_list))
        option_list <- unlist(option_list, recursive = FALSE)
    options_set <- get_options(package_name = package_name,
                               flatten_list = FALSE)
    if (isTRUE(overwrite)) {
        if (is.null(options_set)) {
            .options(package_name, option_list)
        } else {
            if (length(options_set) == 1L)
                options_set <- as.list(options_set)
            .options(package_name, utils::modifyList(options_set, option_list))
        }
    } else {
        is_option_unset <- ! (names(option_list) %in% names(options_set))
        if (any(is_option_unset))
            .options(package_name,
                     append(options_set, option_list[is_option_unset]))
    }
    return(invisible(TRUE))
}

#' Get Options For Packages
#'
#' A convenience function for \code{\link{getOption}}.
#'
#' @param package_name The package's name.
#' @param ... See \code{\link{getOption}}.
#' @param remove_names [boolean(1)]\cr Remove the names?
#' @param flatten_list [boolean(1)]\cr Return a vector?
#' @return A (possibly named) list or a vector.
#' @family option functions
#' @export
#' @examples
#' example("set_options", package = "fritools2")
get_options <- function(..., package_name = .packages()[1],
                        remove_names = FALSE, flatten_list = TRUE) {
    if (requireNamespace("checkmate", quietly = TRUE)) {
        checkmate::qassert(remove_names, "B1")
        checkmate::qassert(flatten_list, "B1")
    }
    if (missing(...)) {
        option_list <- getOption(package_name)
    } else {
        option_names <- as.vector(...)
        options_set <- getOption(package_name)
        option_list  <- options_set[names(options_set) %in% option_names]
    }
    if (flatten_list) option_list <- unlist(option_list)
    if (remove_names) names(option_list)  <- NULL
    if (!is.null(option_list)) {
        attr(option_list, "package") <- package_name
    }
    return(option_list)
}

#' Opt-out Via Option
#'
#' Check whether or not a package option (set via \code{\link{set_options}})
#' \emph{force} is not set or set to \code{\link[base:TRUE]{TRUE}}.
#' @param x The option under which an element \code{"force"} is to be searched
#' for.
#' @export
#' @return \code{\link[base:TRUE]{TRUE}} if option \code{x[["force"]]} is either
#' \code{\link[base:TRUE]{TRUE}} or \code{\link[base:NULL]{NULL}} (i.e. not set
#' at all).
#' @family option functions
#' @family logical helpers
#' @export
#' @examples
#' is_force()
#' set_options(list(force = FALSE))
#' get_options(flatten_list = FALSE)
#' is_force()
is_force <- function(x = .packages()[1]) {
    res <- is_null_or_true(getOption(x)[["force"]])
    return(res)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


options("fritools2" = NULL)
expect_identical(get_options(package_name = "fritools2"), NULL)
set_options(package_name = "fritools2", max_file_width = 100)
expect_identical(
  get_options(package_name = "fritools2"),
  structure(c("max_file_width" = 100),
    package = "fritools2"
  )
)
expect_identical(
  get_options(
    package_name = "fritools2",
    flatten_list = FALSE
  ),
  structure(c("max_file_width" = 100),
    package = "fritools2"
  )
)
expect_identical(
  get_options(
    package_name = "fritools2",
    "max_file_width"
  ),
  structure(c("max_file_width" = 100),
    package = "fritools2"
  )
)
expect_identical(
  get_options(
    package_name = "fritools2",
    "max_file_width",
    flatten_list = FALSE
  ),
  structure(c("max_file_width" = 100),
    package = "fritools2"
  )
)
expect_identical(get_options(
  package_name = "fritools2",
  "max_file_width",
  remove_names = TRUE
), structure(c(100),
  package = "fritools2"
))
expect_identical(get_options(
  package_name = "fritools2",
  "max_file_width",
  remove_names = TRUE,
  flatten_list = FALSE
), structure(c(100),
  package = "fritools2"
))

set_options(package_name = "fritools2", list(
  max_lines = 30,
  max_lines_of_code = 20
))
result <- get_options(package_name = "fritools2")
expectation <- structure(c(
  max_file_width = 100, max_lines = 30,
  max_lines_of_code = 20
), package = "fritools2")
expect_identical(result, expectation)

set_options(
  package_name = "fritools2", list(
    max_lines = 0,
    max_lines_of_code = 0
  ),
  overwrite = FALSE
)
result <- get_options(package_name = "fritools2") ## didn't change anything
expect_identical(result, expectation)

set_options(
  package_name = "fritools2", list(
    max_lines = 0,
    max_lines_of_code = 0
  ),
  overwrite = TRUE
)
result <- get_options(
  package_name = "fritools2", flatten_list = FALSE,
  remove_names = TRUE
)
expectation <- structure(list(100, 0, 0), package = "fritools2")
expect_identical(result, expectation)



set_options(
  package_name = "fritools2", list(
    max_lines = 0,
    max_lines_of_code = 0,
    max_nesting_depth = 3
  ),
  overwrite = FALSE
)
result <- get_options(package_name = "fritools2")
expectation <- structure(c(
  max_file_width = 100, max_lines = 0,
  max_lines_of_code = 0,
  max_nesting_depth = 3
), package = "fritools2")
expect_identical(result, expectation)




options("fritools2" = NULL)
expectation <- TRUE
result <- is_force("fritools2")
expect_identical(result, expectation)
set_options(package_name = "fritools2", force = TRUE)
result <- is_force("fritools2")
expect_identical(result, expectation)

expectation <- FALSE
set_options(
  package_name = "fritools2", list(force = "not true"),
  overwrite = TRUE
)
result <- is_force("fritools2")
expect_identical(result, expectation)





options("fritools2" = NULL)
expectation <- structure(c(force = TRUE), package = "fritools2")
set_options(force = TRUE)
result <- get_options()
expect_identical(result, expectation)

#' Sum up the Numeric Columns of a Data Frame
#'
#' I often need to calculate the sums of the numeric columns of a
#' \code{\link{data.frame}}. While \code{\link{colSums}} requires the data frame
#' to be numeric, this is a convenience wrapper to select numeric columns only.
#' @param x A  \code{\link{data.frame}}.
#' @param ... Arguments passed to \code{\link{colSums}}.
#' @return A named vector of column sums (see \code{\link{colSums}}).
#' @family statistics
#' @export
#' @examples
#' try(colSums(iris))
#' column_sums(iris)
#' names(iris) # no column sum for `Species`
column_sums <- function(x, ...) {
    res <- colSums(x[TRUE, sapply(x, is.numeric)], ...)
    return(res)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


result <- column_sums(iris)
expectation <- c(
  Sepal.Length = 876.5, Sepal.Width = 458.6,
  Petal.Length = 563.7, Petal.Width = 179.9
)
expect_equal(result, expectation, tolerance = 1e-3)

#' Convert Character Numbers to Numeric
#'
#' If you read text containing  (possibly German, i.e. the decimals separated by
#' comma and dots inserted for what they think of as readability) numbers, you
#' may want to convert them to numeric.
#' @param x A string representing a (possibly German) number.
#' @return The number as a numeric.
#' @export
#' @family bits and pieces
#' @examples
#' line_in_text <- "foo bar 10.303,70 foo bar  1.211.000,55 foo bar"
#' words <- unlist(strsplit(line_in_text, split = " "))
#' print(na.omit(sapply(words, str2num)), digits = 9)
#' print(str2num(words[c(3, 4, 7)]), digits = 9)
#' print(str2num(words[7]), digits = 9)
str2num <- function(x) {
    res <- trimws(x)
    res <- sub(" .*", "", sub(",", ".", gsub("\\.", "", res)))
    res <- suppressWarnings(as.numeric(res))
    res <- res[!is.na(res)]
    if (is_of_length_zero(res)) res <- NA
    return(res)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


result <- str2num("1.000.000,11")
expectation <- 1e06 + 0.11
expect_equal(result, expectation)
result <- str2num("not_a_number")
expect_true(is.na(result))

#' Compute Relative Differences Between the Values of Two Vectors
#'
#' We often try to compare vectors on near equality. This is a wrapper to
#' \code{\link{all.equal}} for our convenience. It also implements relative
#' difference and change as discussed in
#' \url{https://en.wikipedia.org/wiki/Relative_change_and_difference}.
#' @param current One vector.
#' @param reference Another vector, for \code{type = all.equal}, this is passed
#' as \code{target}, for \code{type = all.equal} this can be thought of as the
#' "correct" value or the state "before".
#' @param type The method to be used. See Details.
#' @details
#' The default method (\code{type = all.equal}) applies
#' \code{\link{all.equal}} onto the two
#' vectors. Method \code{type = difference} is somewhat the same as the default,
#' method \code{type = change} takes account of the sign of the differences.
#' @return
#' @export
#' @family statistics
#' @family vector comparing functions
#' @examples
#' n <- 500
#' x <- rnorm(n)
#' y <- x + rnorm(n, sd = 0.0001)
#' plot(relative_difference(x, y), x)
#' plot(relative_difference(x, y, "difference"), x)
#' # They do approximately the same:
#' max(relative_difference(relative_difference(x, y),
#'                             relative_difference(x, y, "difference")))
#' # Takes sign into account:
#' plot(relative_difference(x, y, "change"), x)
#' max(relative_difference(relative_difference(x, y),
#'                         abs(relative_difference(x, y, "change"))))
relative_difference <- function(current, reference,
                                type = c("all.equal", "difference", "change")) {
     switch(match.arg(type),
           "all.equal" = {
               m <- cbind(current = current, reference = reference)
               cmp <- function(x) {
                   res <- all.equal(target = x["reference"],
                                    current = x["current"],
                                    check.attributes = FALSE)
                   return(res)
               }
               ae <- apply(m, 1, cmp)
               rc <- as.numeric(lapply(strsplit(ae, split = ": "), "[", 2))
               rc[is.na(rc)] <- 0
               res <- rc
           },
           "difference" = {
               if (identical(current, reference)) {
                   res <- 0
               } else {
                   denominator <- (abs(current) + abs(reference)) / 2
                   denominator <- ifelse(denominator == 0,  1e-10, denominator)
                   res <- abs((current - reference) / denominator)
               }
           },
           "change" = {
               denominator <- ifelse(reference == 0, 1e-10, reference)
               res <- (current - reference) / abs(denominator)
           }
           )
    return(res)
}

#' Is a Key a Valid Potential Primary Key for a \code{data.frame}?
#'
#' I sometimes see tables with obscure structure so I try to guess their primary
#' keys.
#' @param data The \code{data.frame} for which you want to find valid potential
#' primary key.
#' @param key Character vector containing a subset of the columns names of
#' \code{data}.
#' @param verbose Be verbose?
#' @export
#' @family bits and pieces
#' @return \code{\link{TRUE}}, if \code{key} is a valid primary key,
#' \code{\link{FALSE}} otherwise.
#' @examples
#' is_valid_primary_key(mtcars, "qsec")
#' is_valid_primary_key(mtcars, "carb")
#' is_valid_primary_key(mtcars, c("qsec", "gear"))
#' is_valid_primary_key(mtcars, c("qsec", "carb"))
#' cars <- mtcars
#' cars$id <-  seq_len(nrow(cars))
#' is_valid_primary_key(cars, "id")
is_valid_primary_key <- function(data, key, verbose = TRUE) {
    is_valid <- identical(nrow(data), nrow(unique(data[key])))
    if (isTRUE(verbose)) {
        if (is_valid) {
            message("[", paste(key, collapse = ", "), "]",
                    " is a valid primary key for ",
                    deparse(substitute(data)), ".")
        } else {
            warning("[", paste(key, collapse = ", "), "]",
                    " is not a valid primary key for `",
                    deparse(substitute(data)), "`!")
        }
    }
    return(is_valid)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


data(mtcars)
expect_true(!is_valid_primary_key(mtcars, "qsec"))
expect_true(!is_valid_primary_key(mtcars, "carb"))
expect_true(!is_valid_primary_key(mtcars, c("qsec", "gear")))
expect_true(is_valid_primary_key(mtcars, c("qsec", "carb")))
cars <- mtcars
cars[["id"]] <- seq_len(nrow(cars))
expect_true(is_valid_primary_key(cars, "id"))

if (interactive()) pkgload::load_all(".")
test_FUNCTION <- function() {
    result <- FUNCTION(FIXME)
    expectation <- FIXME
    RUnit::checkIdentical(result, expectation)
}
if (interactive()) {
    test_FUNCTION()
}

#' Find Memory Hogs
#'
#' List objects in an \R environment by size.
#' @param unit  The unit to use.
#' @param envir  The environment where to look for objects.
#' @param return_numeric  Return a numeric vector? If set to
#' \code{\link{FALSE}}, a character vector including the unit will be returned,
#' which might be less usable but easier to read.
#' @param ... Arguments passed to \code{\link{order}}, defaults to
#' \code{decreasing = FALSE}.
#' @return A named vector of memory usages.
#' @family R memory functions
#' @export
#' @examples
#' va <- rep(mtcars, 1)
#' vb <- rep(mtcars, 1000)
#' vc <- rep(mtcars, 2000)
#' vd <- rep(mtcars, 100)
#' memory_hogs()
#' memory_hogs(unit = "Mb", decreasing = TRUE)
#' memory_hogs(unit = "Mb", decreasing = TRUE, return_numeric = FALSE)
#' \dontrun{
#' # remove the two largest objects:
#' rm(list = names(tail(memory_hogs(decreasing = FALSE), n = 2)))
#' memory_hogs(unit = "Mb")
#' }
memory_hogs <- function(unit =  c("b", "Kb", "Mb", "Gb", "Tb", "Pb"),
                        return_numeric = TRUE, ..., envir = .GlobalEnv) {
    u <- match.arg(unit)
    f <- function(x) {
        object_sizes <- utils::object.size(get(x, envir = envir))
        prints <- utils::capture.output(print(object_sizes, standard = "legacy",
                                              units = u))
        return(prints)
    }
    z <- sapply(ls(envir = envir), f)
    if (u == "b") u <- "bytes"
    x <- as.numeric(gsub(paste0(" ", u), "", z))
    names(x) <- names(z)
    if (isTRUE(return_numeric)) {
        res <- x
    } else {
        res <- z
    }
    if (! missing(...)) {
        res <- res[order(x, ...)]
    } else {
        res <- res[order(x, decreasing = FALSE)]
    }
    if (isTRUE(return_numeric)) {
        attr(res, "units") <- u
    }
    return(res)
}

#' Set or Get the \code{path} Attribute to or from an Object
#'
#' We set paths on some objects, these are convenience wrappers to
#' \code{\link{attr}}.
#' @name paths
#' @param x An object.
#' @family file utilities
#' @examples
#' x <- 2
#' path <- tempfile()
#' touch(path)
#' x <- set_path(x, path)
#' get_path(x)
NULL

#' @export
#' @param force Force the retrieval, even if the path is not valid? Only meant
#' for unit testing, leave alone!
#' @rdname paths
#' @aliases get_path
#' @return For \code{get_path} the value of \code{attr(x, "path")}.
get_path <- function(x, force = FALSE) {
    path <- attr(x, "path")
    if (is.null(path)) {
        throw(paste0("No `path` attribute set on ",
                     deparse(substitute(x)), "."))
    } else {
        if (!is_path(path) && !isTRUE(force)) {
            name <- deparse(substitute(x))
            throw(paste0("File `path` does not exists. ",
                        "Attribute `path` of object `", name,
                        "` points to void. ",
                        "Use\n `attr(", name, ", \"path\") <- path`\nfirst."))
        }
        if (!utils::file_test(op = "-f", path))
            throw("`path` not a file, probably a directory.")
    }
    return(path)
}

#' @export
#' @param path The path to be set.
#' @param action Do we have a read or write process? Passed by
#' \code{\link{read_csv}} and \code{\link{write_csv}}. Leave alone otherwise.
#' @param overwrite Overwrite an existing \emph{path} attribute instead of
#' throwing an error?
#' @rdname paths
#' @aliases set_path
#' @return For \code{set_path} the modified object.
set_path <- function(x, path, action = c(NA, "read", "write"),
                     overwrite = FALSE) {
    #% check the path given.
    if (!is.null(attr(x, "path")) && ! isTRUE(overwrite)) {
        throw("Attribute `path` already set, skipping!")
    }
    if (!is_path(path)) {
        warning("Use\n `attr(x, \"path\") <- path`\ninstead.")
        throw("`path` does not exists.")
    }
    if (!utils::file_test(op = "-f", path))
        throw("`path` is a directory, not a file.")

    #% file modification time
    attr(path, "mtime") <- file.mtime(path)
    #% read/write dates
    ##% set defaults first, will be NULL if non-existant
    if (is.null(attr(path, "last_read"))) attr(path, "last_read") <- NA
    if (is.null(attr(path, "last_written"))) attr(path, "last_written") <- NA
    rw <- match.arg(action)
    switch(rw,
           "NA" = { # Do nothing!
           },
           "read" = attr(path, "last_read") <- Sys.time(),
           "write" = attr(path, "last_written") <- Sys.time(),
           throw("Argument `action` has unkown value!"))
    #% add attribute
    attr(x, "path") <- path
    return(x)
}

#' Get the \code{mtime} Attribute to or from an Object
#'
#' We set modification times on some objects, this is a convenience wrappers to
#' \code{\link{attr}}.
#' @param x An object.
#' @family file utilities
#' @export
#' @return The value of \code{attr(attr(x, "path", "mtime")}.
#' @examples
#' x <- 2
#' path <- tempfile()
#' touch(path)
#' x <- set_path(x, path)
#' get_mtime(x)
get_mtime <- function(x) {
    mtime <- attr(attr(x, "path"), "mtime")
    if (is.null(mtime)) {
        throw(paste0("No `path/mtime` attribute set on ",
                     deparse(substitute(x)), "."))
    }
    return(mtime)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


x <- 3
# no path set
expect_error(get_path(x))
# path is a directory
expect_error(set_path(x, tempdir()))
# path does not exists
expect_error(set_path(x, tempfile()))
# externally set path
attr(x, "path") <- tempdir()
# path is a directory
expect_error(get_path(x))
# create a file
tempfile <- tempfile()
touch(tempfile)
# path already set
expect_error(set_path(x, tempfile()))
# overwrite
x <- set_path(x, tempfile, overwrite = TRUE)
result <- get_path(x)
expect_identical(strip_off_attributes(result), tempfile)
# csv
a <- write_csv(mtcars, tempfile)
expect_identical(get_mtime(a), file.mtime(tempfile))
# change something
a[1, 1] <- 0.0
a <- write_csv(a)
expect_identical(get_mtime(a), file.mtime(get_path(a)))
# wrong path set
attr(x, "path") <- tempfile()
expect_error(get_path(x))







x <- mtcars
expect_error(get_mtime(x))
tempfile <- tempfile()
touch(tempfile)
x <- set_path(x, tempfile)
expect_true(file.mtime(tempfile) == get_mtime(x))
expect_true(is(get_mtime(x), "POSIXt"))
result <- get_path(x)
expect_true(is_path(result))
expect_true(is(attr(result, "mtime"), "POSIXt"))
expect_true(is.na(attr(result, "last_read")))
expect_true(is.na(attr(result, "last_written")))
Sys.sleep(1)
touch(tempfile)
expect_true(file.mtime(tempfile) > get_mtime(x))
Sys.sleep(1)
if (!is_cran() && !is_windows()) {
  # TODO: Check fails on CRAN Windows, but that seems strange.
  x <- write_csv(x)
  expect_identical(file.mtime(tempfile), get_mtime(x))
  expect_equal(
    get_mtime(x),
    attr(get_path(x), "last_written"),
    tolerance = 0.1
  )
  expect_true(is.na(attr(result, "last_read")))
  Sys.sleep(1)
  y <- read_csv(tempfile)
  expect_true(get_mtime(y) < attr(get_path(y), "last_read"))
  expect_true(is.na(attr(get_path(y), "last_written")))
  # Now we change it:
  y[1, 2] <- y[1, 2] + 1
  Sys.sleep(1)
  y <- write_csv(y)
  expect_equal(
    get_mtime(x),
    attr(get_path(x), "last_written"),
    tolerance = 0.1
  )
  # Now this is ... a feature: it hasn't changed on disc,
  # because the content doesn't change:
  Sys.sleep(1)
  y <- write_csv(y)
  expect_true(get_mtime(y) < attr(get_path(y), "last_written"))
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}

unlink(dir(tempdir(), full.names = TRUE))
f <- file.path(tempdir(), paste0("a", ".csv"))
cars <- mtcars[1:2, TRUE]
a <- write_csv(cars, file = f)
expect_true(file.exists(f))
expect_identical(strip_off_attributes(get_path(a)), f)

# % get_path
x <- 3
# no path set
expect_error(get_path(x))
# path is a directory
expect_error(set_path(x, tempdir()))
# path does not exists
expect_error(set_path(x, tempfile()))
# externally set path
attr(x, "path") <- tempdir()
# path is a directory
expect_error(get_path(x))
# create a file
tempfile <- tempfile()
touch(tempfile)
# path already set
expect_error(set_path(x, tempfile()))
# overwrite
x <- set_path(x, tempfile, overwrite = TRUE)
result <- strip_off_attributes(get_path(x))
expect_identical(result, tempfile)

call_vim <- function(file, title = file) {
    # a function to be passed to options(editor)
    if (is_installed("gvim")) {
        vim <- "gvim"
    } else {
        if (is_installed("vim")) {
            vim <- "vim"
        } else {
            throw("Can't find gvim nor vim. Please install first.")
        }
    }
    system2(vim, args = c("-p", file))
}

#' Edit a File With VIM if Possible
#'
#' Just a wrapper to \code{\link{file.edit}}, trying to use [g]vim as editor, if
#' installed.
#' @param ... See \code{\link{file.edit}}.
#' @return See \code{\link{file.edit}}.
#' @export
#' @family operating system functions
#' @examples
#' if (interactive()) {
#'     path <- file.path(tempdir(), "foo.txt")
#'     writeLines(c("abc", "xyz"), con = path)
#'     vim(path)
#' }
vim <- function(...) {
    if (! interactive()) {
        warning("Editing files non-interactively.")
    }
    if (is_installed("vim")) old <- options(editor = call_vim)
    res <- utils::file.edit(...)
    if (exists("old", inherits =  FALSE)) options(old)
    return(res)
}

#' Check Whether an Object Contains a Valid File System Path
#'
#' @param x The object.
#' @template return_boolean
#' @family file utilities
#' @export
#' @examples
#' is_path(tempdir())
#' path <- tempfile()
#' is_path(path)
#' touch(path)
#' is_path(path)
is_path <- function(x) {
    result <- is.character(x) && file.exists(x)
    return(result)
}

#' Load a Package's Internals
#'
#' Load objects not exported from a package's namespace.
#'
#'
#' @param package The name of the package as a string.
#' @param ... Arguments passed to \code{\link{ls}}, all.names = TRUE could be a
#' good idea.
#' @seealso \code{\link[codetools:checkUsageEnv]{codetools::checkUsageEnv}}.
#' @template return_invisibly_true
#' @family package functions
#' @export
#' @examples
#' load_internal_functions("fritools2")
load_internal_functions <- function(package, ...) {
    if (requireNamespace("checkmate", quietly = TRUE))
        checkmate::qassert(package, "S1")
    require(package, character.only = TRUE)
    exported_names <- ls(paste("package", package, sep = ":"), ...)
    is_exported_name_function <- vapply(exported_names,
                                        function(x) is.function(get(x)), TRUE)
    exported_functions <- exported_names[is_exported_name_function]
    package_namespace <- asNamespace(package)
    package_names <- ls(envir = package_namespace)
    is_package_name_function <-
        vapply(package_names,
               function(x) is.function(get(x, envir = package_namespace)),
               TRUE)
    package_functions <- package_names[is_package_name_function]
    internal_functions <- setdiff(package_functions, exported_functions)
    for (name in internal_functions) {
        assign(name, get(name, envir = package_namespace, inherits = FALSE),
               envir = parent.frame())
    }
    return(invisible(TRUE))
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


load_internal_functions("fritools2")
result <- environmentName(environment(throw))
expectation <- "fritools2"
expect_identical(result, expectation)

#' Calculate the Golden Ratio
#'
#' Divide a length using the golden ratio.
#' @param x The sum of the two quantities to be in the golden ratio.
#' @return A numeric vector of length 2, containing the two quantities \emph{a}
#' and \emph{b}, \emph{a}  being the larger.
#' @family bits and pieces
#' @export
#' @examples
#' golden_ratio(10)
golden_ratio <- function(x) {
  golden_ratio <- (1 + sqrt(5)) / 2
  a <- x / golden_ratio
  b <- x - a
  return(list(a = a, b = b))
}

#' Wipe Clean the \code{tempdir()}
#'
#' I often need a clean temporary directory.
#' @param recreate Use the method described in the examples section of
#' \code{\link{tempdir}} (using \code{tempdir(check = TRUE)}, this results in a
#' new path.)
#' @family R memory functions
#' @family operating system functions
#' @return The path to the temporary directory.
#' @export
#' @examples
#' \dontrun{
#' dir.create(t <- file.path(tempdir(), "foo"))
#' touch(f1 <- file.path(t, "first.R"),
#'       f2 <- file.path(t, "second.R"))
#' dir(tempdir(), recursive = TRUE)
#' wipe_tempdir()
#' dir(tempdir(), recursive = TRUE)
#' }
wipe_tempdir <- function(recreate = FALSE) {
    if (isTRUE(recreate)) {
        unlink(tempdir(), recursive = TRUE, force = TRUE)
        res <- tempdir(check = TRUE)
    } else {
        unlink(dir(tempdir(), all.files = TRUE, full.names = TRUE),
               recursive = TRUE)
        res <- tempdir()
    }
    return(invisible(res))
}

#' View a File or Directory
#'
#' Call \code{shell.exec} on windows, mimic \code{shell.exec}
#' otherwise.
#' @param path A path to a file or directory.
#' @param program A program to use.
#' @template return_invisibly_null
#' @family operating system functions
#' @export
#' @examples
#' path <- file.path(tempdir(), "foo.txt")
#' writeLines(c("abc", "xyz"), con = path)
#' view(path)
view <- function(path, program = NA) {
    qpath <- shQuote(normalizePath(path, mustWork = TRUE))
    if (interactive()) {
        if (!is.na(program) && is_installed(program)) {
            system2(program, qpath, wait = FALSE)
        } else {
            if (fritools2::is_windows()) {
                shell.exec(path) # Exclude Linting: this is windows only
            } else {
                if (fritools2::is_installed("thunar")) {
                    system2("thunar", qpath, wait = FALSE)
                } else {
                    if (dir.exists(qpath)) {
                        list.files(qpath, full.names = TRUE)
                    } else {
                        throw(paste0("Don't know how to open ", qpath, "."))
                    }
                }
            }
        }
    } else {
        warning("You are in batch mode.")
    }
    return(invisible(NULL))
}

#' Force Copying a File While backing it up
#'
#' \code{\link{file.copy}} has an argument \code{overwrite} that allows for
#' overwriting existing files. But I often want to overwrite an existing file
#' while creating a backup copy of that file.
#' @param from See \code{\link{file.copy}}.
#' @param to See \code{\link{file.copy}}.
#' @template stop_on_error
#' @param ... Arguments passed to \code{\link{file.copy}}.
#' @template return_boolean_vector
#' @family file utilities
#' @family operating system functions
#' @export
#' @examples
#' touch(f1 <- file.path(tempdir(), "first.R"),
#'       f2 <- file.path(tempdir(), "second.R"))
#' dir.create(t <- file.path(tempdir(), "foo"))
#' file_copy(from = c(f2, f1), to = t)
#' dir(t)
#' touch(f1)
#' touch(f2)
#' file_copy(from = c(f2, f1), to = t)
#' dir(t)
#' list.files(tempdir(), pattern = "first.*\\.R")
#' dir <- file.path(tempdir(), "subdir")
#' dir.create(dir)
#' file_copy(f1, dir)
#' touch(f1)
#' file_copy(f1, dir)
#' list.files(dir, pattern = "first.*\\.R")
file_copy <- function(from, to, stop_on_error = FALSE, ...) {
    if (length(to) == 1 && dir.exists(to)) {
        target <- file.path(to, basename(from))
    } else {
        target <- to
    }
    is_target_newer <- sapply(file.mtime(from) <= file.mtime(target), isTRUE)
    if (any(is_target_newer)) {
        msg <- paste0("`", target[is_target_newer], "` is newer than ", "`",
                      from[is_target_newer], "`, skipping.")
        if (isTRUE(stop_on_error)) {
            throw(msg)
        } else {
            warning(msg)
        }
    }
    is_target_exists <- file.exists(target)
    if (any(is_target_exists)) {
        do.call("file_save",
                # capture NA by sapply(..., isTRUE)
                as.list(target[sapply(is_target_exists & !is_target_newer,
                                      isTRUE)]))
    }

    res <- rep(FALSE, length(from))
    for (idx in seq(along = from)) {
        res[idx] <- file.copy(from = from[idx], to = target[idx],
                         overwrite = (is_target_exists & !is_target_newer)[idx],
                         recursive = (dir.exists(from)[idx]),
                         ...)
    }
    return(invisible(res))
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


f1 <- file.path(tempdir(), "first.R")
f2 <- file.path(tempdir(), "second.R")
expect_true(isFALSE(file_copy(f2, f1)))
touch(f1)
touch(f2)
touch(f2) # make sure that f2 is newer than f1
expect_true(isFALSE(file_copy(f1, f2, stop_on_error = FALSE)))
expect_error(file_copy(f1, f2, stop_on_error = TRUE))
expect_true(file_copy(f2, f1))
file_names <- list.files(tempdir(), pattern = "first.*\\.R")
expect_identical(length(file_names), 2L)
pattern <- "first_[0-9]{4}_[0-9]{2}_[0-9]{2}_[0-9]{2}_[0-9]{2}_[0-9]{2}\\.R"
expect_true(any(grepl(pattern, file_names)))
dir <- file.path(tempdir(), "subdir")
dir.create(dir)
file_copy(f1, dir)
file_names <- list.files(dir, pattern = "first.*\\.R")
expect_identical(length(file_names), 1L)
expect_identical("first.R", file_names)
Sys.sleep(1)
touch(f1)
expect_true(file_copy(f1, dir))
file_names <- list.files(dir, pattern = "first.*\\.R")
expect_identical(length(file_names), 2L)
expect_true(any(grepl(pattern, file_names)))
expectation <- c(FALSE, TRUE)
result <- file_copy(c(f1, f2), dir)
if (interactive()) {
  print(result)
  print(expectation)
  print("#%% times")
  print(sapply(dir(dir, full.names = TRUE), file.mtime))
  print(sapply(list(f1, f2), file.mtime))
}
expect_identical(result, expectation)
Sys.sleep(1)
touch(f1)
expectation <- c(TRUE, FALSE)
result <- file_copy(c(f1, f2), dir, stop_on_error = FALSE)
expect_identical(result, expectation)

#' Check Whether Two Times Differ Less Than A Given Value
#'
#' This is just a wrapper to \code{\link{difftime}}.
#'
#' @param time1 See \code{\link{difftime}}.
#' @param time2 See \code{\link{difftime}}.
#' @param less_than The number of \bold{units} that would be too much of a
#' difference.
#' @param units See \code{\link{difftime}}.
#' @param verbose Be verbose?
#' @param visible Set to \code{\link{FALSE}} to return
#' \code{\link{invisible}}.
#' @param stop_on_error Throw an error if the time lag is not less than
#' \bold{less_than}.
#' @export
#' @family bits and pieces
#' @return \code{\link{TRUE}} if the times do not differ `that much`, but see
#' \bold{stop_on_error}.
#' @examples
#' a <- as.POSIXct(0, origin = "1970-01-01", tz = "GMT")
#' b <- as.POSIXct(60*60*24, origin = "1970-01-01", tz = "GMT")
#' c <- as.POSIXct(60*60*24 - 1, origin = "1970-01-01", tz = "GMT")
#' is_difftime_less(a, b)
#' is_difftime_less(a, c)
#' print(is_difftime_less(a, b, verbose = TRUE))
#' print(is_difftime_less(a, c, verbose = TRUE))
#' try(is_difftime_less(a, b, stop_on_error = TRUE))
#' is_difftime_less(a, c, verbose = TRUE, stop_on_error = TRUE)
is_difftime_less <- function(time1, time2, less_than = 1, units = "days",
                             verbose = FALSE, visible = !verbose,
                             stop_on_error = FALSE) {
    res <- NULL
    if (abs(difftime(time1, time2, units = units)) >= less_than) {
        msg <- paste0("Time difference between ", time1, " and ", time2,
                "\nis at least ", less_than, " ", units, ".")
        if (isTRUE(stop_on_error)) {
            throw(msg)
        } else {
            if (isTRUE(verbose)) {
                warning(msg)
            }
            res <- FALSE
        }
    } else {
        if (isTRUE(verbose)) {
            msg <- paste0("Time difference between ", time1, " and ", time2,
                          "\nis less than ", less_than, " ", units, ".")
            message(msg)
        }
        res <- TRUE
    }
    if (isTRUE(visible)) {
        return(res)
    } else {
        return(invisible(res))
    }
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


a <- as.POSIXct(0, origin = "1970-01-01", tz = "GMT")
b <- as.POSIXct(60 * 60 * 24, origin = "1970-01-01", tz = "GMT")
c <- as.POSIXct(60 * 60 * 24 - 1, origin = "1970-01-01", tz = "GMT")
expect_true(!is_difftime_less(a, b, verbose = TRUE))
expect_true(is_difftime_less(a, c, verbose = TRUE))
expect_error(is_difftime_less(a, b,
  verbose = TRUE,
  stop_on_error = TRUE
))
expect_true(is_difftime_less(a, c,
  verbose = TRUE,
  stop_on_error = TRUE
))

#' Is an \R Package Installed?
#'
#' Is an \R package installed?
#' @param x Name of the package as character string.
#' @param version Required minimum version of the package as character string.
#' @template return_boolean
#' @family logical helpers
#' @family operating system functions
#' @family package functions
#' @family version functions
#' @export
#' @examples
#' is_r_package_installed("base", "300.0.0")
#' is_r_package_installed("fritools2", "1.0.0")
is_r_package_installed <- function(x, version = "0") {
    installed <- tryCatch(utils::packageVersion(x), error = function(e) NA)
    is_installed  <- !is.na(installed)
    is_sufficient <- is_version_sufficient(installed = as.character(installed),
                                           required = as.character(version))
    return(is_installed && is_sufficient)
}

#' Is an External Program Installed?
#'
#' Is an external program installed?
#'
#' @param program Name of the program.
#' @template return_boolean
#' @family logical helpers
#' @family operating system functions
#' @export
#' @examples
#' if (is_running_on_fvafrcu_machines() || is_running_on_gitlab_com()) {
#'     # NOTE: There are CRAN machines where neither "R" nor "R-devel" is in
#'     # the path, so we skipt this example on unkown machines.
#'     is_installed("R")
#' }
#' is_installed("probably_not_installed")
is_installed <- function(program) {
    is_installed <- nchar(Sys.which(program)) > 0
    is_installed <- unname(is_installed)
    is_installed <- isTRUE(is_installed)
    return(is_installed)
}

#' Remove Trailing Blank Lines From Files
#'
#' Trailing blank lines are classical lints.
#' @param ... Arguments passed to \code{\link{find_files}}.
#' @family file utilities
#' @template return_invisibly_null
#' @export
#' @examples
#' dir <- tempfile()
#' dir.create(dir)
#' file.copy(system.file("runit_tests", package = "fritools2"), dir,
#'           recursive = TRUE)
#' delete_trailing_blank_lines(path = dir, recursive = TRUE)
#' unlink(dir, recursive = TRUE)
delete_trailing_blank_lines <- function(...) {
    files <- find_files(...)
    for (f in files) {
        lines <- readLines(f)
        idx <- !grepl("^$", lines)
        idx[seq_len(max(which(idx == TRUE)))] <- TRUE
        lines <- lines[idx]
        writeLines(lines, f)
    }
    return(invisible(NULL))
}

#' Create a Copies of  Files
#'
#' I often want a timestamped copies as backup of files or directories.
#' @param ... Paths to files.
#' @param file_extension_pattern A Pattern to mark a file extension. If matched,
#' the time stamp will get inserted before that pattern.
#' @param force Force even if \code{file_extension_pattern} is not matched. Set
#' to \code{\link{FALSE}} to skip stamping such files.
#' @param overwrite Passed to \code{\link{file.copy}}.
#' @param recursive Passed to \code{\link{file.copy}}. Defaults to `if the
#' current path is a directory, then TRUE, else FALSE`.
#' @template stop_on_error
#' @template return_boolean_vector
#' @family operating system functions
#' @family file utilities
#' @export
#' @examples
#' f1 <- tempfile()
#' f2 <- tempfile()
#' try(file_save(f1))
#' touch(f1)
#' file_save(f1, recursive = FALSE)
#' f2 <- paste0(file.path(tempfile()), ".txt")
#' touch(f2)
#' file_save(f1, f2)
#' file_save(f1, f2)
#' file_save(f1, f2, overwrite = TRUE)
#' dir(tempdir())
file_save <- function(..., file_extension_pattern = "\\.[A-z]{1,5}$",
                       force = TRUE, recursive = NA,
                       stop_on_error = TRUE, overwrite = FALSE) {
    l <- list(...)
    fep <- file_extension_pattern # sooth lintr
    res <- sapply(l, function(x) {
                      res <- .file_save(x = x,
                                        file_extension_pattern = fep,
                                        force = force,
                                        recursive = dir.exists(x),
                                        stop_on_error = stop_on_error,
                                        overwrite = overwrite)
                      return(res)
                       },
                      USE.NAMES = FALSE)
    return(res)
}

.file_save <- function(x, file_extension_pattern = "\\.[A-z]{1,5}$",
                       force = TRUE, recursive = dir.exists(x),
                       stop_on_error = TRUE, overwrite = FALSE) {
    res <- FALSE
    if (!file.exists(x)) {
        msg <- paste0("`", x, "` is not a path to a file.")
        if (isTRUE(stop_on_error)) {
            throw(msg)
        } else {
            warning(msg)
        }
    } else {
        if (grepl(file_extension_pattern, x)) {
            to <- sub(paste0("(", file_extension_pattern, ")"),
                      paste0(format(file.mtime(x), "_%Y_%m_%d_%H_%M_%S"),
                             "\\1"),
                      x)
            res <- file.copy(from = x, to = to, recursive = recursive,
                             overwrite = overwrite)
        } else {
            if (isTRUE(force)) {
                to <-  paste0(x, format(file.mtime(x), "_%Y_%m_%d_%H_%M_%S"))
                res <- file.copy(from = x, to = to, recursive = recursive,
                                 overwrite = overwrite)
            } else {
                warning("Set `force` to `TRUE` to force.")
            }
        }
    }
    return(res)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


f <- file.path(tempfile())
expect_identical(file_save(f, stop_on_error = FALSE), FALSE)
expect_error(file_save(f))
touch(f)
expect_identical(file_save(f, force = FALSE), FALSE)
expect_identical(length(dir(dirname(f), pattern = basename(f))), 1L)
file_save(f, recursive = FALSE)
expect_identical(length(dir(dirname(f), pattern = basename(f))), 2L)
f1 <- paste0(f, ".txt")
touch(f1)
expect_identical(length(dir(dirname(f), pattern = basename(f))), 3L)
file_save(f1)
expect_identical(length(dir(dirname(f), pattern = basename(f))), 4L)
# now both files already exist
expect_true(any(file_save(f, f1), overwrite = TRUE))
expect_true(!any(file_save(f, f1)))

#' Mock the Unix \command{touch} Utility
#'
#' Creating files or ensuring that their file modification times change.
#' @param ... Paths to files.
#' @family file utilities
#' @return The Paths to the files touched.
#' @export
#' @examples
#' file1 <- tempfile()
#' file2 <- tempfile()
#' touch(file1, file2)
#' t1 <- file.mtime(file1, file2)
#' touch(file2)
#' t2 <- file.mtime(file1, file2)
#' t1 < t2
#' file <- file.path(tempfile(), "path", "not", "there.txt")
#' touch(file)
#' file.exists(file)
touch <- function(...) {
    paths <- list(...)
    lapply(paths, .touch)
    return(invisible(unlist(paths)))
}
.touch <- function(path) {
    if (!file.exists(path)) {
        if (!dir.exists(dirname(path))) dir.create(dirname(path),
                                                   recursive = TRUE)
        cat("# This file was generated by fritools2::touch()", file = path)
    } else {
        tmp <- tempfile()
        on.exit(unlink(tmp))
        file.copy(path, tmp)
        if (file.mtime(tmp) <= file.mtime(path)) Sys.sleep(2)
        file.copy(tmp, path, overwrite = TRUE)
    }
}

#' @description \code{touch2} is an alternate - yet not faster - implementation.
#' @rdname touch
#' @export
#'
touch2 <- function(...) {
    # I tought I could save time (file.copy and file.mtime can digest multiple
    # paths) by using an index, but it didn't work out.
    paths <- unlist(list(...))
    idx <- file.exists(paths)
    if (any(idx)) {
        tmp <- replicate(sum(idx), tempfile())
        on.exit(unlink(tmp))
        file.copy(paths[idx], tmp)
        if (any(file.mtime(tmp) <= file.mtime(paths[idx]))) Sys.sleep(2)
        file.copy(tmp, paths[idx], overwrite = TRUE)
    }
    invisible(sapply(paths[!idx], .touch2))
    return(invisible(unlist(paths)))
}
.touch2 <- function(path) {
    if (!dir.exists(dirname(path))) dir.create(dirname(path),
                                               recursive = TRUE)
    cat("# This file was generated by fritools2::touch()", file = path)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


p1 <- tempfile()
p2 <- tempfile()
p3 <- tempfile()
touch(p1)
touch(p2)
Sys.sleep(2)
touch(p3)
# one file
expect_true(is_files_current(p3,
  newer_than = 1, units = "days",
  within = 4, within_units = "secs"
))
# more files
expect_true(is_files_current(p1, p2, p3,
  newer_than = 1,
  units = "days",
  within = 4, within_units = "secs"
))
# within not TRUE
expect_true(!is_files_current(p1, p2, p3,
  newer_than = 1,
  units = "days",
  within = 1, within_units = "secs"
))
# newer not TRUE
expect_true(!is_files_current(p1, p2, p3,
  newer_than = 1,
  units = "secs",
  within = 4, within_units = "secs"
))

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


file1 <- tempfile()
file2 <- tempfile()
touch(file1, file2)
t1 <- file.mtime(file1, file2)
touch(file2)
t2 <- file.mtime(file1, file2)
expect_identical(t1 < t2, c(FALSE, TRUE))
file <- file.path(tempfile(), "path", "not", "there.txt")
touch(file)
expect_true(file.exists(file))





file1 <- tempfile()
file2 <- tempfile()
touch2(file1, file2)
t1 <- file.mtime(file1, file2)
touch2(file2)
t2 <- file.mtime(file1, file2)
expect_identical(t1 < t2, c(FALSE, TRUE))
file <- file.path(tempfile(), "path", "not", "there.txt")
touch2(file)
expect_true(file.exists(file))

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


dir.create(file.path(tempdir(), "new"))
touch(file.path(tempdir(), "file1.txt"))
Sys.sleep(2)
touch(file.path(tempdir(), "file2.txt"))
Sys.sleep(2)
touch(file.path(tempdir(), "new", "file3.txt"))
expectation <- "file2.txt"
found <- file_modified_last(path = tempdir(), pattern = "file.\\.txt$")
result <- basename(found)
expect_identical(expectation, result)
expectation <- "file3.txt"
found <- file_modified_last(
  path = tempdir(), pattern = "file.\\.txt$",
  recursive = TRUE
)
result <- basename(found)
expect_identical(expectation, result)

#' Is \R Running on CRAN?
#'
#' \emph{This is a verbatim copy of \code{fda::CRAN} of
#' \pkg{fda} version 5.1.9.}
#'
#'  This function allows package developers to run tests themselves that
#'  should not run on CRAN or with \preformatted{R CMD check --as-cran} because
#'  of compute time constraints with CRAN tests.
#'
#'  The "Writing R Extensions" manual says that \command{R CMD check} can be
#'  customized "by setting environment variables _R_CHECK_*_:, as
#'  described in" the Tools section of the "R Internals" manual.
#'
#'  \command{R CMD check} was tested with R 3.0.1 under Fedora 18 Linux and with
#'  \command{Rtools} 3.0 from April 16, 2013 under Windows 7.  With the
#'  \preformatted{'--as-cran'}
#'  option, 7 matches were found;  without it, only 3 were found.  These
#'  numbers were unaffected by the presence or absence of the '--timings'
#'  parameter.  On this basis, the default value of n_R_CHECK4CRAN was set
#'  at 5.
#'
#'  1.  \code{x. <- Sys.getenv()}
#'
#'  2.  Fix \code{CRAN_pattern} and \code{n_R_CHECK4CRAN} if missing.
#'
#'  3.  Let i be the indices of x. whose names match all the patterns in
#'  the vector x.
#'
#'  4.  Assume this is CRAN if length(i) >= n_R_CHECK4CRAN
#'
#' @param cran_pattern A regular expressions to apply to the names of
#' \code{Sys.getenv()}
#'    to identify possible CRAN parameters.  Defaults to
#'    \code{Sys.getenv('_CRAN_pattern_')} if available and '^_R_' if not.
#' @param n_r_check4cran  Assume this is CRAN if at least n_R_CHECK4CRAN
#' elements of
#'    \code{Sys.getenv()} have names matching x.  Defaults to
#'    \code{Sys.getenv('_n_R_CHECK4CRAN_')} if available and 5 if not.
#' @return
#'  A logical scalar with attributes \samp{'sys_getenv'} containing the
#'  results of \code{Sys.getenv()} and 'matches' containing \code{i} per step 3
#'  above.
#' @export
#' @family test helpers
#' @family logical helpers
#' @examples
#' if (!is_cran()) {
#'     message("Run your tests here.")
#' }
is_cran <- function(cran_pattern, n_r_check4cran) {
    gete <- Sys.getenv()
    ngete <- names(gete)
    i <- seq(along = gete)
    if (missing(cran_pattern)) {
        if ("_cran_pattern_" %in% ngete) {
            cran_pattern <- gete["_cran_pattern_"]
        } else {
            cran_pattern <- "^_R_"
        }
    }
    if (missing(n_r_check4cran)) {
        if ("_n_r_check4cran_" %in% ngete) {
            n_r_check4cran <- as.numeric(gete["_n_r_check4cran_"])
        } else {
            n_r_check4cran <- 5
        }
    }
    for (pati in cran_pattern)
        i <- i[grep(pati, ngete[i])]
    is_cran <- (length(i) >= n_r_check4cran)
    attr(is_cran, "sys_getenv") <- gete
    attr(is_cran, "matches") <- i
    return(is_cran)
}

#' Utilities for the Forest Research Institute of the State
#' Baden-Wuerttemberg
#'
#' Miscellaneous utilities, tools and helper functions.
#'
#' You will find the details in\cr
#' \code{vignette("An_Introduction_to_fritools", package = "fritools2")}.
#'
#' @name fritools2-package
#' @aliases fritools2-package
#' @docType package
#' @keywords package
NULL

#' Check Whether Files are Current
#'
#' I sometimes produce a couple of files by some kind of process and need to
#' check whether they are fairly current and probably product of the same run.
#' So I need to know whether a bunch of files was modified within the
#' last, say, 7 days \emph{and} that their modification dates do not differ by
#' more than, say, 24 hours.
#'
#' @param ... File paths.
#' @param newer_than The number of \bold{units} the files need to be newer than.
#' @param units The unit of \bold{newer_than}. See \code{\link{difftime}}.
#' @param within The number of \bold{units} the files need to be modified
#' within.
#' @param within_units The unit of \bold{within}. See \code{\link{difftime}}.
#' @export
#' @family file utilities
#' @template return_boolean
#' @examples
#' p1 <- tempfile()
#' p2 <- tempfile()
#' p3 <- tempfile()
#' touch(p1)
#' touch(p2)
#' Sys.sleep(3)
#' touch(p3)
#' is_files_current(p3, newer_than = 1, units = "days",
#'                  within = 4, within_units = "secs")
#' is_files_current(p1, p2, p3, newer_than = 1, units = "days",
#'                  within = 4, within_units = "secs")
#' is_files_current(p1, p2, p3, newer_than = 1, units = "days",
#'                  within = 1, within_units = "secs")
#' is_files_current(p1, p2, p3, newer_than = 1, units = "secs",
#'                  within = 4, within_units = "secs")
is_files_current <- function(..., newer_than = 1, units = "week",
                             within = 1, within_units = "days") {
    files <- list(...)
    times <-  sort(as.POSIXct(sapply(files, file.mtime), origin = "1970-01-01"))
    if (length(times) > 1) {
        res <- is_difftime_less(max(times), min(times),
                                less_than = within, units = within_units)
    } else {
        res <- TRUE
    }
    res <- res && is_difftime_less(min(times), Sys.time(),
                                   less_than = newer_than,
                                   units = units)
    if (difftime(max(times), Sys.time()) > 0)
        warning("At least one file is newer than Sys.time().")
    return(res)
}

#' Calculate a Weighted Variance
#'
#' Calculate a weighted variance.
#' @param x A numeric \code{\link{vector}} or \code{\link{data.frame}}.
#' @param ... Other arguments ignored.
#' @export
#' @family statistics
weighted_variance <- function(x, ...) {
    UseMethod("weighted_variance", x)
}

#' @rdname weighted_variance
#' @export
#' @param weights A vector of weights.
#' @param weights_counts Are the weights counts of the data? If so, we can
#' calculate the unbiased sample variance, otherwise we calculate the biased
#' (maximum likelihood estimator of the) sample variance.
#' @examples
#' ## GPA from Siegel 1994
#' wt <- c(5,  5,  4,  1)/15
#' x <- c(3.7,3.3,3.5,2.8)
#' var(x)
#' weighted_variance(x = x)
#' weighted_variance(x = x, weights = wt)
#' weighted_variance(x = x, weights = wt, weights_counts = TRUE)
#' weights <- c(5,  5,  4,  1)
#' weighted_variance(x = x, weights = weights)
#' weighted_variance(x = x, weights = weights, weights_counts = FALSE)
weighted_variance.numeric <- function(x, weights, weights_counts = NULL, ...) {
    if (length(x) < 2) {
        variance <- NA
    } else {
        if (missing(weights)) {
            variance <- stats::var(x)
        } else {
            if (!is.logical(weights_counts)) {
                is_counts <- !isTRUE(all.equal(sum(weights), 1))
            } else {
                is_counts <- weights_counts
            }
            if (is_counts) {
                if (isTRUE(all.equal(sum(weights), 1)) &&
                    isTRUE(weights_counts)
                )
                    message("You forced the weights to be counts, ",
                            "but they do sum to 1!")
                mean <- stats::weighted.mean(x = x, w = weights)
                variance <- (x - mean)^2 %*% weights / (sum(weights) - 1)
                variance <- as.numeric(variance)
            } else {
                if (!isTRUE(all.equal(sum(weights), 1)) &&
                    isFALSE(weights_counts)
                )
                    message("You forced the weights not to be counts, ",
                            "but they do not sum to 1!")
                normalized_weights <- weights / sum(weights)
                mean <- sum(x * normalized_weights)
                variance <- sum((x - mean)^2 * normalized_weights)

            }
        }
    }
    return(variance)

}

#' @details
#' The  \code{\link{data.frame}} method is meant for use with
#' \code{\link{tapply}}, see \emph{examples}.
#' @rdname weighted_variance
#' @export
#' @param var The name of the column in \code{x} giving the variable of
#' interest.
#' @param weight The name of the column in \code{x} giving the weights.
#' @examples
#' weighted_variance(x = data.frame(x, wt), var = "x",
#'                              weight = "wt")
#' # apply by groups:
#' fritools2::tapply(object = mtcars,
#'                  index = list(mtcars[["cyl"]], mtcars[["vs"]]),
#'                  func = weighted_variance, var = "mpg", w = "wt")
weighted_variance.data.frame <- function(x, var, weight, ...) {# Exclude Linting
    return(weighted_variance(x = x[[var]], weights = x[[weight]], ...))
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


wt <- c(5, 5, 4, 1) / 15
x <- c(3.7, 3.3, 3.5, 2.8)
expectation <- var(x)
result <- weighted_variance(x = x)
expect_identical(result, expectation)
expectation <- 0.0571555555555556
result <- weighted_variance(x = x, weights = wt)
expect_equal(result, expectation)
result <- weighted_variance(x = x, weights = wt, weights_counts = TRUE)
expect_equal(result, Inf)









result <- fritools2::tapply(
  object = mtcars, index = list(
    mtcars[["cyl"]],
    mtcars[["vs"]]
  ),
  func = weighted_variance, var = "mpg", w = "wt",
  weights_counts = FALSE
)
expectation <- structure(c(
  NA, 0.376572372584046, 6.83915727277488,
  19.1830134992249, 1.94117209543316, NA
),
.Dim = 3:2,
.Dimnames = list(c("4", "6", "8"), c("0", "1"))
)
expect_equal(result, expectation)

#' Convert German umlauts to a more or less suitable ascii representation.
#'
#' @param x A string or \code{data.frame}.
#' @return \bold{x} with the umlauts converted to ascii.
#' @family German umlaut converters
#' @export
convert_umlauts_to_ascii <- function(x) {
    UseMethod("convert_umlauts_to_ascii", x)
}

#' @rdname convert_umlauts_to_ascii
#' @export
#' @examples
#' string <- paste("this is \u00e4 string")
#' print(string)
#' print(convert_umlauts_to_ascii(string))
convert_umlauts_to_ascii.character <- function(x) { # Exclude Linting
    s <- iconv(enc2native(x), to = "UTF-8", sub = "unicode")
    s <- gsub("\u00e4", "ae", s)
    s <- gsub("\u00c4", "Ae", s)
    s <- gsub("\u00f6", "oe", s)
    s <- gsub("\u00d6", "Oe", s)
    s <- gsub("\u00fc", "ue", s)
    s <- gsub("\u00dc", "Ue", s)
    s <- gsub("\u00df", "sz", s)
    return(s)
}

#' @rdname convert_umlauts_to_ascii
#' @export
#' @examples
#' string <- paste("this is \u00e4 string")
#' df <- data.frame(v1 = c(string, "foobar"),
#'                  v2 = c("foobar", string), v3 = 3:4)
#' names(df)[3] <- "y\u00dfy"
#' convert_umlauts_to_ascii(df)
convert_umlauts_to_ascii.data.frame <- function(x) { # Exclude Linting
    f <- function(x) {
        res <- x
        if (is.character(res))
            res <- convert_umlauts_to_ascii(res)
        return(res)
    }
    df <- data.frame(lapply(x, f))
    attributes(df) <- attributes(x)
    names(df) <- convert_umlauts_to_ascii(names(x))
    colnames(df) <- convert_umlauts_to_ascii(colnames(x))
    return(df)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}
if (fritools2::is_version_sufficient(
  fritools2::get_package_version("base"),
  "4.0.0"
)) {
  if (interactive()) pkgload::load_all(".")

  string <- c(
    "\u00e4", "\u00c4", "\u00f6", "\u00d6", "\u00fc", "\u00dc",
    "\u00df"
  )
  expectation <- c("ae", "Ae", "oe", "Oe", "ue", "Ue", "sz")
  result <- convert_umlauts_to_ascii(string)
  expect_identical(result, expectation)
  df <- data.frame(
    v1 = c(string, "foobar"),
    v2 = c("foobar", string), v3 = 3:4
  )
  names(df)[3] <- "y\u00dfy"
  result <- convert_umlauts_to_ascii(df)
  expectation <-
    structure(list(
      v1 = c(
        "ae", "Ae", "oe", "Oe", "ue", "Ue", "sz",
        "foobar"
      ),
      v2 = c(
        "foobar", "ae", "Ae", "oe", "Oe", "ue", "Ue",
        "sz"
      ),
      yszy = c(3L, 4L, 3L, 4L, 3L, 4L, 3L, 4L)
    ),
    class = "data.frame", row.names = c(NA, -8L)
    )
  expect_identical(result, expectation)
}
if (interactive()) {

}

#' Remove Trailing Whitespace From Files
#'
#' Trailing whitespace is a classical lint.
#' @param ... Arguments passed to \code{\link{find_files}}.
#' @family file utilities
#' @template return_invisibly_null
#' @export
#' @examples
#' dir <- tempfile()
#' dir.create(dir)
#' file.copy(system.file("runit_tests", package = "fritools2"), dir,
#'           recursive = TRUE)
#' delete_trailing_whitespace(path = dir, recursive = TRUE)
#' unlink(dir, recursive = TRUE)
delete_trailing_whitespace <- function(...) {
    files <- find_files(...)
    for (f in files) {
        lines <- readLines(f)
        lines <- sub("\\s+$", "", lines)
        writeLines(lines, f)
    }
    return(invisible(NULL))
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


template <- system.file("files", "blanks.txt", package = "fritools2")
input <- tempfile()
file.copy(template, input)
delete_trailing_whitespace(file_names = input)
result <- readLines(input)
template <- readLines(template)
expect_identical(
  nchar(result),
  as.integer(nchar(template) - c(2, 0, 0, 0))
)

#' Does the Return Value of a Command Signal Success?
#'
#' This is just a wrapper to ease the evaluation of return values from external
#' commands:
#' External commands return 0 on success, which is
#' \code{\link[base:FALSE]{FALSE}}, when converted to logical.
#'
#' @param x The external commands return value.
#' @template return_boolean
#' @family logical helpers
#' @family operating system functions
#' @export
#' @examples
#' is_success(0)
#' is_success(1)
#' is_success(-1)
is_success <- function(x) return(!as.logical(x))

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


result <- is_success(0)
expectation <- TRUE
expect_identical(result, expectation)
result <- is_success(-1)
expectation <- FALSE
expect_identical(result, expectation)

#' Develop Unit Testing for a Code File
#'
#' Looking at the output of
#' \code{\link[covr:zero_coverage]{covr::zero_coverage}}, I want to open a code
#' file an the corresponding unit testing file.
#' @param file The path to the code file, assuming the working directory to be
#' the root of an R package under development.
#' @param force_runit If there is no corresponding \pkg{RUnit} test file:
#' create one?
#' @param force_tiny If there is no corresponding \pkg{tinytest} test file:
#' create one?
#' @template return_invisibly_null
#' @export
#' @family test_helpers
#' @family file utilities
#' @examples
#' \dontrun{
#'     develop_test(file = "R/develop_test.R", force_runit = TRUE)
#'     unlink("inst/tinytest/test_develop_test.R")
#'     unlink("inst/runit_tests/runit-develop_test.R")
#' }
develop_test <- function(file, force_runit = FALSE, force_tiny = TRUE) {
    if (!file.exists(file)) stop("No such file: ", file)
    files <- list("source" = file)
    func <- sub("\\.[Rr]", "", basename(file))
    runit_file <- file.path("inst", "runit_tests",
                            paste0("runit-", basename(file)))
    if (file.exists(runit_file)) {
        files[["runit"]] <- runit_file
    } else {
        if (isTRUE(force_runit)) {
            code <- readLines(system.file("templates", "runit.R",
                                          package = "fritools2"))
            writeLines(gsub("(_|\\<)FUNCTION\\>", paste0("\\1", func), code),
                       sep = "\n", con = runit_file)
            files[["runit"]] <- runit_file
        }
    }
    tiny_file <- file.path("inst", "tinytest",
                            paste0("test_", basename(file)))
    if (file.exists(tiny_file)) {
        files[["tinytest"]] <- tiny_file
    } else {
        if (isTRUE(force_tiny)) {
            code <- readLines(system.file("templates", "tinytest.R",
                                          package = "fritools2"))
            writeLines(gsub("(_|\\<)FUNCTION\\>", paste0("\\1", func), code),
                       sep = "\n", con = tiny_file)
            files[["tinytest"]] <- tiny_file
        }
    }
    do.call(vim, files)
    return(invisible(NULL))
}

#' Convert a German Comma Separated File into a Comma Separated File
#'
#' @param file Path to the file.
#' @param ... Arguments passed to \code{\link{read_csv}}
#' @return \code{\link[base:invisible]{Invisibly}} the return value of
#' \code{\link{write_csv}}, but called for its side effect.
#' @family CSV functions
#' @export
#' @examples
#' f <- tempfile()
#' write.csv2(mtcars, file = f)
#' res <- csv2csv(f)
#' readLines(get_path(res), n = 1)
#' write.csv(mtcars, file = f)
#' readLines(get_path(res), n = 1)
csv2csv <- function(file, ...) {
    content <- read_csv(file = file, ...)
    res <- write_csv(content, csv_type = "standard")
    return(invisible(res))
}

has_digest <- function() return(requireNamespace("digest", quietly = TRUE))

#' Set a Hash Attribute on an Object
#'
#' @param x The object.
#' @return The modified object.
#' @family hash functions for objects
#' @export
set_hash <- function(x) {
    if (has_digest()) {
        attr(x, "hash") <- NULL
        attr(x, "hash") <- digest::digest(x)
    } else {
        throw("Package digest is needed, please install.")
    }
    return(x)
}

#' Separate an Object from its Hash Attribute
#'
#' We calculate a hash value of an object and store it as an attribute of the
#' objects, the hash value of that object will change. So we need to split the
#' hash value from the object to see whether or not the objected changed.
#' @param x The object.
#' @return A list containing the object and its hash attribute.
#' @family hash functions for objects
#' @export
un_hash <- function(x) {
    hash <- attr(x, "hash")
    attr(x, "hash") <- NULL
    res <- list(object = x, hash = hash)
    return(res)
}

is_german_csv <- function(file) {
    l <- readLines(file, n = 1)
    n_semicolae <- length(unlist(strsplit(l, split = ";")))
    n_commatae <- length(unlist(strsplit(l, split = ",")))
    is_german_csv <- n_semicolae >= n_commatae
    return(is_german_csv)
}

is_missing_cell11 <- function(file, quote = "\"") {
    is_german_csv <- is_german_csv(file)
    if (is_german_csv) {
        sep <- ";"
    } else {
        sep <- ","
    }
    l <- readLines(file, n = 1)
    res <- identical(unlist(strsplit(l, split = sep, useBytes = TRUE))[1],
                     paste0(quote, quote))
    return(res)
}

#' Read and Write a Comma Separated File
#'
#' Functions to read and write CSV files. The objects returned by these
#' functions are \code{\link{data.frame}s} with the following attributes:
#' \describe{
#'   \item{path}{The path to the file on disk.}
#'   \item{csv}{The type of CSV: either \code{standard} or \code{german}.}
#'   \item{hash}{The hash value computed with \pkg{digest}'s digest function, if
#'   \pkg{digest} is installed.}
#' }
#' @name csv
#' @param file The path to the file to be read or written.
#' @family CSV functions
NULL

#' @description
#' \code{read_csv} is a wrapper to determine whether to use
#' \code{\link[utils:read.csv2]{utils:read.csv2}} or
#' \code{\link[utils:read.csv]{utils:read.csv}}.
#' It sets the above three arguments.
#' @rdname csv
#' @aliases read_csv
#' @param ... Arguments passed to \code{\link[utils:read.csv]{utils::read.csv}}
#' or \code{\link[utils:read.csv2]{utils::read.csv2}}.
#' @return For \bold{\code{read_csv}}: An object read from the file.
#' @export
#' @examples
#' # read from standard CSV
#' f <- tempfile()
#' write.csv(mtcars, file = f)
#' str(read_csv(f))
#' f <- tempfile()
#' write.csv2(mtcars, file = f)
#' str(read_csv(f))
read_csv <- function(file, ...) {
    arguments <- list(...)
    if (!is.null(arguments[["quote"]])) {
        quote <- arguments[["quote"]]
    } else {
        quote <- "\""
    }
    if (!is_false(arguments[["header"]])) {
        has_header <- TRUE
        if (!("row.names" %in% names(arguments)) &&
            is_missing_cell11(file, quote = quote)) {
            # header not set to false and no rownames given and cell[1,1] empty
            # string
            arguments[["row.names"]] <- 1
        }
    } else {
        has_header <- FALSE
    }
    if (is.null(arguments[["row.names"]])) {
        if (has_header && is_missing_cell11(file, quote = quote)) {
            has_rownames <- TRUE
        } else {
            has_rownames <- FALSE
        }
    } else {
        has_rownames <- !is_false(arguments[["row.names"]])
    }
    if (is_german_csv(file)) {
        x <- do.call(utils::read.csv2, as.list(c(file = file, arguments)))
        assert_read_successfully(x, file, has_header, has_rownames, sep = ";")
        attr(x, "csv") <- "german"
    } else {
        x <- do.call(utils::read.csv, as.list(c(file = file, arguments)))
        assert_read_successfully(x, file, has_header, has_rownames, sep = ",")
        attr(x, "csv") <- "standard"
    }
    x <- set_path(x, path = file, action = "read")
    if (has_digest()) x <- set_hash(x)
    return(x)
}

#' @description
#' \code{write_csv} compares the \code{hash} value stored in the object's
#' attribute
#' with the objects current hash value. If they differ, it writes the object to
#' the \code{file} argument or, if not given, to the \code{path}
#' stored in the object's attribute. If no \code{csv_type} is given, it uses
#' the \code{csv} type stored in object's attribute.
#' If \pkg{digest} is not installed, the object will (unconditionally) be
#' written to disk.
#' @rdname csv
#' @aliases write_csv
#' @param x The object to write to disk.
#' @param csv_type Which \command{csv} type is to be used. If \code{NA}, the
#' \code{csv} attribute is read from the object.
#' @return For \bold{\code{write_csv}}: The object with updated \code{hash}
#' (and possibly \code{path} and \code{csv})
#' attribute.
#' @export
#' @examples
#' # write to standard CSV
#' f <- tempfile()
#' d <- mtcars
#' str(d <- write_csv(d, file = f))
#' file.mtime(f)
#' Sys.sleep(2) # make sure the mtime would have changed
#' write_csv(d, file = f)
#' file.mtime(f)
write_csv <- function(x, file = NULL,
                      csv_type = c(NA, "standard", "german")) {
    tmp <- un_hash(x)
    if (is.null(file)) {
        file <- get_path(x) # do not strip attributes, we need them!
    }
    csv <- match.arg(csv_type)
    if (is.na(csv)) {
        csv <- attr(x, "csv")
        if (is.null(csv)) {
            csv <- "standard"
            message("Set csv type to standard!")
        }
    }
    if (is.null(tmp[["hash"]]) ||
        !identical(csv, attr(x, "csv")) ||
        !has_digest() ||
        !identical(digest::digest(tmp[["object"]]), tmp[["hash"]])) {
        f <- switch(csv,
                    "standard" = utils::write.csv,
                    "german" = utils::write.csv2,
                    throw("Give a csv_type."))
        do.call(f, list(file = file, x = x))
    }
    attr(x, "csv") <- csv
    x <- set_path(x, file, action = "write", overwrite = TRUE)
    if (has_digest()) {
        res <- set_hash(x)
    } else {
        res <- x
    }
    return(invisible(res))
}

#' Bulk Read Comma Separated Files
#'
#' Import a bunch of comma separated files or
#' all comma separated files below a directory using
#' \code{\link{read_csv}}.
#' @param is_latin1 Are the files encoded in "Latin1"?
#' @param stop_on_error Stop if any of the files is not read? Warn and continue
#' otherwise.
#' @param paths A vector of file paths or the directory to find files.
#' @param pattern see \code{\link{find_files}}.
#' Ignored, if \code{paths} is not a directory.
#' @param all_files see \code{\link{find_files}}.
#' Ignored, if \code{paths} is not a directory.
#' @param recursive see \code{\link{find_files}}.
#' Ignored, if \code{paths} is not a directory.
#' @param ignore_case see \code{\link{find_files}}.
#' Ignored, if \code{paths} is not a directory.
#' @param find_all see \code{\link{find_files}}.
#' Ignored, if \code{paths} is not a directory.
#' @param select see \code{\link{find_files}}.
#' Ignored, if \code{paths} is not a directory.
#' @param ... Arguments passed to \code{\link{read_csv}}.
#' @return A named list, each element holding the contents of one \command{csv}
#' file read by \code{\link{read_csv}}.
#' @family CSV functions
#' @export
#' @examples
#' unlink(dir(tempdir(), full.names = TRUE))
#' data(mtcars)
#' mt_german <- mtcars
#' rownames(mt_german)[1] <- "Mazda R\u00f64"
#' names(mt_german)[1] <- "mg\u00dc"
#' #% read from directory
#' for (i in 1:10) {
#'     f <- file.path(tempdir(), paste0("f", i, ".csv"))
#'     write.csv(mtcars[1:5, TRUE], file = f)
#'     f <- file.path(tempdir(), paste0("f", i, "_german.csv"))
#'     write.csv2(mt_german[1:7, TRUE], file = f, fileEncoding = "Latin1")
#' }
#' bulk <- bulk_read_csv(tempdir())
#'
#' #% pass a path
#' f <- list.files(tempdir(), pattern = ".*\\.csv$", full.names = TRUE)[1]
#' bulk <- bulk_read_csv(f)
#'
#' #% pass multiple path
#' f <- list.files(tempdir(), pattern = ".*\\.csv$", full.names = TRUE)[2:4]
#' bulk <- bulk_read_csv(f)
bulk_read_csv <- function(paths,
                          stop_on_error = FALSE,
                          is_latin1 = TRUE,
                          pattern = ".*\\.csv$", all_files = TRUE,
                          recursive = FALSE, ignore_case = FALSE,
                          find_all = FALSE, select = NA,
                          ...) {
    if (identical(length(paths), 1L) && dir.exists(paths)) {
        paths <- find_files(path = paths, pattern = pattern,
                            all_files = all_files, recursive = recursive,
                            ignore_case = ignore_case, find_all = find_all,
                            select = select)
    }
    res <- list()
    for (infile in paths) {
        name <- sub("\\.csv$", "", ignore.case = TRUE, basename(infile))
        res[[name]] <- tryCatch({
            if (isTRUE(is_latin1)) {
                tmp <- read_csv(infile, fileEncoding = "Latin1", ...)
                tmp <- convert_umlauts_to_ascii(tmp)
                if (has_digest()) {
                    tmp <- set_hash(tmp)
                }
            } else {
                tmp <- read_csv(infile, ...)
            }
        },
        error = identity)
        if (inherits(res[[name]], "error")) {
            if (isTRUE(stop_on_error)) {
                throw("Reading file failed.", res[[name]])
            } else {
                warning("Reading of file ", infile, " failed.")
            }
        }
    }
    return(res)
}

#' Bulk Write Comma Separated Files
#'
#' Write a bunch of objects to disk using \code{\link{write_csv}}.
#' @param x A list of objects to be written to \command{csv}.
#' @param ... Arguments passed to
#' \code{\link{write_csv}}.
#' @return The list holding the return values of \code{\link{write_csv}}.
#' @family CSV functions
#' @export
#' @examples
#' unlink(dir(tempdir(), full.names = TRUE))
#' data(mtcars)
#' mt_german <- mtcars
#' rownames(mt_german)[1] <- "Mazda R\u00f64"
#' names(mt_german)[1] <- "mg\u00dc"
#' for (i in 1:10) {
#'     f <- file.path(tempdir(), paste0("f", i, ".csv"))
#'     write.csv(mtcars[1:5, TRUE], file = f)
#'     f <- file.path(tempdir(), paste0("f", i, "_german.csv"))
#'     write.csv2(mt_german[1:7, TRUE], file = f, fileEncoding = "Latin1")
#' }
#' #% read
#' bulk <- bulk_read_csv(tempdir())
#'
#' print(mtime <- file.info(list.files(tempdir(), full.names = TRUE))["mtime"])
#' bulk[["f2"]][3, 5] <- bulk[["f2"]][3, 5] + 2
#' Sys.sleep(2) # make sure the mtimes would change
#' result <- bulk_write_csv(bulk)
#' print(new_times <- file.info(dir(tempdir(), full.names = TRUE))["mtime"])
#' index_change <- grep("f2\\.csv", rownames(mtime))
#' if (requireNamespace("digest", quietly = TRUE)) {
#'     only_f2_changed <- all((mtime == new_times)[-c(index_change)]) &&
#'         (mtime < new_times)[c(index_change)]
#'     RUnit::checkTrue(only_f2_changed)
#' } else {
#'     RUnit::checkTrue(all(mtime < new_times))
#' }
bulk_write_csv <- function(x, ...) {
    res <- lapply(x, write_csv, ...)
    return(invisible(res))
}

#' Check the Number of Lines and Fields in a File
#'
#'
#' @param path Path to a file.
#' @param sep A character separating the fields in the file.
#' @return A list giving the number of lines, number of fields and an boolean
#' indicating whether all lines have the same number of fields.
#' @family CSV functions
#' @export
#' @examples
#' f <- tempfile()
#' write.csv2(mtcars, file = f)
#' check_ascii_file(f)
check_ascii_file <- function(path, sep = ";") {
    lines <- readLines(path)
    num_lines <- length(lines)
    num_fields <- vapply(X = lines, FUN.VALUE = numeric(1), USE.NAMES = FALSE,
                         function(x) length(unlist(strsplit(x, split = sep)))
                         )
    max_fields <- max(num_fields)
    res <- list(number_of_lines = num_lines,
                number_of_fields = max_fields,
                is_all_lines_all_fields =  identical(max_fields,
                                                     min(num_fields))
                )
    return(res)
}

is_read_successfully <- function(x, file, is_header, is_rownames, sep) {
    nrow <- as.integer(check_ascii_file(file, sep)[["number_of_lines"]] -
                       as.numeric(is_header))
    ncol <- as.integer(check_ascii_file(file, sep)[["number_of_fields"]] -
                       as.numeric(is_rownames))
    is_identical <- identical(nrow, nrow(x)) && identical(ncol, ncol(x))
    return(is_identical)
}

assert_read_successfully <- function(x, file, is_header, is_rownames, sep) {
    if (!is_read_successfully(x, file, is_header, is_rownames, sep)) {
        warning("You might want to change argument ",
                "(to function utils::read.csv or fritools2::read_csv) ",
                "`quote` to \"\".")
        throw(paste0("File ", file, " not read successfully!"))
    }
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
} else {
  has_digest <- fritools2:::has_digest
}




unlink(dir(tempdir(), full.names = TRUE))
cars <- mtcars[1:2, TRUE]
f <- file.path(tempdir(), paste0("cars_german", ".csv"))
utils::write.csv2(cars, file = f)
l <- readLines(f, n = 1)
n_semicolae <- length(unlist(strsplit(l, split = ";")))
n_commatae <- length(unlist(strsplit(l, split = ",")))
expect_true(n_semicolae >= n_commatae)
res <- csv2csv(f, row.names = 1)
expect_true(all.equal(
  strip_off_attributes(res),
  strip_off_attributes(cars)
))
l <- readLines(f, n = 1)
n_semicolae <- length(unlist(strsplit(l, split = ";")))
n_commatae <- length(unlist(strsplit(l, split = ",")))
expect_true(n_semicolae < n_commatae)







# % provide data
unlink(dir(tempdir(), full.names = TRUE))
f <- file.path(tempdir(), paste0("a", ".csv"))
cars <- mtcars[1:2, TRUE]
a <- write_csv(cars, file = f)
expect_true(file.exists(f))
if (fritools2::is_running_on_fvafrcu_machines()) {
  expect_identical(strip_off_attributes(get_path(a)), f)
}

mtime <- file.info(f)[["mtime"]]
Sys.sleep(1)

# % read file, setup expectation
a <- read_csv(f)
expectation <-
  structure(list(
    mpg = c(21L, 21L), cyl = c(6L, 6L),
    disp = c(160L, 160L), hp = c(110L, 110L),
    drat = c(3.9, 3.9), wt = c(2.62, 2.875),
    qsec = c(16.46, 17.02), vs = c(0L, 0L), am = c(1L, 1L),
    gear = c(4L, 4L), carb = c(4L, 4L)
  ),
  class = "data.frame",
  row.names = c("Mazda RX4", "Mazda RX4 Wag"),
  csv = "standard",
  path = structure("/tmp/RtmpGyKumR/a.csv",
    mtime = structure(1638788464.61958,
      class = c(
        "POSIXct",
        "POSIXt"
      )
    ),
    last_read = structure(1639122705.98385,
      class = c(
        "POSIXct",
        "POSIXt"
      )
    ),
    last_written = NA
  ),
  hash = "062a5b662192887791002c72fd196426"
  )
if (!has_digest()) {
  expectation <- un_hash(expectation)[["object"]]
}

# % write
result <- write_csv(a)
## % check return
expect_identical(
  strip_off_attributes(expectation),
  strip_off_attributes(result)
)

## % no writing with digest
if (has_digest()) {
  expect_identical(mtime, file.info(f)[["mtime"]])
} else {
  expect_true(mtime < file.info(f)[["mtime"]])
}

## % modify data and write to disk
a[1, 3] <- 300
write_csv(a)

## % on a: hash not updated, still writing:
expect_true(mtime < file.info(f)[["mtime"]])
## % on result: hash already updated, not writing with digest:
mtime <- file.info(f)[["mtime"]]
Sys.sleep(1)
write_csv(result)
if (has_digest()) {
  expect_identical(mtime, file.info(f)[["mtime"]])
} else {
  expect_true(mtime < file.info(f)[["mtime"]])
}
if (has_digest()) {
  ## % We update the hash value:
  a <- set_hash(a)
  write_csv(a)
  expect_identical(mtime, file.info(f)[["mtime"]])
}







# % provide data
unlink(dir(tempdir(), full.names = TRUE))
data(mtcars)
mt_german <- mtcars
rownames(mt_german)[1] <- "Mazda R\u00f64"
names(mt_german)[1] <- "mg\u00dc"
for (i in 1:10) {
  f <- file.path(tempdir(), paste0("f", i, ".csv"))
  write.csv(mtcars[1:5, TRUE], file = f)
  f <- file.path(tempdir(), paste0("f", i, "_german.csv"))
  write.csv2(mt_german[1:7, TRUE], file = f, fileEncoding = "Latin1")
}
# % pass a path
f <- list.files(tempdir(), pattern = ".*\\.csv$", full.names = TRUE)[1]
bulk <- bulk_read_csv(f)
expect_identical(length(bulk), 1L)

# % pass multiple path
f <- list.files(tempdir(), pattern = ".*\\.csv$", full.names = TRUE)[2:4]
bulk <- bulk_read_csv(f)
expect_identical(length(bulk), 3L)

# % read
bulk <- bulk_read_csv(tempdir())
path <- structure("/tmp/RtmpGyKumR/f1.csv",
  mtime = structure(1638788464.61958,
    class = c("POSIXct", "POSIXt")
  ),
  last_read = structure(1639122705.98385,
    class = c("POSIXct", "POSIXt")
  ),
  last_written = NA
)
expectation <- structure(list(
  mpg = c(21, 21, 22.8, 21.4, 18.7),
  cyl = c(6L, 6L, 4L, 6L, 8L),
  disp = c(160L, 160L, 108L, 258L, 360L),
  hp = c(110L, 110L, 93L, 110L, 175L),
  drat = c(3.9, 3.9, 3.85, 3.08, 3.15),
  wt = c(2.62, 2.875, 2.32, 3.215, 3.44),
  qsec = c(16.46, 17.02, 18.61, 19.44, 17.02),
  vs = c(0L, 0L, 1L, 1L, 0L),
  am = c(1L, 1L, 1L, 0L, 0L),
  gear = c(4L, 4L, 4L, 3L, 3L),
  carb = c(4L, 4L, 1L, 1L, 2L)
),
class = "data.frame",
row.names = c(
  "Mazda RX4", "Mazda RX4 Wag",
  "Datsun 710", "Hornet 4 Drive",
  "Hornet Sportabout"
),
csv = "standard",
path = path,
hash = "9f8cabc0eaa06329b42d697642497a29"
)
if (!has_digest()) {
  expectation <- un_hash(expectation)[["object"]]
}
expect_identical(
  strip_off_attributes(bulk[["f1"]]),
  strip_off_attributes(expectation)
)

# % write
result <- bulk_write_csv(bulk)
expectation <- bulk
expect_identical(
  lapply(result, strip_off_attributes),
  lapply(expectation, strip_off_attributes)
)
dmtime <- file.info(list.files(tempdir(),
  full.names = TRUE,
  pattern = ".*\\.csv"
))["mtime"]
mtime <- lapply(bulk, get_mtime)
mtime <- as.data.frame(do.call(c, mtime))
expect_true(all(mtime == dmtime))

Sys.sleep(1)

bulk[["f2"]][3, 5] <- bulk[["f2"]][3, 5] + 2
result <- bulk_write_csv(bulk)
new_times <- file.info(dir(tempdir(), full.names = TRUE))["mtime"]
index_change <- which(rownames(mtime) == "f2")
if (has_digest()) {
  only_f2_changed <- all((dmtime == new_times)[-c(index_change)]) &&
    (dmtime < new_times)[c(index_change)]
  expect_true(only_f2_changed)
} else {
  expect_true(all(mtime < new_times))
}

# When a file is not read.
file.remove(f[1])
bulk <- bulk_read_csv(f)
expect_true(inherits(bulk[[1]], "error"))
expect_error(bulk_read_csv(f, stop_on_error = TRUE))

#' Is a Version Requirement Met?
#'
#' Just a wrapper to \code{\link{compareVersion}}, I regularly forget how to use
#' it.
#' @param installed The version available.
#' @param required The version required.
#' @return  \code{\link{TRUE}}, if so,  \code{\link{FALSE}} otherwise.
#' @family logical helpers
#' @family package functions
#' @family version functions
#' @export
#' @examples
#' is_version_sufficient(installed = "1.0.0", required = "2.0.0")
#' is_version_sufficient(installed = "1.0.0", required = "1.0.0")
#' is_version_sufficient(installed = get_package_version("base"),
#'                       required = "3.5.2")
is_version_sufficient <- function(installed, required) {
    is_sufficient <- utils::compareVersion(installed, required) >= 0
    return(is_sufficient)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


result <- is_version_sufficient(installed = "1.0.0", required = "2.0.0")
expect_identical(result, FALSE)
result <- is_version_sufficient(installed = "1.0.0", required = "1.0.0")
expect_identical(result, TRUE)
result <- is_version_sufficient(installed = "1.0.1", required = "1.0.0")
expect_identical(result, TRUE)

#' Find Files on Disk
#'
#' Look for files on disk, either scanning a vector of names or searching for
#' files with \code{\link{list.files}} and throw an error if no files are found.
#'
#' This is a wrapper to either \code{\link{file.exists}} or
#' \code{\link{list.files}}, that ensures that (some) files exists. This may
#' come handy if you want to perform some kind of file manipulation e.g. with
#' one of the functions listed under \cr
#' \strong{See Also} \emph{Other file utilities:}.
#' @param file_names character vector of file names (to be checked if the files
#'        exist).
#' @param select A named list of numerical vectors of maximum length 2 named
#' \code{min} and/or \code{max}.
#' If given, file searching will be restricted to file attributes corresponding
#' to the names in the list ranging between \code{min} and \code{max}. See
#' \emph{examples}.
#' @param path see \code{\link{list.files}}.
#' @param pattern see \code{\link{list.files}}.
#' @param all_files see \code{\link{list.files}}, argument \code{all.files}.
#' @param recursive see \code{\link{list.files}}.
#' @param ignore_case see \code{\link{list.files}}, argument \code{ignore.case}.
#' @param find_all Throw an error if not all files (given by \emph{file_names})
#' are found?
#'
#' @note This is merely a wrapper around \code{\link{file.exists}} or
#'       \code{\link{list.files}}, depending on whether \emph{file_names} is
#'       given.
#'
#' @export
#' @family searching functions
#' @family file utilities
#' @return A character vector of file names.
#' @examples
#' #% create some files
#' files <- unname(sapply(file.path(tempdir(), paste0(sample(letters, 10),
#'                                               ".", c("R", "Rnw", "txt"))),
#'                        touch))
#' print(files)
#' print(list.files(tempdir(), full.names = TRUE)) # same as above
#' #% file names given
#' find_files(file_names = files[1:3])
#' ##% some do not exist:
#' find_files(file_names = c(files[1:3], replicate(2, tempfile())))
#' try(find_files(file_names = c(files[1:3], replicate(2, tempfile())),
#'                find_all = TRUE))
#' ##% all do not exist:
#' try(find_files(file_names = replicate(2, tempfile())))
#' #% path given
#' find_files(path = tempdir())
#' ##% change pattern
#' find_files(path = tempdir(),
#'            pattern = ".*\\.[RrSs]$|.*\\.[RrSs]nw$|.*\\.txt")
#' ##% find a specific file by it's basename
#' find_files(path = tempdir(), pattern = paste0("^", basename(files[1]), "$"))
#' #% file_names and path given: file_names beats path
#' try(find_files(file_names = tempfile(), path = tempdir()))
#' #% select by file size:
#' write.csv(mtcars, file.path(tempdir(), "mtcars.csv"))
#' find_files(path = tempdir(), pattern = ".*")
#' find_files(path = tempdir(), pattern = ".*",
#'            select = list(size = c(min = 1000))
#'            )
find_files <- function(path = ".", pattern = ".*\\.[RrSs]$|.*\\.[RrSs]nw$",
                       file_names = NA, all_files = TRUE, recursive = FALSE,
                       ignore_case = FALSE, find_all = FALSE, select = NA) {
    if (isTRUE(is.na(file_names))) {
        file_paths <- list.files(path = path, pattern = pattern,
                                  all.files = all_files,
                                  recursive = recursive,
                                  ignore.case = ignore_case,
                                  full.names = TRUE,
                                  include.dirs = FALSE, no.. = TRUE)
        if (!all(sapply(select, is.na))) {
            selection <- lapply(select,
                                function(x) {
                                    if (!"min" %in% names(x)) x["min"] <- -Inf
                                    if (!"max" %in% names(x)) x["max"] <- Inf
                                    return(x)
                                })
            fi <- file.info(file_paths)
            index <- rep(TRUE, length(file_paths))
            for (i in seq(along = selection)) {
                col <- names(selection[i])
                index <- fi[[col]] >= selection[[i]][["min"]] &
                    fi[[col]] <= selection[[i]][["max"]] & index

            }
            file_paths <- row.names(fi)[index]
        }
    } else {
        files_exist <- file.exists(file_names)
        msg <- paste(paste("File", file_names[! files_exist], " not found."),
                      collapse = "\n\t")

        if (! all(files_exist)) {
            if (isTRUE(find_all)) {
                throw(msg)
            } else {
                warning(msg)
                file_paths <- file_names[files_exist]
            }
        } else {
            file_paths <- file_names[files_exist]
        }

    }
    if (length(file_paths) == 0) throw("No matching files found.")
    return(sort(file_paths))
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


# % create some files
temp_dir <- tempfile()
if (interactive()) unlink(temp_dir, recursive = TRUE)
path <- file.path(temp_dir, "foo")
dir.create(path, recursive = TRUE)
files <- sort(unname(sapply(
  file.path(
    path, # Exclude Linting false positive
    paste0(
      sample(letters, 10),
      ".", c("R", "Rnw", "txt")
    )
  ),
  touch
)))
result <- files
expectation <- list.files(path, full.names = TRUE)
expect_identical(result, expectation)
# % file names given
expectation <- files[1:3]
result <- find_files(file_names = files[1:3])
expect_identical(result, expectation)
## % some do not exist:
not_there <- replicate(2, tempfile())
result <- suppressWarnings(find_files(file_names = c(
  files[1:3],
  not_there
)))
expect_identical(result, expectation)
### % check for warning
result <- tryCatch(find_files(file_names = c(files[1:3], not_there)),
  warning = function(w) {
    return(w)
  }
)[["message"]]
expectation <- paste(paste("File", not_there, " not found."),
  collapse = "\n\t"
)
expect_identical(result, expectation)

### % check for find_all
expect_error(find_files(
  file_names = c(files[1:3], not_there),
  find_all = TRUE
))
## % all do not exist:
expect_error(find_files(file_names = replicate(2, tempfile())))
# % path given
result <- find_files(path = temp_dir, recursive = TRUE,
                     pattern = "\\.R$|\\.Rnw$")
expectation <- grep("\\.R$|\\.Rnw$", files, value = TRUE)
expect_identical(result, expectation)
## % none found (by not searching recursively):
expect_error(find_files(path = temp_dir, recursive = FALSE,
                        pattern = "\\.R$|\\.Rnw$"))
## % change pattern
result <- find_files(
  path = temp_dir,
  pattern = ".*\\.[RrSs]$|.*\\.[RrSs]nw$|.*\\.txt",
  recursive = TRUE
)
expectation <- files
expect_identical(result, expectation)
## % find a specific file by it's basename
result <- find_files(path = path, pattern = paste0(
  "^",
  basename(files[1]), "$"
))
expectation <- files[1]
expect_identical(result, expectation)
# % file_names and path given: file_names beats path
result <- find_files(file_names = files[1], path = temp_dir)
expectation <- files[1]
expect_identical(result, expectation)






# % create some files
temp_dir <- tempfile()
if (interactive()) unlink(temp_dir, recursive = TRUE)
path <- file.path(temp_dir, "foo")
dir.create(path, recursive = TRUE)
files <- sort(unname(sapply(
  file.path(
    path, # Exclude Linting false positive
    paste0(
      sample(letters, 10),
      ".", c("R", "Rnw", "txt")
    )
  ),
  touch
)))
write.csv(mtcars, file.path(path, "mtcars.csv"))
find_files(path = path, pattern = ".*")
result <- find_files(
  path = path, pattern = ".*",
  select = list(size = c(min = 1000))
)
expectation <- list.files(path, full.names = TRUE, pattern = "mtcars")
expect_identical(result, expectation)

#' Copy a Path from Clipboard to \R
#'
#' I often have to work under Windows, where file paths cannot just be pasted
#' into the code, so I adapted code from
#' \url{https://www.r-bloggers.com/2015/12/stop-fiddling-
#' around-with-copied-paths-in-windows-r/}.
#' Under Windows, the de-windowsified path is copied to the clipboard.
#' @note It makes only sense to call \code{clipboard_path} in an interactive \R
#' session.
#' @return The de-windowsified path.
#' @export
#' @family operating system functions
#' @family file utilities
clipboard_path <- function() {
    if (interactive()) {
        scanned <- scan(file = "clipboard", what = "")
        if (is_windows()) {
            items <- unlist(strsplit(scanned, split = "\\\\"))
            value <- paste0("file.path(\"",
                            paste(items, collapse = "\", \""),
                            "\")")
            utils::writeClipboard(value)
            message("Copied `", value, "` to clipboard.")
        } else {
            items <- unlist(strsplit(scanned, split = "/"))
            value <- paste0("file.path('",
                            paste(items, collapse = "', '"),
                            "')")
            message(gsub("'", "\"", value))
        }
        res <-  do.call("file.path", as.list(items))
        if (!file.exists(res))
            warning("Are you sure you copied a path to the clipboard?",
                    "\nGot ", scanned)
        return(res)
    } else {
        throw("clipboard_path() needs an interactive R session.")
    }
}

#' Strip Attributes off an Object
#'
#' @param x An object.
#' @return The object.
#' @family bits and pieces
#' @seealso \code{\link[base:unname]{base::unname}}
#' @export
#' @examples
#' y <- stats::setNames(1:3, letters[1:3])
#' attr(y, "myattr") <- "qwer"
#' comment(y) <- "qwer"
#' strip_off_attributes(y)
strip_off_attributes <- function(x) {
    attributes(x) <- NULL
    return(x)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


x <- 1:3
y <- stats::setNames(x, letters[1:3])
attr(y, "myattr") <- "qwer"
comment(y) <- "qwer"
expect_identical(x, strip_off_attributes(y))

#' Remove All Objects From an Environment
#'
#' Wipe an environment, typically \code{\link{.GlobalEnv}}, clean.
#' @param environment The environment that should be wiped clean.
#' Defaults to \code{\link{.GlobalEnv}}.
#' @param all_names See argument \code{all.names} for \code{\link{ls}}.
#' @return A character vector containing the names of objects removed, but
#' called for its side effect of removing all objects from the environment.
#' @export
#' @family R memory functions
#' @examples
#' an_object <- 1
#' wipe_clean()
#' ls()
#' e <- new.env()
#' assign("a", 1, envir = e)
#' assign("b", 1, envir = e)
#' ls(envir = e)
#' wipe_clean(envir = e)
#' ls(envir = e)
#' RUnit::checkIdentical(length(ls(envir = e)), 0L)
wipe_clean <- function(environment = getOption("wipe_clean_environment"),
                       all_names = TRUE) {
    if (!is.null(environment)) {
    objects <- ls(name = environment, all.names = all_names)
    rm(list = objects, envir = environment)
    } else {
        throw(paste("Argument `environment` not given and option;",
                    "`wipe_clean_environment` not set!"))
    }
    return(invisible(objects))
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


e <- new.env()
assign("a", 1, envir = e)
expect_identical(ls(envir = e), "a")
result <- wipe_clean(envir = e)
expectation <- "a"
expect_identical(result, expectation)
expect_identical(length(ls(envir = e)), 0L)

#' Query Installed Package Version
#'
#' \code{\link{packageVersion}} converts to class \code{\link{package_version}},
#' which then again would need to be converted for \code{\link{compareVersion}}.
#' So this is a modified copy of \code{\link{packageVersion}} skipping the
#' conversion to \code{\link{package_version}}.
#'
#' @param x A character giving the package name.
#' @param lib_loc See argument \code{lib.loc} in
#' \code{\link{packageDescription}}.
#' @return  A character giving the package version.
#' @family version functions
#' @family package functions
#' @export
#' @examples
#' get_package_version("base")
#' try(get_package_version("mgcv"))
#' utils::compareVersion("1000.0.0", get_package_version("base"))
#' utils::compareVersion("1.0", get_package_version("base"))
#' # from ?is_version_sufficient:
#' is_version_sufficient(installed = get_package_version("base"),
#'                       required = "1.0")
get_package_version <- function(x, lib_loc = NULL) {
    version <- suppressWarnings(utils::packageDescription(x, lib.loc = lib_loc,
                                                          fields = "Version"))
    if (is.na(version)) stop(packageNotFoundError(x, lib_loc, sys.call()))
    return(version)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


result <- call_conditionally(get_package_version,
  condition = TRUE,
  args = list(x = "fritools2"),
  fallback = "0.0"
)
expectation <- get_package_version("fritools2")
expect_identical(result, expectation)
call_conditionally(get_package_version,
  condition = FALSE,
  args = list(x = "fritools2"),
  fallback = "0.0"
)
result <- call_conditionally(get_package_version,
  condition = FALSE,
  args = list(x = "fritools2"),
  fallback = "0.0"
)
expectation <- "0.0"
expect_identical(result, expectation)
result <- call_conditionally(get_package_version,
  condition = TRUE,
  args = list(x = "not_there"),
  harden = TRUE,
  fallback = "-1"
)
expectation <- "-1"
expect_identical(result, expectation)

expect_error(call_conditionally(get_package_version,
  condition = TRUE,
  args = list(x = "not_there"),
  harden = FALSE,
  fallback = "-1"
))

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


v <- get_package_version("base")
expect_true(is(package_version(v), "package_version"))
expect_error(get_package_version("no_package"))

#' Is the System Running a Windows Machine?
#'
#' Is the system running a windows machine?
#' @return \code{\link{TRUE}} if so, \code{\link{FALSE}} otherwise.
#' @family logical helpers
#' @family operating system functions
#' @export
#' @examples
#' is_windows()
is_windows <- function() return(.Platform[["OS.type"]] == "windows")

#' Execute Code in a Temporary Working Directory
#'
#' This is a verbatim copy of \code{withr::with_dir} from of \pkg{withr}'s
#' version 2.4.1.
#' I often need \pkg{withr} only to import \code{withr::with_dir}, which is a
#' really simple function. So I just hijack \code{withr::with_dir}.
#' @param new The new working directory.
#' @param code Code to execute in the temporary working directory.
#' @return The results of the evaluation of the \code{code} argument.
#' @family operating system functions
#' @export
#' @examples
#' temp_dir <- file.path(tempfile())
#' dir.create(temp_dir)
#' with_dir(temp_dir, getwd())
with_dir <- function(new, code) {
    old <- setwd(dir = new)
    on.exit(setwd(old))
    force(code)
}

#' Call a Function Conditionally
#'
#' \pkg{whoami} 1.3.0 uses things like
#' \code{system("getent passwd $(whoami)", intern = TRUE)}
#' which I can not \code{\link{tryCatch}}, as it gives no error nor warning.
#' So this function returns a fallback if the condition given is not
#' \code{\link{TRUE}}.
#'
#' @param f The function passed to \code{\link{do.call}}.
#' @param ... arguments passed to \code{\link{do.call}}.
#' @param condition An expression.
#' @param fallback See \emph{Description}.
#' @param harden Set to \code{\link{TRUE}} to return \code{fallback} if
#' \code{\link{do.call}} fails.
#' @return The return value of \code{f} or \code{fallback}.
#' @family call functions
#' @export
#' @examples
#' call_conditionally(get_package_version,
#'                    condition = TRUE,
#'                    args = list(x = "fritools2"),
#'                    fallback = "0.0")
#' call_conditionally(get_package_version,
#'                    condition = FALSE,
#'                    args = list(x = "fritools2"),
#'                    fallback = "0.0")
#' call_conditionally(get_package_version,
#'                    condition = TRUE,
#'                    args = list(x = "not_there"),
#'                    harden = TRUE,
#'                    fallback = "0.0")
call_conditionally <- function(f, condition, fallback, ..., harden = FALSE) {
    res <- if (isTRUE(condition)) {
        if (isTRUE(harden)) {
            tryCatch(do.call(what = f, ...),
                     error = function(e) return(fallback),
                     warning = function(w) return(fallback))
        } else {
            do.call(what = f, ...)
        }
    } else {
        fallback
    }
    return(res)
}

#' Call a Function Given an External Dependency on Non-Windows Systems
#'
#' Just a specialized version of \code{\link{call_conditionally}}.
#'
#' @inheritParams call_conditionally
#' @param dependency The external dependency, see \emph{Examples}.
#' @return The return value of \code{f} or \code{fallback}.
#' @family call functions
#' @export
#' @examples
#' call_safe(whoami::email_address, dependency = "whoami",
#'           args = list(fallback = "foobar@nowhere.com"),
#'           fallback = "nobar@nowhere.com")
#' call_safe(whoami::email_address, dependency = "this_is_not_installed",
#'           args = list(fallback = "foobar@nowhere.com"),
#'           fallback = "nobar@nowhere.com")
call_safe <- function(f, dependency, fallback = "Fallback", ...) {
    call_conditionally(f = f,
                       condition = is_installed(dependency) || is_windows(),
                       fallback = fallback, ..., harden = TRUE)
}

#' Search Files for a Pattern
#'
#' This is an approximation of \kbd{unix} \command{find} and \command{grep}.
#' @param what A regex pattern for which to search.
#' @param verbose Be verbose?
#' @param exclude A regular expression for excluding files.
#' @param ... Arguments passed to \code{\link{list.files}}.
#' @export
#' @return \link[base:invisible]{Invisibly} a vector of names of files
#' containing the pattern given by \code{what}.
#' @family searching functions
#' @family file utilities
#' @examples
#' write.csv(mtcars, file.path(tempdir(), "mtcars.csv"))
#'  for (i in 0:9) {
#'      write.csv(iris, file.path(tempdir(), paste0("iris", i, ".csv")))
#'  }
#' search_files(what = "Mazda", path = tempdir(), pattern = "^.*\\.csv$")
#' search_files(what = "[Ss]etosa", path = tempdir(), pattern = "^.*\\.csv$")
#' x <- search_files(path = tempdir(),
#'                   pattern = "^.*\\.csv$",
#'                   exclude = "[2-9]\\.csv$",
#'                   what = "[Ss]etosa")
#' summary(x)
#' summary(x, type = "what")
#' summary(x, type = "matches")
#' try(search_files(what = "ABC", path = tempdir(), pattern = "^.*\\.csv$"))
search_files <- function(what, verbose = TRUE, exclude = NULL, ...) {
    files <- list.files(..., full.names = TRUE)
    if (! is.null(exclude))
        files <- grep(pattern = exclude, x = files, value = TRUE, invert = TRUE)
    res <- NULL
    for (file in files) {
        lines <- suppressWarnings(readLines(file))
        hits <- suppressWarnings(grepl(lines, pattern = what))
        matches <- suppressWarnings(grep(lines, pattern = what, value = TRUE))
        if (any(hits)) {
            if (isTRUE(verbose)) message("Found `", what, "` in file ", file)
            tmp <- cbind(file, what, matches)
            res <- rbind(res, tmp)
        }
    }
    if (is.null(res)) {
        throw(paste("Pattern", what, "not found."))
    } else {
        res <- as.data.frame(res)
        class(res) <- c("filesearch", class(res))
        return(invisible(res))
    }

}

#' Summarize File Searches
#'
#' A custom summary function for objects returned by \code{\link{search_files}}.
#' @param object An object returned by \code{\link{search_files}}.
#' @param type Type of summary.
#' @param ... Needed for compatibility.
#' @export
#' @return A summarized object.
#' @family searching functions
#' @inherit search_files examples
summary.filesearch <- function(object, ...,
                               type = c("file", "what", "matches")) {
    if (!inherits(object, "filesearch"))
        throw("object is not of class `filesearch`.")
    type <- match.arg(type)
    r <- switch(type,
           "file" = {
               unique(object[type])
           },
           "what" = {
               unique(object[TRUE, c("file", "what")])
           },
           "matches" = {
               object[TRUE, c("file", "matches")]
           })
    return(r)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}
if (fritools2::is_version_sufficient(
  fritools2::get_package_version("base"),
  "4.0.0"
)) {
  if (interactive()) pkgload::load_all(".")

  unlink(dir(tempdir(), full.names = TRUE))
  path <- tempdir()
  data(mtcars)
  data(iris)
  write.csv(mtcars, file.path(path, "mtcars.csv"))
  for (i in 0:9) {
    write.csv(iris, file.path(path, paste0("iris", i, ".csv")))
  }
  expect_error(search_files(
    what = "FOO", path = path,
    verbose = FALSE,
    pattern = ".*\\.csv$"
  ))
  expect_error(summary.filesearch("foo"))
  found <- search_files(
    what = "Mazda", path = path, verbose = FALSE,
    pattern = ".*\\.csv$"
  )
  result <- as.character(summary(found))
  expectation <- file.path(path, "mtcars.csv")
  expect_identical(result, expectation)
  search_pattern <- "5\\.3"
  found <- search_files(
    what = search_pattern, path = path,
    pattern = "iris.*\\.csv$",
    exclude = ".*[2-9]\\.csv",
    verbose = FALSE
  )
  result <- strip_off_attributes(unlist(summary(object = found)))
  expectation <- file.path(path, paste0("iris", 0:1, ".csv"))
  expect_identical(result, expectation)
  result <- strip_off_attributes(unlist(summary(
    object = found,
    type = "file"
  )))
  expectation <- file.path(path, paste0("iris", 0:1, ".csv"))
  expect_identical(result, expectation)
  result <- summary(object = found, type = "what")
  expectation <- data.frame(
    file = file.path(
      path,
      paste0("iris", 0:1, ".csv")
    ),
    what = search_pattern
  )
  expect_identical(
    strip_off_attributes(result),
    strip_off_attributes(expectation)
  )
  result <- summary(object = found, type = "matches")
  expectation <-
    structure(list(
      file = rep(file.path(path, paste0(
        "iris", 0:1,
        ".csv"
      )),
      each = 3
      ),
      matches = c(
        "\"49\",5.3,3.7,1.5,0.2,\"setosa\"",
        "\"112\",6.4,2.7,5.3,1.9,\"virginica\"",
        "\"116\",6.4,3.2,5.3,2.3,\"virginica\"",
        "\"49\",5.3,3.7,1.5,0.2,\"setosa\"",
        "\"112\",6.4,2.7,5.3,1.9,\"virginica\"",
        "\"116\",6.4,3.2,5.3,2.3,\"virginica\""
      )
    ),
    row.names = c(NA, 6L),
    class = c("filesearch", "data.frame")
    )
  expect_identical(result, expectation)
}
if (interactive()) {

}

#' Get the Path of the R Code File in Case of an \command{R CMD BATCH} Run
#'
#' Retrieve the path from parsing the command line arguments of a
#' \command{R CMD BATCH} run.
#' @export
#' @return A vector of \code{\link{mode}} character giving the name of the R
#' code file. Will be character(0) if not in an \command{R CMD BATCH} run.
#' @family script path getter functions
#' @examples
#' get_r_cmd_batch_script_path()
get_r_cmd_batch_script_path <- function() {
    r_call <- commandArgs(trailingOnly = FALSE)
    path <- r_call[which(r_call == "-f") + 1]
    return(path)
}

#' Get the Path of the R Code File in Case of an \command{Rscript} Run
#'
#' Retrieve the path from parsing the command line arguments of a
#' \command{Rscript} run.
#' @export
#' @return A vector of \code{\link{mode}} character giving the name of the R
#' code file. Will be character(0) if not in an \command{Rscript} run.
#' @family script path getter functions
#' @examples
#' get_rscript_script_path()
get_rscript_script_path <- function() {
    r_call <- commandArgs(trailingOnly = FALSE)
    path <- sub("--file=", "", r_call[grep("--file", r_call)])
    return(path)
}

#' Get the Path of the R Code File
#'
#' This is just a wrapper for \code{\link{get_rscript_script_path}} and
#' \code{\link{get_r_cmd_batch_script_path}}.
#' @export
#' @return A vector of \code{\link{length}} 1 and \code{\link{mode}}
#' character giving the name of the R code file if R was run via
#' \command{R CMD BATCH} or
#' \command{Rscript}.
#' @family script path getter functions
#' @examples
#' get_script_path()
get_script_path <- function() {
    path <- c(get_r_cmd_batch_script_path(), get_rscript_script_path())
    return(path)
}

#' Get the Name of the R Code File or set it to \code{default}
#'
#' The code file name is retrieved only for \command{R CMD BATCH} and
#' \command{Rscript},
#' if R is used interactively, the name is set to \code{default},
#' even if you're working with code stored in a (named) file on disk.
#' @param default the name to return if R is run interactively.
#' @export
#' @return A vector of \code{\link{length}} 1 and \code{\link{mode}}
#' character giving the name of the R code file if R was run via
#' \command{R CMD BATCH} or
#' \command{Rscript}, the given default otherwise.
#' @family script path getter functions
#' @examples
#' get_script_name(default = 'foobar.R')
get_script_name <- function(default = "interactive_R_session") {
    path <- get_script_path()
    if (as.logical(length(path))) {
        name <- basename(path)
    } else {
        name <- default
    }
    return(name)
}

#' Is R Run in Batch Mode (via \command{R CMD BATCH} or \command{Rscript})?
#'
#' Just a wrapper to \code{\link{interactive}}.
#' @export
#' @template return_boolean
#' @family logical helpers
#' @examples
#' is_batch()
is_batch <- function() {
    is_batch <- !interactive()
    return(is_batch)
}

get_r_binary <- function() {
    return(Sys.which(if (is_installed("R-devel")) "R-devel" else "R"))
}

r_cmd_build <- function(..., path = ".",
                        defaults = c("--no-build-vignettes"),
                        output_directory = path,
                      r_args = "--vanilla") {
    wd <- setwd(output_directory)
    on.exit(setwd(wd))
    args <- unique(c(defaults, unlist(list(...))))
    system2(get_r_binary(), args = c(r_args, "CMD build", args, path))
    gct <- tryCatch(get_current_tarball(output_directory),
                    error = function(e) return(FALSE))
    return(invisible(gct))
}

get_desc_value <- function(lines, what) {
    v <- trimws(strsplit(grep(paste0("^", what), lines, value = TRUE),
                         split = ":")[[1]][2])
    return(v)
}

get_current_tarball <- function(path, package_dir = path) {
    is_tarball <- !dir.exists(path) && file.exists(path) &&
        grepl("^.*\\.tar\\.gz$", path)
    if (is_tarball) {
        tarball <- path
    } else {
        description_path <- file.path(package_dir, "DESCRIPTION")
        if (file.exists(description_path)) {
            lines <- readLines(description_path)
            tarball <- paste0(get_desc_value(what = "Package", lines = lines),
                              "_",
                              get_desc_value("Version", lines = lines),
                              ".tar.gz")
            tarball <- file.path(path, tarball)
        } else {
            candiates <- list.files(path, full.names = TRUE,
                                               pattern = "^.*\\.tar\\.gz")
            info <- file.info(candiates)["mtime"]
            info[["name"]] <- rownames(info)
            info <- info[order(info$mtime), ]
            tarball <- info[nrow(info), "name"]
        }
    }
    tarball <- normalizePath(tarball, mustWork = TRUE)
    if (identical(length(tarball), 0L))
        throw(paste("Tarball does not exist!"))
    return(tarball)
}

r_cmd_check <- function(..., path = ".", defaults = c("--no-build-vignettes"),
                      r_args = "--vanilla") {
    tarball <- get_current_tarball(path)
    args <- unique(c(defaults, unlist(list(...))))
    r <- system2(get_r_binary(), args = c(r_args, "CMD check", args, tarball))
    return(r)
}


#' Install a Tarball or a Directory
#'
#' \code{\link[devtools:install]{devtools::install}} by defaults first builds
#' the tarball. Then it calls
#' \code{\link[callr:rcmd]{callr::rcmd}}, which allows for a lot of options.
#'
#' @param ... Arguments passed to \command{CMD INSTALL}.
#' @param r_args Arguments passed to \command{R}.
#' @param r An R binary.
#' @param path A path to a directory, but see \emph{try_tarball}.
#' @param try_tarball If \code{\link{TRUE}}, the \code{path} may be a tarball,
#' else it is interpreted as a path to a directory.
#' @export
#' @keywords internal
#' @family  bits and pieces
#' @return The return value of \code{\link{system2}}.
r_cmd_install <- function(..., path = ".",
                        r = get_r_binary(),
                        r_args = "--vanilla", try_tarball = TRUE) {
    args <- unlist(list(...))
    if (isTRUE(try_tarball)) {
        tarball <- tryCatch(get_current_tarball(path), error = identity)
        if (inherits(tarball, "error")) {
            warning(tarball[["message"]], " using directorty `",
                    normalizePath(path, mustWork = TRUE), "`.")
            what <- path
        } else {
            what <- tarball
        }
    } else {
        what <- path
    }
    r <- system2(r, args = c(r_args, "CMD INSTALL", args, what))
    return(r)
}

#' Compare Two Vectors
#'
#' Side-by-side comparison of two vectors.
#' The vectors get sorted and are compared element-wise.
#' So the result will be as long as the union of the two vectors plus their
#' number of values unique to one of them.
#' @param x,y Two vectors of the same mode.
#' @param differences_only Report only the differences?
#' @export
#' @family searching functions
#' @family vector comparing functions
#' @return A matrix containing the side-by-side comparison.
#' @examples
#' data(mtcars)
#' cars <- rownames(mtcars)
#' carz <- cars[-grep("Merc", cars)]
#' cars <- cars[nchar(cars) < 15]
#' cars <- c(cars, "foobar")
#' compare_vectors(cars, carz)
compare_vectors <- function(x, y, differences_only = FALSE) {
    names <- c(deparse(substitute(x)), deparse(substitute(y)))
    u <-  sort(union(x, y))
    x_only <- setdiff(u, x)
    y_only <- setdiff(u, y)
    ix <- match(x_only, u)
    iy <- match(y_only, u)
    m <- cbind(x = u, y = u)
    m[iy, "y"] <- NA
    m[ix, "x"] <- NA
    dimnames(m)[[2]] <- names
    dimnames(m)[[1]] <- u
    if (isTRUE(differences_only))
        m <- m[apply(apply(m, 2, is.na), 1, any), TRUE]
    if (is.vector(m))
        m <- as.matrix(t(m))
    return(m)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


data(mtcars)
cars <- rownames(mtcars)
carz <- cars[-grep("Merc", cars)]
cars <- cars[nchar(cars) < 15]
expectation <-
  structure(c(
    "AMC Javelin", NA, "Camaro Z28", NA, "Datsun 710",
    NA, "Duster 360", "Ferrari Dino", "Fiat 128", "Fiat X1-9",
    "Ford Pantera L",
    "Honda Civic", "Hornet 4 Drive", NA, NA, "Lotus Europa",
    "Maserati Bora",
    "Mazda RX4", "Mazda RX4 Wag", "Merc 230", "Merc 240D",
    "Merc 280",
    "Merc 280C", "Merc 450SE", "Merc 450SL", "Merc 450SLC", NA,
    "Porsche 914-2",
    "Toyota Corolla", "Toyota Corona", "Valiant", "Volvo 142E",
    "AMC Javelin",
    "Cadillac Fleetwood", "Camaro Z28", "Chrysler Imperial",
    "Datsun 710",
    "Dodge Challenger", "Duster 360", "Ferrari Dino",
    "Fiat 128",
    "Fiat X1-9", "Ford Pantera L", "Honda Civic",
    "Hornet 4 Drive",
    "Hornet Sportabout", "Lincoln Continental", "Lotus Europa",
    "Maserati Bora",
    "Mazda RX4", "Mazda RX4 Wag", NA, NA, NA, NA, NA, NA, NA,
    "Pontiac Firebird",
    "Porsche 914-2", "Toyota Corolla", "Toyota Corona",
    "Valiant",
    "Volvo 142E"
  ),
  .Dim = c(32L, 2L),
  .Dimnames = list(
    c(
      "AMC Javelin", "Cadillac Fleetwood",
      "Camaro Z28", "Chrysler Imperial",
      "Datsun 710", "Dodge Challenger",
      "Duster 360", "Ferrari Dino", "Fiat 128",
      "Fiat X1-9", "Ford Pantera L",
      "Honda Civic", "Hornet 4 Drive",
      "Hornet Sportabout", "Lincoln Continental",
      "Lotus Europa", "Maserati Bora",
      "Mazda RX4", "Mazda RX4 Wag", "Merc 230",
      "Merc 240D", "Merc 280", "Merc 280C",
      "Merc 450SE", "Merc 450SL", "Merc 450SLC",
      "Pontiac Firebird", "Porsche 914-2",
      "Toyota Corolla", "Toyota Corona",
      "Valiant", "Volvo 142E"
    ),
    c("cars", "carz")
  )
  )
result <- compare_vectors(cars, carz)
expect_identical(result, expectation)
expect_identical(
  sort(intersect(cars, carz)),
  row.names(result)[complete.cases(result)]
)

#' Find Missing Documentation
#'
#' For \pkg{fritools2}, we make exhaustive use of categorizing functions into
#' families with the `See also` section of the man pages (which are generated by
#' the @family tags in the code files).
#' @name missing_docs
#' @param path Path to a (package) directory.
#' @param list_families List the function families defined so far.
#' @family searching functions
NULL


#' @export
#' @rdname missing_docs
#' @aliases find_missing_see_also
#' @return For `find_missing_see_also`: a character vector of man pages with
#' missing `See also` sections.
find_missing_see_also <- function(path, list_families = TRUE) {
    files_with_seealso <- search_files("seealso",
                                       path = file.path(path, "man"),
                                       verbose = FALSE)
    files_with_seealso <- summary(files_with_seealso)[["file"]]
    files <- list.files(path = file.path(path, "man"), full.names = TRUE)
    diff <- compare_vectors(files, files_with_seealso, differences_only = TRUE)
    missings <- diff[is.na(diff[, "files_with_seealso"]), "files"]
    result <- strip_off_attributes(missings)
    if (isTRUE(list_families)) {
        families <- NULL
        for (file in files_with_seealso) {
            families <- c(families,
                          fromto(x = readLines(file), from = "^Other.*:",
                                 to = ".*", shift_to = -1))
        }
        message("Families so far: \n",
                paste(unique(sub(": *$", "", families)), collapse = "\n"))
    }
    return(result)
}

#' @export
#' @param clean Remove temporary directory?
#' @rdname missing_docs
#' @aliases find_missing_family
#' @return For `find_missing_family`: a character vector of function names with
#' missing `@family` tags.
find_missing_family <- function(path, list_families = TRUE, clean = TRUE) {
    wdir <- tempfile()
    dir.create(wdir)
    if (isTRUE(clean)) on.exit(unlink(wdir, recursive = TRUE))
    function_files <- NULL
    code_files <- list.files(file.path(path, "R"), full.names = TRUE)
    code_files <- grep("(-package.R|zzz.R)$", code_files, value = TRUE,
                       invert = TRUE)
    for (code_file in code_files) {
        function_files <- c(function_files,
                            split_code_file(code_file, wdir,
                                            write_to_disk = FALSE))
    }
    with_family <- search_files("^#' *@family", path = wdir, verbose = FALSE)
    with_doc <- search_files("^#' ", path = wdir, verbose = FALSE)
    with_rd_name <- search_files("^#' @rdname", path = wdir, verbose = FALSE)
    files_with_family <- summary(with_family)[["file"]]
    files_with_doc_no_rd <- setdiff(summary(with_doc)[["file"]],
                              summary(with_rd_name)[["file"]])
    diff <- compare_vectors(files_with_doc_no_rd, files_with_family,
                            differences_only = TRUE)
    missings <- diff[is.na(diff[, "files_with_family"]), "files_with_doc_no_rd"]
    result <- sub("\\.[rR]$", "", basename(strip_off_attributes(missings)))
    if (isTRUE(list_families)) {
        message("Families so far: \n",
                paste(unique(with_family$matches), collapse = "\n"))
    }
    return(result)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


path <- system.file("source", package = "fritools2")
result <- basename(find_missing_see_also(path, list_families = FALSE))
expectation <- "fritools2-package.Rd"
expect_identical(result, expectation)

result <- capture.output(find_missing_see_also(path, list_families = TRUE),
  type = "message"
)

expect_true("Families so far: " %in% result &&
  any(grepl("^Other", result)))
result <- is_of_length_zero(find_missing_family(path,
  list_families = FALSE
),
class = "character"
)
expect_true(result)

result <- capture.output(find_missing_family(path, list_families = TRUE),
  type = "message"
)
expect_true("Families so far: " %in% result &&
  any(grepl("^#' @family", result)))

#' Get the File Modified Last
#'
#' I often look for the  file modified last under some directory.
#' @param ... Arguments passed to \code{\link{find_files}}.
#' @return The path to the file last modified.
#' @family searching functions
#' @family file utilities
#' @export
#' @examples
#' for (suffix in c(".txt", ".ascii"))
#'     for (f in file.path(tempdir(), letters))
#'         touch(paste0(f, suffix))
#' list.files(tempdir())
#' file_modified_last(path = tempdir(), pattern = "\\.txt$")
#' dir.create(file.path(tempdir(), "new"))
#' touch(file.path(tempdir(), "new", "file.txt"))
#' file_modified_last(path = tempdir(), pattern = "\\.txt$")
#' file_modified_last(path = tempdir(), pattern = "\\.txt$", recursive = TRUE)
file_modified_last <- function(...) {
    lf <- find_files(...)
    mtime <- file.info(lf)["mtime"]
    i <- order(mtime[["mtime"]], decreasing = TRUE)[1]
    newest <- normalizePath(rownames(mtime)[i], mustWork = TRUE)
    return(newest)
}

#' Determine Subset Sizes Close to Equality
#'
#' Determine the sizes of k subsets of a set with
#' n elements in such a way that the sizes are as
#' equal as possible.
#'
#' @param n The size of the set.
#' @param k The number of subsets.
#' @return A vector of k sizes of the subsets.
#' @family subsetting functions
#' @export
#' @examples
#' subset_sizes(n = 100, k = 6)
#' subset_sizes(n = 2, k = 6)
subset_sizes <- function(n, k) {
    res <- NULL
    if (n < 2) throw("Got n < 2, can't determine subsets.")
    if (k > n) {
        warning("Got k > n, this will result in subsizes of 1 and 0.")
        res <- c(rep(1, n), rep(0, k - n))
    } else {
        if (n %% k == 0) {
            res <- rep(n / k, k)
        } else {
            fl <- floor(n / k)
            ce <- ceiling(n / k)
            res <- c(rep(ce, round((n / k - fl) * k)),
                     rep(fl, round((1 - (n / k - fl)) * k)))
        }
    }
    return(res)
}

#' Determine Indices and Sizes of Subsets
#'
#' Create starting and stopping indices for subsets defined by
#' \code{\link{subset_sizes}}.
#' @param n The size of the set.
#' @param k The number of subsets.
#' @return A matrix with starting index, size, and stopping index for each
#' subset.
#' @family subsetting functions
#' @export
#' @examples
#' index_groups(n = 100, k = 6)
#' index_groups(n = 2, k = 6)
index_groups <- function(n, k) {
    sizes <- subset_sizes(n, k)
    start <- 1 + c(0, cumsum(sizes)[1:(k - 1)])
    stop <- start + sizes - 1
    res <- cbind(id = 1:k, start, sizes, stop)[sizes > 0, TRUE]
    return(res)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


# % regular
result <- subset_sizes(n = 100, k = 6)
expectation <- c(17, 17, 17, 17, 16, 16)
expect_identical(result, expectation)
# % symmetric
result <- subset_sizes(n = 10, k = 5)
expectation <- rep(2, 5)
expect_identical(result, expectation)

# % k too big
## % check the warning
result <- tryCatch(subset_sizes(n = 2, k = 6),
  warning = function(w) {
    return(w)
  }
)
expectation <- tryCatch(
  {
    msg <- "Got k > n, this will result in subsizes of 1 and 0."
    warning(msg)
  },
  warning = function(w) {
    return(w)
  }
)
expect_identical(result[["message"]], expectation[["message"]])
## % check the result
result <- suppressWarnings(subset_sizes(n = 2, k = 6))
expectation <- c(1, 1, 0, 0, 0, 0)
expect_identical(result, expectation)

# % n too small
expect_error(subset_sizes(n = 1, k = 6))





result <- index_groups(n = 10, k = 3)
expectation <- structure(c(1, 2, 3, 1, 5, 8, 4, 3, 3, 4, 7, 10),
  .Dim = 3:4,
  .Dimnames = list(
    NULL,
    c("id", "start", "sizes", "stop")
  )
)
expect_identical(result, expectation)

#' Count Observations per Groups
#'
#' I tend to forget the syntax that works with
#' \code{\link[stats:aggregate]{stats::aggregate}}.
#' @param x A \code{\link{data.frame}}.
#' @param ... Columns in \code{x}.
#' @family statistics
#' @return A \code{\link{data.frame}} with the counts per groups.
#' @export
#' @examples
#' count_groups(mtcars, "am", "gear")
#' RUnit::checkEquals(dplyr::count(mtcars, am, gear),
#'                    count_groups(mtcars, "am", "gear"), checkNames = FALSE)
count_groups <- function(x, ...) {
    l <- list(...)
    by <- list()
    for (i in seq(along = l)) {
        name <- names(l)[i]
        if (is.null(name)) name <- l[[i]]
        by[name] <- x[l[[i]]]
    }
    res <- stats::aggregate(x = x[l[[1]]],
                     by,
                     length)
    names(res)[dim(res)[2]] <- "count"
    return(res)
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


result <- count_groups(mtcars, "am", "gear")
expectation <- structure(list(
  am = c(0, 0, 1, 1),
  gear = c(3, 4, 4, 5),
  count = c(15L, 4L, 8L, 5L)
),
row.names = c(NA, -4L), class = "data.frame"
)
expect_identical(result, expectation)

#' Apply a Function Over a Ragged Array
#'
#' This is a modified version of \code{\link[base:tapply]{base::tapply}} to
#' allow for \code{\link{data.frame}}s to be passed as \code{X}.
#' @param object See \code{\link[base:tapply]{base::tapply}} \emph{X}.
#' @param index See \code{\link[base:tapply]{base::tapply}} \emph{INDEX}.
#' @param func See \code{\link[base:tapply]{base::tapply}} \emph{FUN}.
#' @param ... See \code{\link[base:tapply]{base::tapply}}.
#' @param default See \code{\link[base:tapply]{base::tapply}}.
#' @param simplify See \code{\link[base:tapply]{base::tapply}}.
#' @return See \code{\link[base:tapply]{base::tapply}}.
#' @family bits and pieces
#' @export
#' @examples
#' result <- fritools2::tapply(warpbreaks[["breaks"]], warpbreaks[, -1], sum)
#' expectation <- base::tapply(warpbreaks[["breaks"]], warpbreaks[, -1], sum)
#' RUnit::checkIdentical(result, expectation)
#' data("mtcars")
#' s <- stats::aggregate(x = mtcars[["mpg"]],
#'                       by = list(mtcars[["cyl"]], mtcars[["vs"]]),
#'                       FUN = mean)
#' t <- base::tapply(X = mtcars[["mpg"]],
#'                   INDEX = list(mtcars[["cyl"]], mtcars[["vs"]]),
#'                   FUN = mean)
#' if (require("reshape", quietly = TRUE)) {
#'     suppressWarnings(tm <- na.omit(reshape::melt(t)))
#'     if (RUnit::checkEquals(s, tm, check.attributes = FALSE))
#'         message("Works!")
#' }
#' message("If you don't pass weigths, this is equal to:")
#' w <- base::tapply(X = mtcars[["mpg"]], INDEX = list(mtcars[["cyl"]],
#'                                                     mtcars[["vs"]]),
#'                   FUN = stats::weighted.mean)
#' all.equal(w, t, check.attributes = FALSE)
#' message("But how do you pass those weights?")
#' # we define a wrapper to pass the column names for a data.frame:
#' weighted_mean <- function(df, x, w) {
#'     stats::weighted.mean(df[[x]], df[[w]])
#' }
#' if (RUnit::checkIdentical(stats::weighted.mean(mtcars[["mpg"]],
#'                                                mtcars[["wt"]]),
#'                           weighted_mean(mtcars, "mpg", "wt")))
#'     message("Works!")
#' message("base::tapply can't deal with data.frames:")
#' try(base::tapply(X = mtcars, INDEX = list(mtcars[["cyl"]], mtcars[["vs"]]),
#'                  FUN = weighted_mean, x = "mpg", w = "wt"))
#' wm <- fritools2::tapply(object = mtcars, index = list(mtcars[["cyl"]],
#'                                                 mtcars[["vs"]]),
#'                        func = weighted_mean, x = "mpg", w = "wt")
#' subset <- mtcars[mtcars[["cyl"]] == 6 & mtcars[["vs"]] == 0, c("mpg", "wt")]
#' stats::weighted.mean(subset[["mpg"]], subset[["wt"]]) == wm
tapply <- function(object, index, func = NULL, ..., default = NA,
                   simplify = TRUE) {
    func <- if (!is.null(func))
        match.fun(func)
    if (!is.list(index))
        index <- list(index)
    index <- lapply(index, as.factor)
    num_i <- length(index)
    if (!num_i)
        stop("'index' is of length zero")
    if (is.data.frame(object)) {
        if (!all(lengths(index) == nrow(object)))
            stop("arguments must have same length")
    } else {
        if (!all(lengths(index) == length(object)))
            stop("arguments must have same length")
    }
    namelist <- lapply(index, levels)
    extent <- lengths(namelist, use.names = FALSE)
    cumextent <- cumprod(extent)
    if (cumextent[num_i] > .Machine[["integer.max"]])
        stop("total number of levels >= 2^31")
    storage.mode(cumextent) <- "integer"
    ngroup <- cumextent[num_i]
    group <- as.integer(index[[1L]])
    if (num_i > 1L)
        for (i in 2L:num_i) group <- group + cumextent[i - 1L] *
            (as.integer(index[[i]]) - 1L)
    if (is.null(func))
        return(group)
    levels(group) <- as.character(seq_len(ngroup))
    class(group) <- "factor"
    ans <- split(object, group)
    names(ans) <- NULL
    idx <- as.logical(lengths(ans))
    ans <- lapply(X = ans[idx], FUN = func, ...)
    ansmat <- array(if (simplify && all(lengths(ans) == 1L)) {
                        ans <- unlist(ans, recursive = FALSE, use.names = FALSE)
                        if (!is.null(ans) && is.na(default) && is.atomic(ans))
                            vector(typeof(ans))
                        else
                            default
                   } else {
                       vector("list", prod(extent))
                   },
                   dim = extent, dimnames = namelist)
    if (length(ans)) {
        ansmat[idx] <- ans
    }
    ansmat
}

if (interactive()) {
  pkgload::load_all()
  library("tinytest")
}


result <- fritools2::tapply(warpbreaks[["breaks"]], warpbreaks[, -1], sum)
expectation <- base::tapply(warpbreaks[["breaks"]], warpbreaks[, -1], sum)
expect_identical(result, expectation)







weighted_mean <- function(df, x, w) {
  stats::weighted.mean(df[[x]], df[[w]])
}
data("mtcars")
wm <- fritools2::tapply(
  object = mtcars, index = list(
    mtcars[["cyl"]],
    mtcars[["vs"]]
  ),
  func = weighted_mean, x = "mpg", w = "wt"
)
for (cyl in c(4, 6, 8)) {
  for (vs in c(0, 1)) {
    subset <- mtcars[
      mtcars[["cyl"]] == cyl & mtcars[["vs"]] == vs,
      c("mpg", "wt")
    ]
    if (interactive()) {
      print(stats::weighted.mean(
        subset[["mpg"]],
        subset[["wt"]]
      ))
    }
    expect_identical(
      stats::weighted.mean(
        subset[["mpg"]],
        subset[["wt"]]
      ),
      wm[as.character(cyl), as.character(vs)]
    )
  }
}

if (interactive()) {
    pkgload::load_all()
    library("tinytest")
}

file_paths <- list.files(path = system.file("tinytest", package = "fritools2"),
                         pattern = ".*\\.R", full.names = TRUE)
file_paths <- grep("test_grep_file.R", file_paths, invert = TRUE, value = TRUE)
res <- grep_file(path = file_paths,
                 pattern = "forSureNotThere", a = 3, b = 2, ignore.case = TRUE)
tinytest::expect_true(all(res == FALSE))
result <- grep_file(path = file_paths, pattern = "^ *#", a = 1, b = 1,
                    ignore.case = TRUE)
expectation <- c("test_boolean.R", "test_call.R", "test_column_sums.R",
                 "test_compare_vectors.R",
                 "test_convert_umlauts_to_ascii.R",
                 "test_convert_umlauts_to_tex.R",
                 "test_count_groups.R", "test_csv.R",
                 "test_delete_trailing_blank_lines.R",
                 "test_delete_trailing_whitespace.R", "test_file_copy.R",
                 "test_file_modified_last.R",
                 "test_file_save.R", "test_find_files.R",
                 "test_find_missing_see_also.R",
                 "test_fritools.R", "test_fromto.R",
                 "test_get_package_version.R",
                 "test_get_unique_string.R", "test_golden_ratio.R",
                 "test_importing.R",
                 "test_installed.R", "test_is_cran.R",
                 "test_is_difftime_less.R",
                 "test_is_files_current.R", "test_is_of_length_zero.R",
                 "test_is_path.R",
                 "test_is_success.R", "test_is_valid_primary_key.R",
                 "test_is_version_sufficient.R",
                 "test_is_windows.R", "test_load_internal_functions.R",
                 "test_memory_hogs.R",
                 "test_options.R", "test_paths.R",
                 "test_round_half_away_from_zero.R",
                 "test_search_files.R", "test_search_rows.R",
                 "test_split_code_file.R",
                 "test_str2num.R", "test_strip_off_attributes.R",
                 "test_subsets.R",
                 "test_tapply.R", "test_test_helpers.R", "test_throw.R",
                 "test_touch.R",
                 "test_weighted_variance.R", "test_wipe_clean.R",
                 "test_with_dir.R"
                 )
tinytest::expect_true(all(expectation %in% basename(names(result))))

%\VignetteIndexEntry{Not an Introduction to fritools2}
\documentclass[a4paper]{article}
\bibliographystyle{unsrt}
\usepackage{listings}
\lstset{ %
basicstyle=\footnotesize
, commentstyle=\color{PineGreen}
, numberstyle=\tiny\color{Gray}
, rulecolor=\color{Black}
, keywordstyle=\color{Blue}
, stringstyle=\color{Sepia}
, showstringspaces=false
, language=R
}
\usepackage[%
    colorlinks=true,
    pdfborder={0 0 0},
    linkcolor=blue
]{hyperref}

\usepackage[utf8]{inputenc}
\title{Not an Introduction to fritools2}
\author{Andreas Dominik Cullmann}

\SweaveOpts{echo=false}
\SweaveOpts{eval=false}
\SweaveOpts{print=false}
\SweaveOpts{width=4.5}
\SweaveOpts{height=5}
\begin{document}
\maketitle

This is not a vignette at all as \emph{fritools2} is just a set of
miscellaneous utilities, tools and helper functions written for the
Forest Research Institute of the State Baden-Wuerttemberg, Germany.

 \emph{fritools2} does not use any other packages than those in the R Core.
It does not import or depend on any third party package.


All functions belong to at least one of these families:
<<missings, echo = TRUE, eval = TRUE, fig=FALSE>>==
path <- system.file(package = "fritools2")
if (file.exists(file.path(path, "source")))
    path <- file.path(path, "source")
fmsa <- fritools2::find_missing_see_also
f <- capture.output(type = "message",
                    missing <- suppressWarnings(fmsa(path = path,
                                                     list_families = TRUE)))
print(f)
if (length(missing) > 1) {
    print(missing)
    stop("Functions without context.")
} else {
    print("All functions with context.")
}
@
That's all.

\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/with_dir.R
\name{with_dir}
\alias{with_dir}
\title{Execute Code in a Temporary Working Directory}
\usage{
with_dir(new, code)
}
\arguments{
\item{new}{The new working directory.}

\item{code}{Code to execute in the temporary working directory.}
}
\value{
The results of the evaluation of the \code{code} argument.
}
\description{
This is a verbatim copy of \code{withr::with_dir} from of \pkg{withr}'s
version 2.4.1.
I often need \pkg{withr} only to import \code{withr::with_dir}, which is a
really simple function. So I just hijack \code{withr::with_dir}.
}
\examples{
temp_dir <- file.path(tempfile())
dir.create(temp_dir)
with_dir(temp_dir, getwd())
}
\seealso{
Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{vim}()},
\code{\link{wipe_tempdir}()}
}
\concept{operating system functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_r_cmd_check.R
\name{is_r_cmd_check}
\alias{is_r_cmd_check}
\title{Is the Current R Process an \command{R CMD check}?}
\usage{
is_r_cmd_check()
}
\value{
\code{\link[base:TRUE]{TRUE}} on success, 
\code{\link[base:FALSE]{FALSE}} otherwise.
}
\description{
Check for system variables to guess whether or not this is an
\command{R CMD check}.
}
\seealso{
Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}

Other test helpers: 
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_cran}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{run_r_tests_for_known_hosts}()},
\code{\link{set_run_r_tests}()}
}
\concept{logical helpers}
\concept{test helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/throw.R
\name{throw}
\alias{throw}
\title{Throw a Condition}
\usage{
throw(message_string, system_call = sys.call(-1), ...)
}
\arguments{
\item{message_string}{The message to be thrown.}

\item{system_call}{The call to be thrown.}

\item{...}{Arguments to be passed to
\code{\link[base:structure]{base::structure}}.}
}
\value{
The function does never return anything, it stops with a
condition of class c("error", "fritools2", "condition").
}
\description{
Throws a condition of class c("error", "fritools2", "condition").
}
\details{
We use this condition as an error dedicated to \pkg{ fritools2.}
}
\seealso{
Other bits and pieces: 
\code{\link{golden_ratio}()},
\code{\link{is_difftime_less}()},
\code{\link{is_valid_primary_key}()},
\code{\link{r_cmd_install}()},
\code{\link{str2num}()},
\code{\link{strip_off_attributes}()},
\code{\link{tapply}()}
}
\concept{bits and pieces}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/tapply.R
\name{tapply}
\alias{tapply}
\title{Apply a Function Over a Ragged Array}
\usage{
tapply(object, index, func = NULL, ..., default = NA, simplify = TRUE)
}
\arguments{
\item{object}{See \code{\link[base:tapply]{base::tapply}} \emph{X}.}

\item{index}{See \code{\link[base:tapply]{base::tapply}} \emph{INDEX}.}

\item{func}{See \code{\link[base:tapply]{base::tapply}} \emph{FUN}.}

\item{...}{See \code{\link[base:tapply]{base::tapply}}.}

\item{default}{See \code{\link[base:tapply]{base::tapply}}.}

\item{simplify}{See \code{\link[base:tapply]{base::tapply}}.}
}
\value{
See \code{\link[base:tapply]{base::tapply}}.
}
\description{
This is a modified version of \code{\link[base:tapply]{base::tapply}} to
allow for \code{\link{data.frame}}s to be passed as \code{X}.
}
\examples{
result <- fritools2::tapply(warpbreaks[["breaks"]], warpbreaks[, -1], sum)
expectation <- base::tapply(warpbreaks[["breaks"]], warpbreaks[, -1], sum)
RUnit::checkIdentical(result, expectation)
data("mtcars")
s <- stats::aggregate(x = mtcars[["mpg"]],
                      by = list(mtcars[["cyl"]], mtcars[["vs"]]),
                      FUN = mean)
t <- base::tapply(X = mtcars[["mpg"]],
                  INDEX = list(mtcars[["cyl"]], mtcars[["vs"]]),
                  FUN = mean)
if (require("reshape", quietly = TRUE)) {
    suppressWarnings(tm <- na.omit(reshape::melt(t)))
    if (RUnit::checkEquals(s, tm, check.attributes = FALSE))
        message("Works!")
}
message("If you don't pass weigths, this is equal to:")
w <- base::tapply(X = mtcars[["mpg"]], INDEX = list(mtcars[["cyl"]],
                                                    mtcars[["vs"]]),
                  FUN = stats::weighted.mean)
all.equal(w, t, check.attributes = FALSE)
message("But how do you pass those weights?")
# we define a wrapper to pass the column names for a data.frame:
weighted_mean <- function(df, x, w) {
    stats::weighted.mean(df[[x]], df[[w]])
}
if (RUnit::checkIdentical(stats::weighted.mean(mtcars[["mpg"]],
                                               mtcars[["wt"]]),
                          weighted_mean(mtcars, "mpg", "wt")))
    message("Works!")
message("base::tapply can't deal with data.frames:")
try(base::tapply(X = mtcars, INDEX = list(mtcars[["cyl"]], mtcars[["vs"]]),
                 FUN = weighted_mean, x = "mpg", w = "wt"))
wm <- fritools2::tapply(object = mtcars, index = list(mtcars[["cyl"]],
                                                mtcars[["vs"]]),
                       func = weighted_mean, x = "mpg", w = "wt")
subset <- mtcars[mtcars[["cyl"]] == 6 & mtcars[["vs"]] == 0, c("mpg", "wt")]
stats::weighted.mean(subset[["mpg"]], subset[["wt"]]) == wm
}
\seealso{
Other bits and pieces: 
\code{\link{golden_ratio}()},
\code{\link{is_difftime_less}()},
\code{\link{is_valid_primary_key}()},
\code{\link{r_cmd_install}()},
\code{\link{str2num}()},
\code{\link{strip_off_attributes}()},
\code{\link{throw}()}
}
\concept{bits and pieces}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csv.R
\name{set_hash}
\alias{set_hash}
\title{Set a Hash Attribute on an Object}
\usage{
set_hash(x)
}
\arguments{
\item{x}{The object.}
}
\value{
The modified object.
}
\description{
Set a Hash Attribute on an Object
}
\seealso{
Other hash functions for objects: 
\code{\link{un_hash}()}
}
\concept{hash functions for objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch.R
\name{is_batch}
\alias{is_batch}
\title{Is R Run in Batch Mode (via \command{R CMD BATCH} or \command{Rscript})?}
\usage{
is_batch()
}
\value{
\code{\link[base:TRUE]{TRUE}} on success, 
\code{\link[base:FALSE]{FALSE}} otherwise.
}
\description{
Just a wrapper to \code{\link{interactive}}.
}
\examples{
is_batch()
}
\seealso{
Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}
}
\concept{logical helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csv.R
\name{un_hash}
\alias{un_hash}
\title{Separate an Object from its Hash Attribute}
\usage{
un_hash(x)
}
\arguments{
\item{x}{The object.}
}
\value{
A list containing the object and its hash attribute.
}
\description{
We calculate a hash value of an object and store it as an attribute of the
objects, the hash value of that object will change. So we need to split the
hash value from the object to see whether or not the objected changed.
}
\seealso{
Other hash functions for objects: 
\code{\link{set_hash}()}
}
\concept{hash functions for objects}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_package_version.R
\name{get_package_version}
\alias{get_package_version}
\title{Query Installed Package Version}
\usage{
get_package_version(x, lib_loc = NULL)
}
\arguments{
\item{x}{A character giving the package name.}

\item{lib_loc}{See argument \code{lib.loc} in
\code{\link{packageDescription}}.}
}
\value{
A character giving the package version.
}
\description{
\code{\link{packageVersion}} converts to class \code{\link{package_version}},
which then again would need to be converted for \code{\link{compareVersion}}.
So this is a modified copy of \code{\link{packageVersion}} skipping the
conversion to \code{\link{package_version}}.
}
\examples{
get_package_version("base")
try(get_package_version("mgcv"))
utils::compareVersion("1000.0.0", get_package_version("base"))
utils::compareVersion("1.0", get_package_version("base"))
# from ?is_version_sufficient:
is_version_sufficient(installed = get_package_version("base"),
                      required = "1.0")
}
\seealso{
Other version functions: 
\code{\link{is_r_package_installed}()},
\code{\link{is_version_sufficient}()}

Other package functions: 
\code{\link{is_r_package_installed}()},
\code{\link{is_version_sufficient}()},
\code{\link{load_internal_functions}()}
}
\concept{package functions}
\concept{version functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/call.R
\name{call_safe}
\alias{call_safe}
\title{Call a Function Given an External Dependency on Non-Windows Systems}
\usage{
call_safe(f, dependency, fallback = "Fallback", ...)
}
\arguments{
\item{f}{The function passed to \code{\link{do.call}}.}

\item{dependency}{The external dependency, see \emph{Examples}.}

\item{fallback}{See \emph{Description}.}

\item{...}{arguments passed to \code{\link{do.call}}.}
}
\value{
The return value of \code{f} or \code{fallback}.
}
\description{
Just a specialized version of \code{\link{call_conditionally}}.
}
\examples{
call_safe(whoami::email_address, dependency = "whoami",
          args = list(fallback = "foobar@nowhere.com"),
          fallback = "nobar@nowhere.com")
call_safe(whoami::email_address, dependency = "this_is_not_installed",
          args = list(fallback = "foobar@nowhere.com"),
          fallback = "nobar@nowhere.com")
}
\seealso{
Other call functions: 
\code{\link{call_conditionally}()}
}
\concept{call functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsets.R
\name{subset_sizes}
\alias{subset_sizes}
\title{Determine Subset Sizes Close to Equality}
\usage{
subset_sizes(n, k)
}
\arguments{
\item{n}{The size of the set.}

\item{k}{The number of subsets.}
}
\value{
A vector of k sizes of the subsets.
}
\description{
Determine the sizes of k subsets of a set with
n elements in such a way that the sizes are as
equal as possible.
}
\examples{
subset_sizes(n = 100, k = 6)
subset_sizes(n = 2, k = 6)
}
\seealso{
Other subsetting functions: 
\code{\link{index_groups}()}
}
\concept{subsetting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/count_groups.R
\name{count_groups}
\alias{count_groups}
\title{Count Observations per Groups}
\usage{
count_groups(x, ...)
}
\arguments{
\item{x}{A \code{\link{data.frame}}.}

\item{...}{Columns in \code{x}.}
}
\value{
A \code{\link{data.frame}} with the counts per groups.
}
\description{
I tend to forget the syntax that works with
\code{\link[stats:aggregate]{stats::aggregate}}.
}
\examples{
count_groups(mtcars, "am", "gear")
RUnit::checkEquals(dplyr::count(mtcars, am, gear),
                   count_groups(mtcars, "am", "gear"), checkNames = FALSE)
}
\seealso{
Other statistics: 
\code{\link{column_sums}()},
\code{\link{relative_difference}()},
\code{\link{round_half_away_from_zero}()},
\code{\link{weighted_variance}()}
}
\concept{statistics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csv.R
\name{csv2csv}
\alias{csv2csv}
\title{Convert a German Comma Separated File into a Comma Separated File}
\usage{
csv2csv(file, ...)
}
\arguments{
\item{file}{Path to the file.}

\item{...}{Arguments passed to \code{\link{read_csv}}}
}
\value{
\code{\link[base:invisible]{Invisibly}} the return value of
\code{\link{write_csv}}, but called for its side effect.
}
\description{
Convert a German Comma Separated File into a Comma Separated File
}
\examples{
f <- tempfile()
write.csv2(mtcars, file = f)
res <- csv2csv(f)
readLines(get_path(res), n = 1)
write.csv(mtcars, file = f)
readLines(get_path(res), n = 1)
}
\seealso{
Other CSV functions: 
\code{\link{bulk_read_csv}()},
\code{\link{bulk_write_csv}()},
\code{\link{check_ascii_file}()},
\code{\link{csv}}
}
\concept{CSV functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/golden_ratio.R
\name{golden_ratio}
\alias{golden_ratio}
\title{Calculate the Golden Ratio}
\usage{
golden_ratio(x)
}
\arguments{
\item{x}{The sum of the two quantities to be in the golden ratio.}
}
\value{
A numeric vector of length 2, containing the two quantities \emph{a}
and \emph{b}, \emph{a}  being the larger.
}
\description{
Divide a length using the golden ratio.
}
\examples{
golden_ratio(10)
}
\seealso{
Other bits and pieces: 
\code{\link{is_difftime_less}()},
\code{\link{is_valid_primary_key}()},
\code{\link{r_cmd_install}()},
\code{\link{str2num}()},
\code{\link{strip_off_attributes}()},
\code{\link{tapply}()},
\code{\link{throw}()}
}
\concept{bits and pieces}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/str2num.R
\name{str2num}
\alias{str2num}
\title{Convert Character Numbers to Numeric}
\usage{
str2num(x)
}
\arguments{
\item{x}{A string representing a (possibly German) number.}
}
\value{
The number as a numeric.
}
\description{
If you read text containing  (possibly German, i.e. the decimals separated by
comma and dots inserted for what they think of as readability) numbers, you
may want to convert them to numeric.
}
\examples{
line_in_text <- "foo bar 10.303,70 foo bar  1.211.000,55 foo bar"
words <- unlist(strsplit(line_in_text, split = " "))
print(na.omit(sapply(words, str2num)), digits = 9)
print(str2num(words[c(3, 4, 7)]), digits = 9)
print(str2num(words[7]), digits = 9)
}
\seealso{
Other bits and pieces: 
\code{\link{golden_ratio}()},
\code{\link{is_difftime_less}()},
\code{\link{is_valid_primary_key}()},
\code{\link{r_cmd_install}()},
\code{\link{strip_off_attributes}()},
\code{\link{tapply}()},
\code{\link{throw}()}
}
\concept{bits and pieces}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/vim.R
\name{vim}
\alias{vim}
\title{Edit a File With VIM if Possible}
\usage{
vim(...)
}
\arguments{
\item{...}{See \code{\link{file.edit}}.}
}
\value{
See \code{\link{file.edit}}.
}
\description{
Just a wrapper to \code{\link{file.edit}}, trying to use [g]vim as editor, if
installed.
}
\examples{
if (interactive()) {
    path <- file.path(tempdir(), "foo.txt")
    writeLines(c("abc", "xyz"), con = path)
    vim(path)
}
}
\seealso{
Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{wipe_tempdir}()},
\code{\link{with_dir}()}
}
\concept{operating system functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_missing_doc.R
\name{missing_docs}
\alias{missing_docs}
\alias{find_missing_see_also}
\alias{find_missing_family}
\title{Find Missing Documentation}
\usage{
find_missing_see_also(path, list_families = TRUE)

find_missing_family(path, list_families = TRUE, clean = TRUE)
}
\arguments{
\item{path}{Path to a (package) directory.}

\item{list_families}{List the function families defined so far.}

\item{clean}{Remove temporary directory?}
}
\value{
For `find_missing_see_also`: a character vector of man pages with
missing `See also` sections.

For `find_missing_family`: a character vector of function names with
missing `@family` tags.
}
\description{
For \pkg{fritools2}, we make exhaustive use of categorizing functions into
families with the `See also` section of the man pages (which are generated by
the @family tags in the code files).
}
\seealso{
Other searching functions: 
\code{\link{compare_vectors}()},
\code{\link{file_modified_last}()},
\code{\link{find_files}()},
\code{\link{fromto}()},
\code{\link{search_files}()},
\code{\link{search_rows}()},
\code{\link{summary.filesearch}()}
}
\concept{searching functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch.R
\name{get_r_cmd_batch_script_path}
\alias{get_r_cmd_batch_script_path}
\title{Get the Path of the R Code File in Case of an \command{R CMD BATCH} Run}
\usage{
get_r_cmd_batch_script_path()
}
\value{
A vector of \code{\link{mode}} character giving the name of the R
code file. Will be character(0) if not in an \command{R CMD BATCH} run.
}
\description{
Retrieve the path from parsing the command line arguments of a
\command{R CMD BATCH} run.
}
\examples{
get_r_cmd_batch_script_path()
}
\seealso{
Other script path getter functions: 
\code{\link{get_rscript_script_path}()},
\code{\link{get_script_name}()},
\code{\link{get_script_path}()}
}
\concept{script path getter functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paths.R
\name{get_mtime}
\alias{get_mtime}
\title{Get the \code{mtime} Attribute to or from an Object}
\usage{
get_mtime(x)
}
\arguments{
\item{x}{An object.}
}
\value{
The value of \code{attr(attr(x, "path", "mtime")}.
}
\description{
We set modification times on some objects, this is a convenience wrappers to
\code{\link{attr}}.
}
\examples{
x <- 2
path <- tempfile()
touch(path)
x <- set_path(x, path)
get_mtime(x)
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{get_options}
\alias{get_options}
\title{Get Options For Packages}
\usage{
get_options(
  ...,
  package_name = .packages()[1],
  remove_names = FALSE,
  flatten_list = TRUE
)
}
\arguments{
\item{...}{See \code{\link{getOption}}.}

\item{package_name}{The package's name.}

\item{remove_names}{[boolean(1)]\cr Remove the names?}

\item{flatten_list}{[boolean(1)]\cr Return a vector?}
}
\value{
A (possibly named) list or a vector.
}
\description{
A convenience function for \code{\link{getOption}}.
}
\examples{
example("set_options", package = "fritools2")
}
\seealso{
Other option functions: 
\code{\link{is_force}()},
\code{\link{set_options}()}
}
\concept{option functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csv.R
\name{csv}
\alias{csv}
\alias{read_csv}
\alias{write_csv}
\title{Read and Write a Comma Separated File}
\usage{
read_csv(file, ...)

write_csv(x, file = NULL, csv_type = c(NA, "standard", "german"))
}
\arguments{
\item{file}{The path to the file to be read or written.}

\item{...}{Arguments passed to \code{\link[utils:read.csv]{utils::read.csv}}
or \code{\link[utils:read.csv2]{utils::read.csv2}}.}

\item{x}{The object to write to disk.}

\item{csv_type}{Which \command{csv} type is to be used. If \code{NA}, the
\code{csv} attribute is read from the object.}
}
\value{
For \bold{\code{read_csv}}: An object read from the file.

For \bold{\code{write_csv}}: The object with updated \code{hash}
(and possibly \code{path} and \code{csv})
attribute.
}
\description{
Functions to read and write CSV files. The objects returned by these
functions are \code{\link{data.frame}s} with the following attributes:
\describe{
  \item{path}{The path to the file on disk.}
  \item{csv}{The type of CSV: either \code{standard} or \code{german}.}
  \item{hash}{The hash value computed with \pkg{digest}'s digest function, if
  \pkg{digest} is installed.}
}

\code{read_csv} is a wrapper to determine whether to use
\code{\link[utils:read.csv2]{utils:read.csv2}} or
\code{\link[utils:read.csv]{utils:read.csv}}.
It sets the above three arguments.

\code{write_csv} compares the \code{hash} value stored in the object's
attribute
with the objects current hash value. If they differ, it writes the object to
the \code{file} argument or, if not given, to the \code{path}
stored in the object's attribute. If no \code{csv_type} is given, it uses
the \code{csv} type stored in object's attribute.
If \pkg{digest} is not installed, the object will (unconditionally) be
written to disk.
}
\examples{
# read from standard CSV
f <- tempfile()
write.csv(mtcars, file = f)
str(read_csv(f))
f <- tempfile()
write.csv2(mtcars, file = f)
str(read_csv(f))
# write to standard CSV
f <- tempfile()
d <- mtcars
str(d <- write_csv(d, file = f))
file.mtime(f)
Sys.sleep(2) # make sure the mtime would have changed
write_csv(d, file = f)
file.mtime(f)
}
\seealso{
Other CSV functions: 
\code{\link{bulk_read_csv}()},
\code{\link{bulk_write_csv}()},
\code{\link{check_ascii_file}()},
\code{\link{csv2csv}()}
}
\concept{CSV functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/relative_difference.R
\name{relative_difference}
\alias{relative_difference}
\title{Compute Relative Differences Between the Values of Two Vectors}
\usage{
relative_difference(
  current,
  reference,
  type = c("all.equal", "difference", "change")
)
}
\arguments{
\item{current}{One vector.}

\item{reference}{Another vector, for \code{type = all.equal}, this is passed
as \code{target}, for \code{type = all.equal} this can be thought of as the
"correct" value or the state "before".}

\item{type}{The method to be used. See Details.}
}
\description{
We often try to compare vectors on near equality. This is a wrapper to
\code{\link{all.equal}} for our convenience. It also implements relative
difference and change as discussed in
\url{https://en.wikipedia.org/wiki/Relative_change_and_difference}.
}
\details{
The default method (\code{type = all.equal}) applies
\code{\link{all.equal}} onto the two
vectors. Method \code{type = difference} is somewhat the same as the default,
method \code{type = change} takes account of the sign of the differences.
}
\examples{
n <- 500
x <- rnorm(n)
y <- x + rnorm(n, sd = 0.0001)
plot(relative_difference(x, y), x)
plot(relative_difference(x, y, "difference"), x)
# They do approximately the same:
max(relative_difference(relative_difference(x, y),
                            relative_difference(x, y, "difference")))
# Takes sign into account:
plot(relative_difference(x, y, "change"), x)
max(relative_difference(relative_difference(x, y),
                        abs(relative_difference(x, y, "change"))))
}
\seealso{
Other statistics: 
\code{\link{column_sums}()},
\code{\link{count_groups}()},
\code{\link{round_half_away_from_zero}()},
\code{\link{weighted_variance}()}

Other vector comparing functions: 
\code{\link{compare_vectors}()}
}
\concept{statistics}
\concept{vector comparing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/get_unique_string.R
\name{get_unique_string}
\alias{get_unique_string}
\title{Create a Fairly Unique String}
\usage{
get_unique_string()
}
\value{
A fairly unique string.
}
\description{
I sometimes need a fairly unique string, mostly for file names, that should
start with the current date.
}
\examples{
replicate(20, get_unique_string())
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search_rows.R
\name{search_rows}
\alias{search_rows}
\title{Search All Rows Across Columns of a Matrix-like Structure}
\usage{
search_rows(x, pattern = ".*", include_row_names = TRUE)
}
\arguments{
\item{x}{A \code{\link{matrix}} or \code{\link{data.frame}}.}

\item{pattern}{A pattern.}

\item{include_row_names}{Include row names into the search?}
}
\value{
All rows where the pattern was found in at least one column.
}
\description{
I sometimes need to see which rows of a matrix-like structure
contain a string matched by a search pattern.
This somewhat similar to writing a matrix-like structure to disk and then
using \code{\link{search_files}} on it.
}
\examples{
p <- "\\\\<4.0[[:alpha:]]*\\\\>"
search_rows(x = mtcars, pattern = p)
search_rows(x = mtcars, pattern = p, include_row_names = FALSE)
try(search_rows(x = mtcars, pattern = "ABC"))
}
\seealso{
Other searching functions: 
\code{\link{compare_vectors}()},
\code{\link{file_modified_last}()},
\code{\link{find_files}()},
\code{\link{fromto}()},
\code{\link{missing_docs}},
\code{\link{search_files}()},
\code{\link{summary.filesearch}()}
}
\concept{searching functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wipe_tempdir.R
\name{wipe_tempdir}
\alias{wipe_tempdir}
\title{Wipe Clean the \code{tempdir()}}
\usage{
wipe_tempdir(recreate = FALSE)
}
\arguments{
\item{recreate}{Use the method described in the examples section of
\code{\link{tempdir}} (using \code{tempdir(check = TRUE)}, this results in a
new path.)}
}
\value{
The path to the temporary directory.
}
\description{
I often need a clean temporary directory.
}
\examples{
\dontrun{
dir.create(t <- file.path(tempdir(), "foo"))
touch(f1 <- file.path(t, "first.R"),
      f2 <- file.path(t, "second.R"))
dir(tempdir(), recursive = TRUE)
wipe_tempdir()
dir(tempdir(), recursive = TRUE)
}
}
\seealso{
Other R memory functions: 
\code{\link{memory_hogs}()},
\code{\link{wipe_clean}()}

Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{vim}()},
\code{\link{with_dir}()}
}
\concept{R memory functions}
\concept{operating system functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_difftime_less.R
\name{is_difftime_less}
\alias{is_difftime_less}
\title{Check Whether Two Times Differ Less Than A Given Value}
\usage{
is_difftime_less(
  time1,
  time2,
  less_than = 1,
  units = "days",
  verbose = FALSE,
  visible = !verbose,
  stop_on_error = FALSE
)
}
\arguments{
\item{time1}{See \code{\link{difftime}}.}

\item{time2}{See \code{\link{difftime}}.}

\item{less_than}{The number of \bold{units} that would be too much of a
difference.}

\item{units}{See \code{\link{difftime}}.}

\item{verbose}{Be verbose?}

\item{visible}{Set to \code{\link{FALSE}} to return
\code{\link{invisible}}.}

\item{stop_on_error}{Throw an error if the time lag is not less than
\bold{less_than}.}
}
\value{
\code{\link{TRUE}} if the times do not differ `that much`, but see
\bold{stop_on_error}.
}
\description{
This is just a wrapper to \code{\link{difftime}}.
}
\examples{
a <- as.POSIXct(0, origin = "1970-01-01", tz = "GMT")
b <- as.POSIXct(60*60*24, origin = "1970-01-01", tz = "GMT")
c <- as.POSIXct(60*60*24 - 1, origin = "1970-01-01", tz = "GMT")
is_difftime_less(a, b)
is_difftime_less(a, c)
print(is_difftime_less(a, b, verbose = TRUE))
print(is_difftime_less(a, c, verbose = TRUE))
try(is_difftime_less(a, b, stop_on_error = TRUE))
is_difftime_less(a, c, verbose = TRUE, stop_on_error = TRUE)
}
\seealso{
Other bits and pieces: 
\code{\link{golden_ratio}()},
\code{\link{is_valid_primary_key}()},
\code{\link{r_cmd_install}()},
\code{\link{str2num}()},
\code{\link{strip_off_attributes}()},
\code{\link{tapply}()},
\code{\link{throw}()}
}
\concept{bits and pieces}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/load_internal_functions.R
\name{load_internal_functions}
\alias{load_internal_functions}
\title{Load a Package's Internals}
\usage{
load_internal_functions(package, ...)
}
\arguments{
\item{package}{The name of the package as a string.}

\item{...}{Arguments passed to \code{\link{ls}}, all.names = TRUE could be a
good idea.}
}
\value{
\code{\link[base:invisible]{Invisibly}}
\code{\link[base:TRUE]{TRUE}}.
}
\description{
Load objects not exported from a package's namespace.
}
\examples{
load_internal_functions("fritools2")
}
\seealso{
\code{\link[codetools:checkUsageEnv]{codetools::checkUsageEnv}}.

Other package functions: 
\code{\link{get_package_version}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_version_sufficient}()}
}
\concept{package functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/develop_test.R
\name{develop_test}
\alias{develop_test}
\title{Develop Unit Testing for a Code File}
\usage{
develop_test(file, force_runit = FALSE, force_tiny = TRUE)
}
\arguments{
\item{file}{The path to the code file, assuming the working directory to be
the root of an R package under development.}

\item{force_runit}{If there is no corresponding \pkg{RUnit} test file:
create one?}

\item{force_tiny}{If there is no corresponding \pkg{tinytest} test file:
create one?}
}
\value{
\code{\link[base:invisible]{Invisibly}}
\code{\link[base:NULL]{NULL}}.
}
\description{
Looking at the output of
\code{\link[covr:zero_coverage]{covr::zero_coverage}}, I want to open a code
file an the corresponding unit testing file.
}
\examples{
\dontrun{
    develop_test(file = "R/develop_test.R", force_runit = TRUE)
    unlink("inst/tinytest/test_develop_test.R")
    unlink("inst/runit_tests/runit-develop_test.R")
}
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}
\concept{test_helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/column_sums.R
\name{column_sums}
\alias{column_sums}
\title{Sum up the Numeric Columns of a Data Frame}
\usage{
column_sums(x, ...)
}
\arguments{
\item{x}{A  \code{\link{data.frame}}.}

\item{...}{Arguments passed to \code{\link{colSums}}.}
}
\value{
A named vector of column sums (see \code{\link{colSums}}).
}
\description{
I often need to calculate the sums of the numeric columns of a
\code{\link{data.frame}}. While \code{\link{colSums}} requires the data frame
to be numeric, this is a convenience wrapper to select numeric columns only.
}
\examples{
try(colSums(iris))
column_sums(iris)
names(iris) # no column sum for `Species`
}
\seealso{
Other statistics: 
\code{\link{count_groups}()},
\code{\link{relative_difference}()},
\code{\link{round_half_away_from_zero}()},
\code{\link{weighted_variance}()}
}
\concept{statistics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csv.R
\name{check_ascii_file}
\alias{check_ascii_file}
\title{Check the Number of Lines and Fields in a File}
\usage{
check_ascii_file(path, sep = ";")
}
\arguments{
\item{path}{Path to a file.}

\item{sep}{A character separating the fields in the file.}
}
\value{
A list giving the number of lines, number of fields and an boolean
indicating whether all lines have the same number of fields.
}
\description{
Check the Number of Lines and Fields in a File
}
\examples{
f <- tempfile()
write.csv2(mtcars, file = f)
check_ascii_file(f)
}
\seealso{
Other CSV functions: 
\code{\link{bulk_read_csv}()},
\code{\link{bulk_write_csv}()},
\code{\link{csv2csv}()},
\code{\link{csv}}
}
\concept{CSV functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_cran.R
\name{is_cran}
\alias{is_cran}
\title{Is \R Running on CRAN?}
\usage{
is_cran(cran_pattern, n_r_check4cran)
}
\arguments{
\item{cran_pattern}{A regular expressions to apply to the names of
\code{Sys.getenv()}
   to identify possible CRAN parameters.  Defaults to
   \code{Sys.getenv('_CRAN_pattern_')} if available and '^_R_' if not.}

\item{n_r_check4cran}{Assume this is CRAN if at least n_R_CHECK4CRAN
elements of
   \code{Sys.getenv()} have names matching x.  Defaults to
   \code{Sys.getenv('_n_R_CHECK4CRAN_')} if available and 5 if not.}
}
\value{
A logical scalar with attributes \samp{'sys_getenv'} containing the
 results of \code{Sys.getenv()} and 'matches' containing \code{i} per step 3
 above.
}
\description{
\emph{This is a verbatim copy of \code{fda::CRAN} of
\pkg{fda} version 5.1.9.}
}
\details{
This function allows package developers to run tests themselves that
 should not run on CRAN or with \preformatted{R CMD check --as-cran} because
 of compute time constraints with CRAN tests.

 The "Writing R Extensions" manual says that \command{R CMD check} can be
 customized "by setting environment variables _R_CHECK_*_:, as
 described in" the Tools section of the "R Internals" manual.

 \command{R CMD check} was tested with R 3.0.1 under Fedora 18 Linux and with
 \command{Rtools} 3.0 from April 16, 2013 under Windows 7.  With the
 \preformatted{'--as-cran'}
 option, 7 matches were found;  without it, only 3 were found.  These
 numbers were unaffected by the presence or absence of the '--timings'
 parameter.  On this basis, the default value of n_R_CHECK4CRAN was set
 at 5.

 1.  \code{x. <- Sys.getenv()}

 2.  Fix \code{CRAN_pattern} and \code{n_R_CHECK4CRAN} if missing.

 3.  Let i be the indices of x. whose names match all the patterns in
 the vector x.

 4.  Assume this is CRAN if length(i) >= n_R_CHECK4CRAN
}
\examples{
if (!is_cran()) {
    message("Run your tests here.")
}
}
\seealso{
Other test helpers: 
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{run_r_tests_for_known_hosts}()},
\code{\link{set_run_r_tests}()}

Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}
}
\concept{logical helpers}
\concept{test helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/file_copy.R
\name{file_copy}
\alias{file_copy}
\title{Force Copying a File While backing it up}
\usage{
file_copy(from, to, stop_on_error = FALSE, ...)
}
\arguments{
\item{from}{See \code{\link{file.copy}}.}

\item{to}{See \code{\link{file.copy}}.}

\item{stop_on_error}{Throw an exception on error?}

\item{...}{Arguments passed to \code{\link{file.copy}}.}
}
\value{
A vector of  \code{\link[base:TRUE]{boolean}} values indicating
success or failure.
}
\description{
\code{\link{file.copy}} has an argument \code{overwrite} that allows for
overwriting existing files. But I often want to overwrite an existing file
while creating a backup copy of that file.
}
\examples{
touch(f1 <- file.path(tempdir(), "first.R"),
      f2 <- file.path(tempdir(), "second.R"))
dir.create(t <- file.path(tempdir(), "foo"))
file_copy(from = c(f2, f1), to = t)
dir(t)
touch(f1)
touch(f2)
file_copy(from = c(f2, f1), to = t)
dir(t)
list.files(tempdir(), pattern = "first.*\\\\.R")
dir <- file.path(tempdir(), "subdir")
dir.create(dir)
file_copy(f1, dir)
touch(f1)
file_copy(f1, dir)
list.files(dir, pattern = "first.*\\\\.R")
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}

Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_save}()},
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{vim}()},
\code{\link{wipe_tempdir}()},
\code{\link{with_dir}()}
}
\concept{file utilities}
\concept{operating system functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fromto.R
\name{fromto}
\alias{fromto}
\title{Extract All Items of a Vector Between Two Patterns}
\usage{
fromto(
  x,
  from,
  to,
  from_i = 1,
  to_i = 1,
  shift_from = 0,
  shift_to = 0,
  remove_empty_item = TRUE
)
}
\arguments{
\item{x}{A vector.}

\item{from}{A pattern, use NA to start with the first item.}

\item{to}{Another pattern, use NA to stop with the last item.}

\item{from_i}{If the from pattern matches multiple times, which one is to be
used.}

\item{to_i}{Analogously to to_i.}

\item{shift_from}{The number of items to shift from the item selected via
from and from_i.}

\item{shift_to}{Analogously to shift_from.}

\item{remove_empty_item}{Remove empty items?}
}
\value{
The extracted vector.
}
\description{
This comes in handy to cut lines from a file read by \code{\link{readLines}}.
}
\examples{
foo <- c("First", "f1", "A", "f2", rep("B", 4), "t1", "f3", "C", "t2",
         rep("D", 4), "t3", "Last")
fromto(foo, "^f", "^t")
fromto(foo, NA, "^t")
fromto(foo, "^f", NA)
fromto(foo, "^f", "^t", from_i = 2)
fromto(foo, "^f", "^t", from_i = 2, to_i = 2)
fromto(foo, "^f", "^t", from_i = 2, to_i = 2, shift_from = 1, shift_to = -1)
fromto(foo, "^f", "^t", from_i = 2, to_i = 2, shift_from = -1, shift_to = 2)
}
\seealso{
Other searching functions: 
\code{\link{compare_vectors}()},
\code{\link{file_modified_last}()},
\code{\link{find_files}()},
\code{\link{missing_docs}},
\code{\link{search_files}()},
\code{\link{search_rows}()},
\code{\link{summary.filesearch}()}
}
\concept{searching functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/file_modified_last.R
\name{file_modified_last}
\alias{file_modified_last}
\title{Get the File Modified Last}
\usage{
file_modified_last(...)
}
\arguments{
\item{...}{Arguments passed to \code{\link{find_files}}.}
}
\value{
The path to the file last modified.
}
\description{
I often look for the  file modified last under some directory.
}
\examples{
for (suffix in c(".txt", ".ascii"))
    for (f in file.path(tempdir(), letters))
        touch(paste0(f, suffix))
list.files(tempdir())
file_modified_last(path = tempdir(), pattern = "\\\\.txt$")
dir.create(file.path(tempdir(), "new"))
touch(file.path(tempdir(), "new", "file.txt"))
file_modified_last(path = tempdir(), pattern = "\\\\.txt$")
file_modified_last(path = tempdir(), pattern = "\\\\.txt$", recursive = TRUE)
}
\seealso{
Other searching functions: 
\code{\link{compare_vectors}()},
\code{\link{find_files}()},
\code{\link{fromto}()},
\code{\link{missing_docs}},
\code{\link{search_files}()},
\code{\link{search_rows}()},
\code{\link{summary.filesearch}()}

Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}
\concept{searching functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_of_length_zero.R
\name{is_of_length_zero}
\alias{is_of_length_zero}
\title{Is an Object of Length Zero?}
\usage{
is_of_length_zero(x, class = NULL)
}
\arguments{
\item{x}{The object.}

\item{class}{An optional character vector of length 1 giving the class. See
\emph{examples}.}
}
\value{
\code{\link[base:TRUE]{TRUE}} on success, 
\code{\link[base:FALSE]{FALSE}} otherwise.
}
\description{
Some expressions evaluate to \code{\link{integer}(0)} or the like.
}
\examples{
x <- ""; length(x); is_of_length_zero(x)
x <- grep(" ", "")
print(x)
is_of_length_zero(x)
is_of_length_zero(x, "character")
is_of_length_zero(x, "numeric")
is_of_length_zero(x, "integer")
}
\seealso{
Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}
}
\concept{logical helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/strip_off_attributes.R
\name{strip_off_attributes}
\alias{strip_off_attributes}
\title{Strip Attributes off an Object}
\usage{
strip_off_attributes(x)
}
\arguments{
\item{x}{An object.}
}
\value{
The object.
}
\description{
Strip Attributes off an Object
}
\examples{
y <- stats::setNames(1:3, letters[1:3])
attr(y, "myattr") <- "qwer"
comment(y) <- "qwer"
strip_off_attributes(y)
}
\seealso{
\code{\link[base:unname]{base::unname}}

Other bits and pieces: 
\code{\link{golden_ratio}()},
\code{\link{is_difftime_less}()},
\code{\link{is_valid_primary_key}()},
\code{\link{r_cmd_install}()},
\code{\link{str2num}()},
\code{\link{tapply}()},
\code{\link{throw}()}
}
\concept{bits and pieces}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_helpers.R
\name{run_r_tests_for_known_hosts}
\alias{run_r_tests_for_known_hosts}
\title{Force Testing on Known Hosts}
\usage{
run_r_tests_for_known_hosts()
}
\value{
\code{\link[base:invisible]{Invisibly}}
\code{\link[base:NULL]{NULL}}.
}
\description{
Enforce the environment variable RUN_R_TESTS to TRUE on known hosts.
}
\details{
This should go into \code{\link{.onLoad}} to force tests on known hosts.
}
\examples{
get_run_r_tests()
if (isFALSE(get_run_r_tests())) {
    run_r_tests_for_known_hosts()
    get_run_r_tests()
}
}
\seealso{
Other test helpers: 
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_cran}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{set_run_r_tests}()}
}
\concept{test helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_umlauts_to_tex.R
\name{convert_umlauts_to_tex}
\alias{convert_umlauts_to_tex}
\title{Tex Codes for German Umlauts}
\usage{
convert_umlauts_to_tex(x)
}
\arguments{
\item{x}{A string.}
}
\value{
A string with the umlauts converted to plain TeX.
}
\description{
Convert German umlauts in a string to their plain TeX representation.
}
\examples{
string <- paste("this is \u00e4 string")
print(string)
print(convert_umlauts_to_tex(string))
}
\seealso{
Other German umlaut converters: 
\code{\link{convert_umlauts_to_ascii}()}
}
\concept{German umlaut converters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_helpers.R
\name{get_run_r_tests}
\alias{get_run_r_tests}
\title{Get System Variable RUN_R_TESTS}
\usage{
get_run_r_tests(stop_on_failure = FALSE)
}
\arguments{
\item{stop_on_failure}{Throw an error instead of returning
\code{\link{FALSE}} if the environment variable is not set or cannot be
converted to boolean.}
}
\value{
The value RUN_R_TESTS is set to, converted to boolean.
\code{\link{FALSE}} if RUN_R_TESTS is not set or cannot be converted to
boolean.
}
\description{
A convenience wrapper to
\code{\link{get_boolean_envvar}("RUN_R_TESTS")}.
}
\examples{
set_run_r_tests("", force = TRUE) # make sure it is not set.
get_run_r_tests()
try(get_run_r_tests(stop_on_failure = TRUE))
set_run_r_tests("A", force = TRUE) # "A" is not boolean.
get_run_r_tests()
try(get_run_r_tests(stop_on_failure = TRUE))
set_run_r_tests(4213, force = TRUE) # All numbers apart from 0 are TRUE
get_run_r_tests()
set_run_r_tests("0", force = TRUE) # 0 (and "0") is FALSE
get_run_r_tests()
set_run_r_tests("FALSE", force = TRUE)
get_run_r_tests()
set_run_r_tests(TRUE, force = TRUE)
get_run_r_tests()
}
\seealso{
Other test helpers: 
\code{\link{get_boolean_envvar}()},
\code{\link{is_cran}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{run_r_tests_for_known_hosts}()},
\code{\link{set_run_r_tests}()}

Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{get_boolean_envvar}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{vim}()},
\code{\link{wipe_tempdir}()},
\code{\link{with_dir}()}

Other logical helpers: 
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}
}
\concept{logical helpers}
\concept{operating system functions}
\concept{test helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_version_sufficient.R
\name{is_version_sufficient}
\alias{is_version_sufficient}
\title{Is a Version Requirement Met?}
\usage{
is_version_sufficient(installed, required)
}
\arguments{
\item{installed}{The version available.}

\item{required}{The version required.}
}
\value{
\code{\link{TRUE}}, if so,  \code{\link{FALSE}} otherwise.
}
\description{
Just a wrapper to \code{\link{compareVersion}}, I regularly forget how to use
it.
}
\examples{
is_version_sufficient(installed = "1.0.0", required = "2.0.0")
is_version_sufficient(installed = "1.0.0", required = "1.0.0")
is_version_sufficient(installed = get_package_version("base"),
                      required = "3.5.2")
}
\seealso{
Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_windows}()}

Other package functions: 
\code{\link{get_package_version}()},
\code{\link{is_r_package_installed}()},
\code{\link{load_internal_functions}()}

Other version functions: 
\code{\link{get_package_version}()},
\code{\link{is_r_package_installed}()}
}
\concept{logical helpers}
\concept{package functions}
\concept{version functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/wipe_clean.R
\name{wipe_clean}
\alias{wipe_clean}
\title{Remove All Objects From an Environment}
\usage{
wipe_clean(environment = getOption("wipe_clean_environment"), all_names = TRUE)
}
\arguments{
\item{environment}{The environment that should be wiped clean.
Defaults to \code{\link{.GlobalEnv}}.}

\item{all_names}{See argument \code{all.names} for \code{\link{ls}}.}
}
\value{
A character vector containing the names of objects removed, but
called for its side effect of removing all objects from the environment.
}
\description{
Wipe an environment, typically \code{\link{.GlobalEnv}}, clean.
}
\examples{
an_object <- 1
wipe_clean()
ls()
e <- new.env()
assign("a", 1, envir = e)
assign("b", 1, envir = e)
ls(envir = e)
wipe_clean(envir = e)
ls(envir = e)
RUnit::checkIdentical(length(ls(envir = e)), 0L)
}
\seealso{
Other R memory functions: 
\code{\link{memory_hogs}()},
\code{\link{wipe_tempdir}()}
}
\concept{R memory functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/clipboard_path.R
\name{clipboard_path}
\alias{clipboard_path}
\title{Copy a Path from Clipboard to \R}
\usage{
clipboard_path()
}
\value{
The de-windowsified path.
}
\description{
I often have to work under Windows, where file paths cannot just be pasted
into the code, so I adapted code from
\url{https://www.r-bloggers.com/2015/12/stop-fiddling-
around-with-copied-paths-in-windows-r/}.
Under Windows, the de-windowsified path is copied to the clipboard.
}
\note{
It makes only sense to call \code{clipboard_path} in an interactive \R
session.
}
\seealso{
Other operating system functions: 
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{vim}()},
\code{\link{wipe_tempdir}()},
\code{\link{with_dir}()}

Other file utilities: 
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}
\concept{operating system functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_files_current.R
\name{is_files_current}
\alias{is_files_current}
\title{Check Whether Files are Current}
\usage{
is_files_current(
  ...,
  newer_than = 1,
  units = "week",
  within = 1,
  within_units = "days"
)
}
\arguments{
\item{...}{File paths.}

\item{newer_than}{The number of \bold{units} the files need to be newer than.}

\item{units}{The unit of \bold{newer_than}. See \code{\link{difftime}}.}

\item{within}{The number of \bold{units} the files need to be modified
within.}

\item{within_units}{The unit of \bold{within}. See \code{\link{difftime}}.}
}
\value{
\code{\link[base:TRUE]{TRUE}} on success, 
\code{\link[base:FALSE]{FALSE}} otherwise.
}
\description{
I sometimes produce a couple of files by some kind of process and need to
check whether they are fairly current and probably product of the same run.
So I need to know whether a bunch of files was modified within the
last, say, 7 days \emph{and} that their modification dates do not differ by
more than, say, 24 hours.
}
\examples{
p1 <- tempfile()
p2 <- tempfile()
p3 <- tempfile()
touch(p1)
touch(p2)
Sys.sleep(3)
touch(p3)
is_files_current(p3, newer_than = 1, units = "days",
                 within = 4, within_units = "secs")
is_files_current(p1, p2, p3, newer_than = 1, units = "days",
                 within = 4, within_units = "secs")
is_files_current(p1, p2, p3, newer_than = 1, units = "days",
                 within = 1, within_units = "secs")
is_files_current(p1, p2, p3, newer_than = 1, units = "secs",
                 within = 4, within_units = "secs")
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/round_half_away_from_zero.R
\name{round_half_away_from_zero}
\alias{round_half_away_from_zero}
\alias{round_commercially}
\title{Round Half Away From Zero}
\usage{
round_half_away_from_zero(x, digits = 0)

round_commercially(x, digits = 0)
}
\arguments{
\item{x}{A number to be rounded.}

\item{digits}{The number of digits, as in \code{\link{round}}.}
}
\value{
The rounded number.
}
\description{
Commercial rounding is done a lot, especially with invoices. There is even
standard 1333 by the German Institute for Standardization.
\code{\link{round}} rounds half to even, see \code{\link{round}}'s Details
section.

\code{round_commercially} is just a link to \code{round_half_away_from_zero}.
}
\examples{
x <- 22.5
round_half_away_from_zero(x)
round(x)
round_half_away_from_zero(-x)
round(-x)
}
\seealso{
Other statistics: 
\code{\link{column_sums}()},
\code{\link{count_groups}()},
\code{\link{relative_difference}()},
\code{\link{weighted_variance}()}
}
\concept{statistics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch.R
\name{get_script_name}
\alias{get_script_name}
\title{Get the Name of the R Code File or set it to \code{default}}
\usage{
get_script_name(default = "interactive_R_session")
}
\arguments{
\item{default}{the name to return if R is run interactively.}
}
\value{
A vector of \code{\link{length}} 1 and \code{\link{mode}}
character giving the name of the R code file if R was run via
\command{R CMD BATCH} or
\command{Rscript}, the given default otherwise.
}
\description{
The code file name is retrieved only for \command{R CMD BATCH} and
\command{Rscript},
if R is used interactively, the name is set to \code{default},
even if you're working with code stored in a (named) file on disk.
}
\examples{
get_script_name(default = 'foobar.R')
}
\seealso{
Other script path getter functions: 
\code{\link{get_r_cmd_batch_script_path}()},
\code{\link{get_rscript_script_path}()},
\code{\link{get_script_path}()}
}
\concept{script path getter functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/find_files.R
\name{find_files}
\alias{find_files}
\title{Find Files on Disk}
\usage{
find_files(
  path = ".",
  pattern = ".*\\\\.[RrSs]$|.*\\\\.[RrSs]nw$",
  file_names = NA,
  all_files = TRUE,
  recursive = FALSE,
  ignore_case = FALSE,
  find_all = FALSE,
  select = NA
)
}
\arguments{
\item{path}{see \code{\link{list.files}}.}

\item{pattern}{see \code{\link{list.files}}.}

\item{file_names}{character vector of file names (to be checked if the files
exist).}

\item{all_files}{see \code{\link{list.files}}, argument \code{all.files}.}

\item{recursive}{see \code{\link{list.files}}.}

\item{ignore_case}{see \code{\link{list.files}}, argument \code{ignore.case}.}

\item{find_all}{Throw an error if not all files (given by \emph{file_names})
are found?}

\item{select}{A named list of numerical vectors of maximum length 2 named
\code{min} and/or \code{max}.
If given, file searching will be restricted to file attributes corresponding
to the names in the list ranging between \code{min} and \code{max}. See
\emph{examples}.}
}
\value{
A character vector of file names.
}
\description{
Look for files on disk, either scanning a vector of names or searching for
files with \code{\link{list.files}} and throw an error if no files are found.
}
\details{
This is a wrapper to either \code{\link{file.exists}} or
\code{\link{list.files}}, that ensures that (some) files exists. This may
come handy if you want to perform some kind of file manipulation e.g. with
one of the functions listed under \cr
\strong{See Also} \emph{Other file utilities:}.
}
\note{
This is merely a wrapper around \code{\link{file.exists}} or
      \code{\link{list.files}}, depending on whether \emph{file_names} is
      given.
}
\examples{
#\% create some files
files <- unname(sapply(file.path(tempdir(), paste0(sample(letters, 10),
                                              ".", c("R", "Rnw", "txt"))),
                       touch))
print(files)
print(list.files(tempdir(), full.names = TRUE)) # same as above
#\% file names given
find_files(file_names = files[1:3])
##\% some do not exist:
find_files(file_names = c(files[1:3], replicate(2, tempfile())))
try(find_files(file_names = c(files[1:3], replicate(2, tempfile())),
               find_all = TRUE))
##\% all do not exist:
try(find_files(file_names = replicate(2, tempfile())))
#\% path given
find_files(path = tempdir())
##\% change pattern
find_files(path = tempdir(),
           pattern = ".*\\\\.[RrSs]$|.*\\\\.[RrSs]nw$|.*\\\\.txt")
##\% find a specific file by it's basename
find_files(path = tempdir(), pattern = paste0("^", basename(files[1]), "$"))
#\% file_names and path given: file_names beats path
try(find_files(file_names = tempfile(), path = tempdir()))
#\% select by file size:
write.csv(mtcars, file.path(tempdir(), "mtcars.csv"))
find_files(path = tempdir(), pattern = ".*")
find_files(path = tempdir(), pattern = ".*",
           select = list(size = c(min = 1000))
           )
}
\seealso{
Other searching functions: 
\code{\link{compare_vectors}()},
\code{\link{file_modified_last}()},
\code{\link{fromto}()},
\code{\link{missing_docs}},
\code{\link{search_files}()},
\code{\link{search_rows}()},
\code{\link{summary.filesearch}()}

Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}
\concept{searching functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search_files.R
\name{summary.filesearch}
\alias{summary.filesearch}
\title{Summarize File Searches}
\usage{
\method{summary}{filesearch}(object, ..., type = c("file", "what", "matches"))
}
\arguments{
\item{object}{An object returned by \code{\link{search_files}}.}

\item{...}{Needed for compatibility.}

\item{type}{Type of summary.}
}
\value{
A summarized object.
}
\description{
A custom summary function for objects returned by \code{\link{search_files}}.
}
\examples{
write.csv(mtcars, file.path(tempdir(), "mtcars.csv"))
 for (i in 0:9) {
     write.csv(iris, file.path(tempdir(), paste0("iris", i, ".csv")))
 }
search_files(what = "Mazda", path = tempdir(), pattern = "^.*\\\\.csv$")
search_files(what = "[Ss]etosa", path = tempdir(), pattern = "^.*\\\\.csv$")
x <- search_files(path = tempdir(),
                  pattern = "^.*\\\\.csv$",
                  exclude = "[2-9]\\\\.csv$",
                  what = "[Ss]etosa")
summary(x)
summary(x, type = "what")
summary(x, type = "matches")
try(search_files(what = "ABC", path = tempdir(), pattern = "^.*\\\\.csv$"))
}
\seealso{
Other searching functions: 
\code{\link{compare_vectors}()},
\code{\link{file_modified_last}()},
\code{\link{find_files}()},
\code{\link{fromto}()},
\code{\link{missing_docs}},
\code{\link{search_files}()},
\code{\link{search_rows}()}
}
\concept{searching functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{set_options}
\alias{set_options}
\title{Set Options For Packages}
\usage{
set_options(..., package_name = .packages()[1], overwrite = TRUE)
}
\arguments{
\item{...}{See \code{\link{options}}.}

\item{package_name}{The package's name.}

\item{overwrite}{[boolean(1)]\cr Overwrite options already set?}
}
\value{
\code{\link[base:invisible]{Invisibly}}
\code{\link[base:TRUE]{TRUE}}.
}
\description{
A convenience function for \code{\link{options}}.
}
\examples{
options("cleanr" = NULL)
defaults <- list(max_file_width = 80, max_file_length = 300,
                 max_lines = 65, max_lines_of_code = 50,
                 max_num_arguments = 5, max_nesting_depth = 3,
                 max_line_width = 80, check_return = TRUE)

set_options(package_name = "cleanr", defaults)
getOption("cleanr")
set_options(package_name = "cleanr", list(max_line_width = 3,
            max_lines = "This is nonsense!"))
set_options(package_name = "cleanr", check_return = NULL, max_lines = 4000)
get_options(package_name = "cleanr")
}
\seealso{
Other option functions: 
\code{\link{get_options}()},
\code{\link{is_force}()}
}
\concept{option functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csv.R
\name{bulk_write_csv}
\alias{bulk_write_csv}
\title{Bulk Write Comma Separated Files}
\usage{
bulk_write_csv(x, ...)
}
\arguments{
\item{x}{A list of objects to be written to \command{csv}.}

\item{...}{Arguments passed to
\code{\link{write_csv}}.}
}
\value{
The list holding the return values of \code{\link{write_csv}}.
}
\description{
Write a bunch of objects to disk using \code{\link{write_csv}}.
}
\examples{
unlink(dir(tempdir(), full.names = TRUE))
data(mtcars)
mt_german <- mtcars
rownames(mt_german)[1] <- "Mazda R\u00f64"
names(mt_german)[1] <- "mg\u00dc"
for (i in 1:10) {
    f <- file.path(tempdir(), paste0("f", i, ".csv"))
    write.csv(mtcars[1:5, TRUE], file = f)
    f <- file.path(tempdir(), paste0("f", i, "_german.csv"))
    write.csv2(mt_german[1:7, TRUE], file = f, fileEncoding = "Latin1")
}
#\% read
bulk <- bulk_read_csv(tempdir())

print(mtime <- file.info(list.files(tempdir(), full.names = TRUE))["mtime"])
bulk[["f2"]][3, 5] <- bulk[["f2"]][3, 5] + 2
Sys.sleep(2) # make sure the mtimes would change
result <- bulk_write_csv(bulk)
print(new_times <- file.info(dir(tempdir(), full.names = TRUE))["mtime"])
index_change <- grep("f2\\\\.csv", rownames(mtime))
if (requireNamespace("digest", quietly = TRUE)) {
    only_f2_changed <- all((mtime == new_times)[-c(index_change)]) &&
        (mtime < new_times)[c(index_change)]
    RUnit::checkTrue(only_f2_changed)
} else {
    RUnit::checkTrue(all(mtime < new_times))
}
}
\seealso{
Other CSV functions: 
\code{\link{bulk_read_csv}()},
\code{\link{check_ascii_file}()},
\code{\link{csv2csv}()},
\code{\link{csv}}
}
\concept{CSV functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boolean.R
\name{is_not_false}
\alias{is_not_false}
\title{Is an Object Set and not Set to \code{\link[base:FALSE]{FALSE}}?}
\usage{
is_not_false(x, null_is_false = TRUE, ...)
}
\arguments{
\item{x}{The object to be tested.}

\item{null_is_false}{Should \code{\link[base:NULL]{NULL}} be treated as
\code{\link[base:FALSE]{FALSE}}?}

\item{...}{Parameters passed to \code{\link{exists}}. See Examples.}
}
\value{
\code{\link[base:TRUE]{TRUE}} if the object is set to something
different than \code{\link[base:FALSE]{FALSE}},
\code{\link[base:FALSE]{FALSE}}
otherwise.
}
\description{
Sometimes you need to know whether or not an object exists and is not set to
\code{\link[base:FALSE]{FALSE}} (and possibly not
\code{\link[base:NULL]{NULL}}).
}
\examples{
a  <- 1
b  <- FALSE
c  <- NULL
is_not_false(a)
is_not_false(b)
is_not_false(c)
is_not_false(c, null_is_false = FALSE)
is_not_false(not_defined)
f <- function() {
    print(a)
    print(is_not_false(a))
}
f()

f <- function() {
    a <- FALSE
    print(a)
    print(is_not_false(a))
}
f()

f <- function() {
    print(a)
    print(is_not_false(a, null_is_false = TRUE,
                       inherits = FALSE))
}
f()
### We use this to check whether an option is set to something
### different than FALSE:
# Make sure an option is not set:
set_options("test" = NULL, package = "fritools2")
tmp <- get_options("test")
is_not_false(tmp)
is_not_false(tmp, null_is_false = FALSE)
# Does not work on the option directly as it is not an object defined:
options("foo" = NULL)
is_not_false(getOption("foo"), null_is_false = FALSE)
}
\seealso{
Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}
}
\concept{logical helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/boolean.R
\name{is_null_or_true}
\alias{is_null_or_true}
\title{Is an Object \code{\link[base:TRUE]{TRUE}} or \code{\link[base:NULL]{NULL}}?}
\usage{
is_null_or_true(x)
}
\arguments{
\item{x}{The object to be tested.}
}
\value{
\code{\link[base:TRUE]{TRUE}} if the object is set to
\code{\link[base:TRUE]{TRUE}} or \code{\link[base:NULL]{NULL}},
\code{\link[base:FALSE]{FALSE}}
otherwise.
}
\description{
Is an object \code{\link[base:TRUE]{TRUE}} or \code{\link[base:NULL]{NULL}}?
}
\examples{
is_null_or_true("true") # FALSE
is_null_or_true(TRUE) # TRUE
is_null_or_true(NULL) # TRUE
suppressWarnings(rm("not_defined"))
try(is_null_or_true(not_defined)) # error
}
\seealso{
Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}
}
\concept{logical helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/memory_hogs.R
\name{memory_hogs}
\alias{memory_hogs}
\title{Find Memory Hogs}
\usage{
memory_hogs(
  unit = c("b", "Kb", "Mb", "Gb", "Tb", "Pb"),
  return_numeric = TRUE,
  ...,
  envir = .GlobalEnv
)
}
\arguments{
\item{unit}{The unit to use.}

\item{return_numeric}{Return a numeric vector? If set to
\code{\link{FALSE}}, a character vector including the unit will be returned,
which might be less usable but easier to read.}

\item{...}{Arguments passed to \code{\link{order}}, defaults to
\code{decreasing = FALSE}.}

\item{envir}{The environment where to look for objects.}
}
\value{
A named vector of memory usages.
}
\description{
List objects in an \R environment by size.
}
\examples{
va <- rep(mtcars, 1)
vb <- rep(mtcars, 1000)
vc <- rep(mtcars, 2000)
vd <- rep(mtcars, 100)
memory_hogs()
memory_hogs(unit = "Mb", decreasing = TRUE)
memory_hogs(unit = "Mb", decreasing = TRUE, return_numeric = FALSE)
\dontrun{
# remove the two largest objects:
rm(list = names(tail(memory_hogs(decreasing = FALSE), n = 2)))
memory_hogs(unit = "Mb")
}
}
\seealso{
Other R memory functions: 
\code{\link{wipe_clean}()},
\code{\link{wipe_tempdir}()}
}
\concept{R memory functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_helpers.R
\name{set_run_r_tests}
\alias{set_run_r_tests}
\title{Set the System Variable RUN_R_TESTS}
\usage{
set_run_r_tests(x, force = FALSE)
}
\arguments{
\item{x}{A logical, typically some function output.}

\item{force}{Overwrite the variable if already set?}
}
\value{
The value RUN_R_TESTS is set to, \code{\link{NULL}} if nothing is
done.
}
\description{
A convenience wrapper to  \code{\link{Sys.getenv}} for setting
\kbd{RUN_R_TESTS}.
}
\examples{
set_run_r_tests(is_running_on_fvafrcu_machines())
get_run_r_tests()
set_run_r_tests(TRUE, force = TRUE)
get_run_r_tests()
}
\seealso{
Other test helpers: 
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_cran}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{run_r_tests_for_known_hosts}()}
}
\concept{test helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_helpers.R
\name{is_running_on_fvafrcu_machines}
\alias{is_running_on_fvafrcu_machines}
\title{Is the Machine Running the Current \R Process Owned by FVAFRCU?}
\usage{
is_running_on_fvafrcu_machines(type = c("any", "cu", "fvafr"))
}
\arguments{
\item{type}{An optional selection.}
}
\value{
\code{\link[base:TRUE]{TRUE}} on success, 
\code{\link[base:FALSE]{FALSE}} otherwise.
}
\description{
Is the machine running the current \R process known to me?
}
\examples{
is_running_on_fvafrcu_machines()
}
\seealso{
Other test helpers: 
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_cran}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{run_r_tests_for_known_hosts}()},
\code{\link{set_run_r_tests}()}

Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}
}
\concept{logical helpers}
\concept{test helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/r.R
\name{r_cmd_install}
\alias{r_cmd_install}
\title{Install a Tarball or a Directory}
\usage{
r_cmd_install(
  ...,
  path = ".",
  r = get_r_binary(),
  r_args = "--vanilla",
  try_tarball = TRUE
)
}
\arguments{
\item{...}{Arguments passed to \command{CMD INSTALL}.}

\item{path}{A path to a directory, but see \emph{try_tarball}.}

\item{r}{An R binary.}

\item{r_args}{Arguments passed to \command{R}.}

\item{try_tarball}{If \code{\link{TRUE}}, the \code{path} may be a tarball,
else it is interpreted as a path to a directory.}
}
\value{
The return value of \code{\link{system2}}.
}
\description{
\code{\link[devtools:install]{devtools::install}} by defaults first builds
the tarball. Then it calls
\code{\link[callr:rcmd]{callr::rcmd}}, which allows for a lot of options.
}
\seealso{
Other bits and pieces: 
\code{\link{golden_ratio}()},
\code{\link{is_difftime_less}()},
\code{\link{is_valid_primary_key}()},
\code{\link{str2num}()},
\code{\link{strip_off_attributes}()},
\code{\link{tapply}()},
\code{\link{throw}()}
}
\concept{bits and pieces}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/convert_umlauts_to_ascii.R
\name{convert_umlauts_to_ascii}
\alias{convert_umlauts_to_ascii}
\alias{convert_umlauts_to_ascii.character}
\alias{convert_umlauts_to_ascii.data.frame}
\title{Convert German umlauts to a more or less suitable ascii representation.}
\usage{
convert_umlauts_to_ascii(x)

\method{convert_umlauts_to_ascii}{character}(x)

\method{convert_umlauts_to_ascii}{data.frame}(x)
}
\arguments{
\item{x}{A string or \code{data.frame}.}
}
\value{
\bold{x} with the umlauts converted to ascii.
}
\description{
Convert German umlauts to a more or less suitable ascii representation.
}
\examples{
string <- paste("this is \u00e4 string")
print(string)
print(convert_umlauts_to_ascii(string))
string <- paste("this is \u00e4 string")
df <- data.frame(v1 = c(string, "foobar"),
                 v2 = c("foobar", string), v3 = 3:4)
names(df)[3] <- "y\u00dfy"
convert_umlauts_to_ascii(df)
}
\seealso{
Other German umlaut converters: 
\code{\link{convert_umlauts_to_tex}()}
}
\concept{German umlaut converters}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/weighted_variance.R
\name{weighted_variance}
\alias{weighted_variance}
\alias{weighted_variance.numeric}
\alias{weighted_variance.data.frame}
\title{Calculate a Weighted Variance}
\usage{
weighted_variance(x, ...)

\method{weighted_variance}{numeric}(x, weights, weights_counts = NULL, ...)

\method{weighted_variance}{data.frame}(x, var, weight, ...)
}
\arguments{
\item{x}{A numeric \code{\link{vector}} or \code{\link{data.frame}}.}

\item{...}{Other arguments ignored.}

\item{weights}{A vector of weights.}

\item{weights_counts}{Are the weights counts of the data? If so, we can
calculate the unbiased sample variance, otherwise we calculate the biased
(maximum likelihood estimator of the) sample variance.}

\item{var}{The name of the column in \code{x} giving the variable of
interest.}

\item{weight}{The name of the column in \code{x} giving the weights.}
}
\description{
Calculate a weighted variance.
}
\details{
The  \code{\link{data.frame}} method is meant for use with
\code{\link{tapply}}, see \emph{examples}.
}
\examples{
## GPA from Siegel 1994
wt <- c(5,  5,  4,  1)/15
x <- c(3.7,3.3,3.5,2.8)
var(x)
weighted_variance(x = x)
weighted_variance(x = x, weights = wt)
weighted_variance(x = x, weights = wt, weights_counts = TRUE)
weights <- c(5,  5,  4,  1)
weighted_variance(x = x, weights = weights)
weighted_variance(x = x, weights = weights, weights_counts = FALSE)
weighted_variance(x = data.frame(x, wt), var = "x",
                             weight = "wt")
# apply by groups:
fritools2::tapply(object = mtcars,
                 index = list(mtcars[["cyl"]], mtcars[["vs"]]),
                 func = weighted_variance, var = "mpg", w = "wt")
}
\seealso{
Other statistics: 
\code{\link{column_sums}()},
\code{\link{count_groups}()},
\code{\link{relative_difference}()},
\code{\link{round_half_away_from_zero}()}
}
\concept{statistics}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/call.R
\name{call_conditionally}
\alias{call_conditionally}
\title{Call a Function Conditionally}
\usage{
call_conditionally(f, condition, fallback, ..., harden = FALSE)
}
\arguments{
\item{f}{The function passed to \code{\link{do.call}}.}

\item{condition}{An expression.}

\item{fallback}{See \emph{Description}.}

\item{...}{arguments passed to \code{\link{do.call}}.}

\item{harden}{Set to \code{\link{TRUE}} to return \code{fallback} if
\code{\link{do.call}} fails.}
}
\value{
The return value of \code{f} or \code{fallback}.
}
\description{
\pkg{whoami} 1.3.0 uses things like
\code{system("getent passwd $(whoami)", intern = TRUE)}
which I can not \code{\link{tryCatch}}, as it gives no error nor warning.
So this function returns a fallback if the condition given is not
\code{\link{TRUE}}.
}
\examples{
call_conditionally(get_package_version,
                   condition = TRUE,
                   args = list(x = "fritools2"),
                   fallback = "0.0")
call_conditionally(get_package_version,
                   condition = FALSE,
                   args = list(x = "fritools2"),
                   fallback = "0.0")
call_conditionally(get_package_version,
                   condition = TRUE,
                   args = list(x = "not_there"),
                   harden = TRUE,
                   fallback = "0.0")
}
\seealso{
Other call functions: 
\code{\link{call_safe}()}
}
\concept{call functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_valid_primary_key.R
\name{is_valid_primary_key}
\alias{is_valid_primary_key}
\title{Is a Key a Valid Potential Primary Key for a \code{data.frame}?}
\usage{
is_valid_primary_key(data, key, verbose = TRUE)
}
\arguments{
\item{data}{The \code{data.frame} for which you want to find valid potential
primary key.}

\item{key}{Character vector containing a subset of the columns names of
\code{data}.}

\item{verbose}{Be verbose?}
}
\value{
\code{\link{TRUE}}, if \code{key} is a valid primary key,
\code{\link{FALSE}} otherwise.
}
\description{
I sometimes see tables with obscure structure so I try to guess their primary
keys.
}
\examples{
is_valid_primary_key(mtcars, "qsec")
is_valid_primary_key(mtcars, "carb")
is_valid_primary_key(mtcars, c("qsec", "gear"))
is_valid_primary_key(mtcars, c("qsec", "carb"))
cars <- mtcars
cars$id <-  seq_len(nrow(cars))
is_valid_primary_key(cars, "id")
}
\seealso{
Other bits and pieces: 
\code{\link{golden_ratio}()},
\code{\link{is_difftime_less}()},
\code{\link{r_cmd_install}()},
\code{\link{str2num}()},
\code{\link{strip_off_attributes}()},
\code{\link{tapply}()},
\code{\link{throw}()}
}
\concept{bits and pieces}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/compare_vectors.R
\name{compare_vectors}
\alias{compare_vectors}
\title{Compare Two Vectors}
\usage{
compare_vectors(x, y, differences_only = FALSE)
}
\arguments{
\item{x, y}{Two vectors of the same mode.}

\item{differences_only}{Report only the differences?}
}
\value{
A matrix containing the side-by-side comparison.
}
\description{
Side-by-side comparison of two vectors.
The vectors get sorted and are compared element-wise.
So the result will be as long as the union of the two vectors plus their
number of values unique to one of them.
}
\examples{
data(mtcars)
cars <- rownames(mtcars)
carz <- cars[-grep("Merc", cars)]
cars <- cars[nchar(cars) < 15]
cars <- c(cars, "foobar")
compare_vectors(cars, carz)
}
\seealso{
Other searching functions: 
\code{\link{file_modified_last}()},
\code{\link{find_files}()},
\code{\link{fromto}()},
\code{\link{missing_docs}},
\code{\link{search_files}()},
\code{\link{search_rows}()},
\code{\link{summary.filesearch}()}

Other vector comparing functions: 
\code{\link{relative_difference}()}
}
\concept{searching functions}
\concept{vector comparing functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/batch.R
\name{get_script_path}
\alias{get_script_path}
\title{Get the Path of the R Code File}
\usage{
get_script_path()
}
\value{
A vector of \code{\link{length}} 1 and \code{\link{mode}}
character giving the name of the R code file if R was run via
\command{R CMD BATCH} or
\command{Rscript}.
}
\description{
This is just a wrapper for \code{\link{get_rscript_script_path}} and
\code{\link{get_r_cmd_batch_script_path}}.
}
\examples{
get_script_path()
}
\seealso{
Other script path getter functions: 
\code{\link{get_r_cmd_batch_script_path}()},
\code{\link{get_rscript_script_path}()},
\code{\link{get_script_name}()}
}
\concept{script path getter functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/view.R
\name{view}
\alias{view}
\title{View a File or Directory}
\usage{
view(path, program = NA)
}
\arguments{
\item{path}{A path to a file or directory.}

\item{program}{A program to use.}
}
\value{
\code{\link[base:invisible]{Invisibly}}
\code{\link[base:NULL]{NULL}}.
}
\description{
Call \code{shell.exec} on windows, mimic \code{shell.exec}
otherwise.
}
\examples{
path <- file.path(tempdir(), "foo.txt")
writeLines(c("abc", "xyz"), con = path)
view(path)
}
\seealso{
Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{vim}()},
\code{\link{wipe_tempdir}()},
\code{\link{with_dir}()}
}
\concept{operating system functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/installed.R
\name{is_installed}
\alias{is_installed}
\title{Is an External Program Installed?}
\usage{
is_installed(program)
}
\arguments{
\item{program}{Name of the program.}
}
\value{
\code{\link[base:TRUE]{TRUE}} on success, 
\code{\link[base:FALSE]{FALSE}} otherwise.
}
\description{
Is an external program installed?
}
\examples{
if (is_running_on_fvafrcu_machines() || is_running_on_gitlab_com()) {
    # NOTE: There are CRAN machines where neither "R" nor "R-devel" is in
    # the path, so we skipt this example on unkown machines.
    is_installed("R")
}
is_installed("probably_not_installed")
}
\seealso{
Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}

Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{vim}()},
\code{\link{wipe_tempdir}()},
\code{\link{with_dir}()}
}
\concept{logical helpers}
\concept{operating system functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/test_helpers.R
\name{get_boolean_envvar}
\alias{get_boolean_envvar}
\title{Get a Boolean Environment Variable}
\usage{
get_boolean_envvar(x, stop_on_failure = FALSE)
}
\arguments{
\item{x}{The name of the Environment Variable.}

\item{stop_on_failure}{Throw an error instead of returning
\code{\link{FALSE}} if the environment variable is not set or cannot be
converted to boolean.}
}
\value{
The value the environment variable is set to, converted to boolean.
\code{\link{FALSE}} if the environment variable is not set or cannot be
converted to boolean. But see \strong{Arguments}: \emph{stop_on_failure}.
}
\description{
A convenience wrapper to \code{\link{Sys.getenv}}.
}
\details{
As \code{\link{Sys.getenv}} seems to always return a character vector, the
\code{\link{class}} of the value you set it to does not matter.
}
\examples{
message("See\n example(\"get_run_r_tests\", package = \"fritools2\")")
}
\seealso{
Other test helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_cran}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{run_r_tests_for_known_hosts}()},
\code{\link{set_run_r_tests}()}

Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{vim}()},
\code{\link{wipe_tempdir}()},
\code{\link{with_dir}()}
}
\concept{operating system functions}
\concept{test helpers}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_path.R
\name{is_path}
\alias{is_path}
\title{Check Whether an Object Contains a Valid File System Path}
\usage{
is_path(x)
}
\arguments{
\item{x}{The object.}
}
\value{
\code{\link[base:TRUE]{TRUE}} on success, 
\code{\link[base:FALSE]{FALSE}} otherwise.
}
\description{
Check Whether an Object Contains a Valid File System Path
}
\examples{
is_path(tempdir())
path <- tempfile()
is_path(path)
touch(path)
is_path(path)
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/subsets.R
\name{index_groups}
\alias{index_groups}
\title{Determine Indices and Sizes of Subsets}
\usage{
index_groups(n, k)
}
\arguments{
\item{n}{The size of the set.}

\item{k}{The number of subsets.}
}
\value{
A matrix with starting index, size, and stopping index for each
subset.
}
\description{
Create starting and stopping indices for subsets defined by
\code{\link{subset_sizes}}.
}
\examples{
index_groups(n = 100, k = 6)
index_groups(n = 2, k = 6)
}
\seealso{
Other subsetting functions: 
\code{\link{subset_sizes}()}
}
\concept{subsetting functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/touch.R
\name{touch}
\alias{touch}
\alias{touch2}
\title{Mock the Unix \command{touch} Utility}
\usage{
touch(...)

touch2(...)
}
\arguments{
\item{...}{Paths to files.}
}
\value{
The Paths to the files touched.
}
\description{
Creating files or ensuring that their file modification times change.

\code{touch2} is an alternate - yet not faster - implementation.
}
\examples{
file1 <- tempfile()
file2 <- tempfile()
touch(file1, file2)
t1 <- file.mtime(file1, file2)
touch(file2)
t2 <- file.mtime(file1, file2)
t1 < t2
file <- file.path(tempfile(), "path", "not", "there.txt")
touch(file)
file.exists(file)
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()}
}
\concept{file utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/is_success.R
\name{is_success}
\alias{is_success}
\title{Does the Return Value of a Command Signal Success?}
\usage{
is_success(x)
}
\arguments{
\item{x}{The external commands return value.}
}
\value{
\code{\link[base:TRUE]{TRUE}} on success, 
\code{\link[base:FALSE]{FALSE}} otherwise.
}
\description{
This is just a wrapper to ease the evaluation of return values from external
commands:
External commands return 0 on success, which is
\code{\link[base:FALSE]{FALSE}}, when converted to logical.
}
\examples{
is_success(0)
is_success(1)
is_success(-1)
}
\seealso{
Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_force}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}

Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_copy}()},
\code{\link{file_save}()},
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{vim}()},
\code{\link{wipe_tempdir}()},
\code{\link{with_dir}()}
}
\concept{logical helpers}
\concept{operating system functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fritools2-package.R
\docType{package}
\name{fritools2-package}
\alias{fritools2-package}
\title{Utilities for the Forest Research Institute of the State
Baden-Wuerttemberg}
\description{
Miscellaneous utilities, tools and helper functions.
}
\details{
You will find the details in\cr
\code{vignette("An_Introduction_to_fritools", package = "fritools2")}.
}
\keyword{package}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/delete_trailing_blank_lines.R
\name{delete_trailing_blank_lines}
\alias{delete_trailing_blank_lines}
\title{Remove Trailing Blank Lines From Files}
\usage{
delete_trailing_blank_lines(...)
}
\arguments{
\item{...}{Arguments passed to \code{\link{find_files}}.}
}
\value{
\code{\link[base:invisible]{Invisibly}}
\code{\link[base:NULL]{NULL}}.
}
\description{
Trailing blank lines are classical lints.
}
\examples{
dir <- tempfile()
dir.create(dir)
file.copy(system.file("runit_tests", package = "fritools2"), dir,
          recursive = TRUE)
delete_trailing_blank_lines(path = dir, recursive = TRUE)
unlink(dir, recursive = TRUE)
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/file_save.R
\name{file_save}
\alias{file_save}
\title{Create a Copies of  Files}
\usage{
file_save(
  ...,
  file_extension_pattern = "\\\\.[A-z]{1,5}$",
  force = TRUE,
  recursive = NA,
  stop_on_error = TRUE,
  overwrite = FALSE
)
}
\arguments{
\item{...}{Paths to files.}

\item{file_extension_pattern}{A Pattern to mark a file extension. If matched,
the time stamp will get inserted before that pattern.}

\item{force}{Force even if \code{file_extension_pattern} is not matched. Set
to \code{\link{FALSE}} to skip stamping such files.}

\item{recursive}{Passed to \code{\link{file.copy}}. Defaults to `if the
current path is a directory, then TRUE, else FALSE`.}

\item{stop_on_error}{Throw an exception on error?}

\item{overwrite}{Passed to \code{\link{file.copy}}.}
}
\value{
A vector of  \code{\link[base:TRUE]{boolean}} values indicating
success or failure.
}
\description{
I often want a timestamped copies as backup of files or directories.
}
\examples{
f1 <- tempfile()
f2 <- tempfile()
try(file_save(f1))
touch(f1)
file_save(f1, recursive = FALSE)
f2 <- paste0(file.path(tempfile()), ".txt")
touch(f2)
file_save(f1, f2)
file_save(f1, f2)
file_save(f1, f2, overwrite = TRUE)
dir(tempdir())
}
\seealso{
Other operating system functions: 
\code{\link{clipboard_path}()},
\code{\link{file_copy}()},
\code{\link{get_boolean_envvar}()},
\code{\link{get_run_r_tests}()},
\code{\link{is_installed}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_success}()},
\code{\link{is_windows}()},
\code{\link{view}()},
\code{\link{vim}()},
\code{\link{wipe_tempdir}()},
\code{\link{with_dir}()}

Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}
\concept{operating system functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/options.R
\name{is_force}
\alias{is_force}
\title{Opt-out Via Option}
\usage{
is_force(x = .packages()[1])
}
\arguments{
\item{x}{The option under which an element \code{"force"} is to be searched
for.}
}
\value{
\code{\link[base:TRUE]{TRUE}} if option \code{x[["force"]]} is either
\code{\link[base:TRUE]{TRUE}} or \code{\link[base:NULL]{NULL}} (i.e. not set
at all).
}
\description{
Check whether or not a package option (set via \code{\link{set_options}})
\emph{force} is not set or set to \code{\link[base:TRUE]{TRUE}}.
}
\examples{
is_force()
set_options(list(force = FALSE))
get_options(flatten_list = FALSE)
is_force()
}
\seealso{
Other option functions: 
\code{\link{get_options}()},
\code{\link{set_options}()}

Other logical helpers: 
\code{\link{get_run_r_tests}()},
\code{\link{is_batch}()},
\code{\link{is_cran}()},
\code{\link{is_false}()},
\code{\link{is_installed}()},
\code{\link{is_not_false}()},
\code{\link{is_null_or_true}()},
\code{\link{is_of_length_zero}()},
\code{\link{is_r_cmd_check}()},
\code{\link{is_r_package_installed}()},
\code{\link{is_running_on_fvafrcu_machines}()},
\code{\link{is_running_on_gitlab_com}()},
\code{\link{is_success}()},
\code{\link{is_version_sufficient}()},
\code{\link{is_windows}()}
}
\concept{logical helpers}
\concept{option functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/split_code_file.R
\name{split_code_file}
\alias{split_code_file}
\title{Split a Code File Into Multiple Files}
\usage{
split_code_file(
  file,
  output_directory = tempdir(),
  encoding = getOption("encoding"),
  write_to_disk = getOption("write_to_disk")
)
}
\arguments{
\item{file}{The code file to be split.}

\item{output_directory}{Where to create the new files.}

\item{encoding}{The encoding passed to \code{\link{source}}.}

\item{write_to_disk}{Set the output_directory to
\code{dirname(file)}? Just a shortcut.}
}
\value{
\code{\link[base:invisible]{Invisibly}} a vector of paths to the new
files.
}
\description{
I tend to find files with dozens of functions. They don't read well.
So I split a code file into multiple files each containing a single function.
}
\examples{
infile <- system.file("files", "test_helpers.R", package = "fritools2")
\dontrun{
   file.show(infile)
}
paths <- split_code_file(file = infile)
\dontrun{
   file.show(paths[2])
}
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{search_files}()},
\code{\link{touch}()}
}
\concept{file utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/paths.R
\name{paths}
\alias{paths}
\alias{get_path}
\alias{set_path}
\title{Set or Get the \code{path} Attribute to or from an Object}
\usage{
get_path(x, force = FALSE)

set_path(x, path, action = c(NA, "read", "write"), overwrite = FALSE)
}
\arguments{
\item{x}{An object.}

\item{force}{Force the retrieval, even if the path is not valid? Only meant
for unit testing, leave alone!}

\item{path}{The path to be set.}

\item{action}{Do we have a read or write process? Passed by
\code{\link{read_csv}} and \code{\link{write_csv}}. Leave alone otherwise.}

\item{overwrite}{Overwrite an existing \emph{path} attribute instead of
throwing an error?}
}
\value{
For \code{get_path} the value of \code{attr(x, "path")}.

For \code{set_path} the modified object.
}
\description{
We set paths on some objects, these are convenience wrappers to
\code{\link{attr}}.
}
\examples{
x <- 2
path <- tempfile()
touch(path)
x <- set_path(x, path)
get_path(x)
}
\seealso{
Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{search_files}()},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/csv.R
\name{bulk_read_csv}
\alias{bulk_read_csv}
\title{Bulk Read Comma Separated Files}
\usage{
bulk_read_csv(
  paths,
  stop_on_error = FALSE,
  is_latin1 = TRUE,
  pattern = ".*\\\\.csv$",
  all_files = TRUE,
  recursive = FALSE,
  ignore_case = FALSE,
  find_all = FALSE,
  select = NA,
  ...
)
}
\arguments{
\item{paths}{A vector of file paths or the directory to find files.}

\item{stop_on_error}{Stop if any of the files is not read? Warn and continue
otherwise.}

\item{is_latin1}{Are the files encoded in "Latin1"?}

\item{pattern}{see \code{\link{find_files}}.
Ignored, if \code{paths} is not a directory.}

\item{all_files}{see \code{\link{find_files}}.
Ignored, if \code{paths} is not a directory.}

\item{recursive}{see \code{\link{find_files}}.
Ignored, if \code{paths} is not a directory.}

\item{ignore_case}{see \code{\link{find_files}}.
Ignored, if \code{paths} is not a directory.}

\item{find_all}{see \code{\link{find_files}}.
Ignored, if \code{paths} is not a directory.}

\item{select}{see \code{\link{find_files}}.
Ignored, if \code{paths} is not a directory.}

\item{...}{Arguments passed to \code{\link{read_csv}}.}
}
\value{
A named list, each element holding the contents of one \command{csv}
file read by \code{\link{read_csv}}.
}
\description{
Import a bunch of comma separated files or
all comma separated files below a directory using
\code{\link{read_csv}}.
}
\examples{
unlink(dir(tempdir(), full.names = TRUE))
data(mtcars)
mt_german <- mtcars
rownames(mt_german)[1] <- "Mazda R\u00f64"
names(mt_german)[1] <- "mg\u00dc"
#\% read from directory
for (i in 1:10) {
    f <- file.path(tempdir(), paste0("f", i, ".csv"))
    write.csv(mtcars[1:5, TRUE], file = f)
    f <- file.path(tempdir(), paste0("f", i, "_german.csv"))
    write.csv2(mt_german[1:7, TRUE], file = f, fileEncoding = "Latin1")
}
bulk <- bulk_read_csv(tempdir())

#\% pass a path
f <- list.files(tempdir(), pattern = ".*\\\\.csv$", full.names = TRUE)[1]
bulk <- bulk_read_csv(f)

#\% pass multiple path
f <- list.files(tempdir(), pattern = ".*\\\\.csv$", full.names = TRUE)[2:4]
bulk <- bulk_read_csv(f)
}
\seealso{
Other CSV functions: 
\code{\link{bulk_write_csv}()},
\code{\link{check_ascii_file}()},
\code{\link{csv2csv}()},
\code{\link{csv}}
}
\concept{CSV functions}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/search_files.R
\name{search_files}
\alias{search_files}
\title{Search Files for a Pattern}
\usage{
search_files(what, verbose = TRUE, exclude = NULL, ...)
}
\arguments{
\item{what}{A regex pattern for which to search.}

\item{verbose}{Be verbose?}

\item{exclude}{A regular expression for excluding files.}

\item{...}{Arguments passed to \code{\link{list.files}}.}
}
\value{
\link[base:invisible]{Invisibly} a vector of names of files
containing the pattern given by \code{what}.
}
\description{
This is an approximation of \kbd{unix} \command{find} and \command{grep}.
}
\examples{
write.csv(mtcars, file.path(tempdir(), "mtcars.csv"))
 for (i in 0:9) {
     write.csv(iris, file.path(tempdir(), paste0("iris", i, ".csv")))
 }
search_files(what = "Mazda", path = tempdir(), pattern = "^.*\\\\.csv$")
search_files(what = "[Ss]etosa", path = tempdir(), pattern = "^.*\\\\.csv$")
x <- search_files(path = tempdir(),
                  pattern = "^.*\\\\.csv$",
                  exclude = "[2-9]\\\\.csv$",
                  what = "[Ss]etosa")
summary(x)
summary(x, type = "what")
summary(x, type = "matches")
try(search_files(what = "ABC", path = tempdir(), pattern = "^.*\\\\.csv$"))
}
\seealso{
Other searching functions: 
\code{\link{compare_vectors}()},
\code{\link{file_modified_last}()},
\code{\link{find_files}()},
\code{\link{fromto}()},
\code{\link{missing_docs}},
\code{\link{search_rows}()},
\code{\link{summary.filesearch}()}

Other file utilities: 
\code{\link{clipboard_path}()},
\code{\link{delete_trailing_blank_lines}()},
\code{\link{delete_trailing_whitespace}()},
\code{\link{develop_test}()},
\code{\link{file_copy}()},
\code{\link{file_modified_last}()},
\code{\link{file_save}()},
\code{\link{find_files}()},
\code{\link{get_mtime}()},
\code{\link{get_unique_string}()},
\code{\link{is_files_current}()},
\code{\link{is_path}()},
\code{\link{paths}},
\code{\link{split_code_file}()},
\code{\link{touch}()}
}
\concept{file utilities}
\concept{searching functions}
